#!/usr/bin/perl -w

#use strict;

our($OUTFILE, $INFILE, $LOGFILE, %OPTIONS, @INFILES);

# variables for turbo formats
our($BASEWIDTH, $PILOTminW, $PILOTmaxW, $ZEROminW, $ZEROmaxW, $ONEminW, $ONEmaxW,
	    $SYNCminW, $SYNCmaxW, $ENDminW, $ENDmaxW,
	    %PWMS);

#Baud rate limits for FSK, like in Atari OS; bitlengths for samplerate 44100
our(@BITLEN_INTERVALS, $bitlen_index);
@BITLEN_INTERVALS=(32,36,41,45,50,54,59,63,68,72,77,81,86,90,95,100,104,109,113,118,122,127,131,136,140);
$bitlen_index=10; #standard speeds

use Getopt::Long qw(GetOptionsFromArray :config auto_version no_auto_help);
use IO::Handle;
use Pod::Usage;
use Audio::Ecasound qw(:simple);
use POSIX qw(mkfifo);

use constant {
    TRUE => 1,
    FALSE => 0,
    OK => 1,
    NOT_GOOD => -4,
    END_OF_FILE => -1,
    };

use constant {
    ZEROBIT => chr(0),
    ONEBIT => chr(255),
    NEUTRAL => chr(128),
    };

use constant {
    MARK => ONEBIT,
    SPACE => ZEROBIT,
    SILENCE => NEUTRAL,
    START => ZEROBIT,	#start bit is 0
    STOP => ONEBIT,	#stop bit is 1
    LEVELZERO => ZEROBIT,
    LEVELONE => ONEBIT,
    ZERO => ZEROBIT,
    };

use constant {
      HALFBUFFLEN => 4*4096,
      HALFBUFFLEN16 => 4*8192,
    };

our(%DEBUG, $TURBO_FORMATS_REGEX, $CAS_BLOCKS_REGEX);

$TURBO_FORMATS_REGEX = '(^turbo2000$)|(^blizzard$)|(^ast$)|(^att$)|(^um$)|(^turborom$)|(^6000$)|(^mkturbo$)|(^cs2000$)|(^csst$)|(^cstt$)|(^csbtape$)|(^hardturbo$)|(^autoturbo$)|(^tapewiz$)|(^lowsil2000)';
$CAS_BLOCKS_REGEX = '(^FUJI$)|(^data$)|(^baud$)|(^fsk\ $)|(^pwms$)|(^pwml$)|(^pwmc$)|(^pwmd$)';

our %DATASTORE;	# anything can be here

$main::VERSION = 1.06;

#subroutines

  sub findofile {
    # Find wheteher file exists and if yes
    # generate a new filename to avoid overwriting it
    # Argument: filename

    die ("Internal error in findofile") if $_[0] eq "";

    my $filename=shift;
    (my $fname_ext=$filename)=~s/^(.*)\.(....?)$/$2/;
    (my $fname_base=$filename)=~s/^(.*)\.(....?)$/$1/;

    my $i=1;
    while (-e $filename) {
      $filename=$fname_base . "_" . $i++ . "." . $fname_ext;
    }

    return($filename);
  }

  sub prepare_blizzard {

  # This is for Blizzard
  # All widths for samplerate 96000

    $BASEWIDTH = ($_[0] or 48); # 50

    $PILOTminW = 29; #$BASEWIDTH-10;
    $PILOTmaxW = $BASEWIDTH+10;
    $ZEROminW = 10; #arbitrary, was: int($BASEWIDTH/3+0.5)-5;
    $ZEROmaxW = int(95*$BASEWIDTH*0.0104166666667/(0.000563873631408*114*2)/1.773447/1000/0.0104166666667); #based on loader code ; was: int($BASEWIDTH/3+0.5)+4;
    $ONEminW = $ZEROmaxW+1;
    $ONEmaxW = $PILOTmaxW;
    $ENDminW = 0;
    $ENDmaxW = 0;
    $SYNCminW = $ZEROminW;
    $SYNCmaxW = 28; #26
    $PWMS{'pilotmincycles'} = 100;
    $PWMS{'maxpilotjitter'} = 1;
    $PWMS{'autoedgedetect'} = TRUE;
    $PWMS{'iparams'} = ($OPTIONS{'invert'} ? 0b00000110 : 0b00000101);
    $PWMS{'countlevels'} = 0b11;
    $PWMS{'firstbits'} = 2;
    $PWMS{'endian'}=($PWMS{'iparams'} & 0b100) ? "B8" : "b8";
    $PWMS{'PILOTW'} = $BASEWIDTH;
    $PWMS{'ZEROW'} = int($BASEWIDTH/3+1.5);
    $PWMS{'ONEW'} = int($BASEWIDTH/2+0.5);
#    $PWMS{'ZEROW'} = int($BASEWIDTH/3.2+0.5);
#    $PWMS{'ONEW'} = int($BASEWIDTH/1.85+0.5);
    $PWMS{'ENDW'} = 0;
    $PWMS{'ENDIMP'} = 0;
    $PWMS{'ahead'} = 11;
    # frequency of end impulses
    #$PWMS{'highpass'} = int(96000/$PWMS{'ONEW'}+0.5);
    # frequency of narrowest impulses
    $PWMS{'lowpass'} = int(96000/$PWMS{'ZEROW'}+0.5);
    $PWMS{'highpass'} = 1500;
    $PWMS{'highreso'} = 1;
    $PWMS{'lowreso'} = 1;
    $PWMS{'lowgain'} = 1;
    #$PWMS{'lowpass'} = 5760;
    $OPTIONS{'cksum'} = ($OPTIONS{'cksumtype'} or 'modulo256');
    $OPTIONS{'cksumsane'} = TRUE;
    $OPTIONS{'cksumfrom'} = 1;
    $OPTIONS{'cksumto'} = -2;
    $OPTIONS{'cksumpos'} = -1;
    1;
  };

  sub prepare_generic {

  #This is for generic turbo
    $BASEWIDTH = ($_[0] or 49); # 49
    prepare_blizzard($BASEWIDTH);
    $OPTIONS{'cksumsane'} = FALSE;
    1;
  };

  sub prepare_turbo2000 {

  #This is for Turbo 2000 KSO

    if ($DATASTORE{'t2000-nf'}) {
	# it is continuation of long datablock, don't recalculate values
	$PWMS{'pilotmincycles'} = 2;
	$PWMS{'autoedgedetect'} = FALSE;
	return 0;
    } else {
	$PWMS{'pilotmincycles'} = 100;
	$PWMS{'autoedgedetect'} = TRUE;
    }
    $BASEWIDTH = ($_[0] or 48); # 96
    $PILOTminW = 37; #int($BASEWIDTH*0.8298+0.5); #34?
    $PILOTmaxW = 76;
    $ZEROminW = 5; # arbitrary
    $ZEROmaxW = 16; #int($BASEWIDTH/4*1.6667+0.5); #16?
    $ONEminW = $ZEROmaxW+1;
    $ONEmaxW = 36; #$PILOTminW-1; #33?
    $ENDminW = 0;
    $ENDmaxW = 0;
    $SYNCminW=$ZEROminW;
    $SYNCmaxW=$ONEmaxW;
    # bits in iparams  (order: 76543210):
    # bits 1 and 0: 01,10 - order of levels in bit
    # bit 3 - endian
    $PWMS{'maxpilotjitter'} = 4;
    $PWMS{'iparams'} = ($OPTIONS{'invert'} ? 0b00000101 : 0b00000110) ;
    $PWMS{'countlevels'} = 0b01;
    $PWMS{'firstbits'} = 0;
    $PWMS{'endian'}=($PWMS{'iparams'} & 0b100) ? "B8" : "b8";
    # $PWMS{'PILOTW'} = $BASEWIDTH;
    # $PWMS{'ZEROW'} = int($BASEWIDTH/4+0.5);
    # $PWMS{'ONEW'} = int($BASEWIDTH/2+0.5);
    $PWMS{'PILOTW'} = $BASEWIDTH*2;
    $PWMS{'ZEROW'} = int($PWMS{'PILOTW'}/4+0.5);
    $PWMS{'ONEW'} = int($PWMS{'PILOTW'}/2+0.5);
    $PWMS{'ENDW'} = 0;
    $PWMS{'ENDIMP'} = $BASEWIDTH*4;
    $PWMS{'ahead'} = 20;
    # frequency of end impulses
    #$PWMS{'highpass'} = int(96000/$PWMS{'ONEW'}+0.5);
    # frequency of narrowest impulses
    $PWMS{'lowpass'} = int(48000/$PWMS{'ZEROW'}+0.5);
    $PWMS{'highpass'} = 1000;
    $PWMS{'highreso'} = 1;
    $PWMS{'lowreso'} = 2;
    $PWMS{'lowgain'} = 2;
    #$PWMS{'lowpass'} = 4000;
    $OPTIONS{'cksum'} = ($OPTIONS{'cksumtype'} or 'modulo256');
    $OPTIONS{'cksumsane'} = TRUE;
    $OPTIONS{'cksumfrom'} = 1;
    $OPTIONS{'cksumto'} = -2;
    $OPTIONS{'cksumpos'} = -1;
    #$OPTIONS{'amp'}=3;
    1;
  };

  sub prepare_ast {

  #This is for AST

    $BASEWIDTH = ($_[0] or 47); # 34-59
    $ZEROminW = 9;
    $ZEROmaxW = 33;
    $PILOTminW = 34;
    $PILOTmaxW = 59;
    $ONEminW = $PILOTminW;
    $ONEmaxW = $PILOTmaxW;
    $ENDminW = 60;
    $ENDmaxW = 180;
    $SYNCminW=$ZEROminW;
    $SYNCmaxW=$ZEROmaxW;
    $PWMS{'pilotmincycles'} = 256;
    $PWMS{'maxpilotjitter'} = 4;
    $PWMS{'autoedgedetect'} = TRUE;
    $PWMS{'iparams'} = ($OPTIONS{'invert'} ? 0b00000001 : 0b00000010);
    $PWMS{'countlevels'} = 0b11;
    $PWMS{'firstbits'} = 1;
    $PWMS{'endian'}=($PWMS{'iparams'} & 0b100) ? "B8" : "b8";
    $PWMS{'PILOTW'} = $BASEWIDTH;
    $PWMS{'ZEROW'} = int($BASEWIDTH/2+0.5);
    $PWMS{'ONEW'} = $BASEWIDTH;
    $PWMS{'ENDW'} = $BASEWIDTH*3;
    $PWMS{'ENDIMP'} = 0;
    $PWMS{'ahead'} = 11;
    # frequency of end impulses
    $PWMS{'highpass'} = int(96000/(($BASEWIDTH+2)*3)+0.5);
    # resonance of highpass <0,1.41>
    $PWMS{'highreso'} = 1.41;
    # frequency of narrowest impulses
    $PWMS{'lowpass'} = int(96000/(($BASEWIDTH+2)/2)+0.5);
    $PWMS{'lowreso'} = 2;
    $PWMS{'lowgain'} = 2;
    $OPTIONS{'cksum'} = ($OPTIONS{'cksumtype'} or 'xor');
    $OPTIONS{'cksumsane'} = TRUE;
    $OPTIONS{'cksumfrom'} = 1;
    $OPTIONS{'cksumto'} = -1;
    $OPTIONS{'cksumpos'} = 0;
    1;
  };

  sub prepare_um {

  #This is for UM

    $BASEWIDTH = ($_[0] or 49); # 49
    prepare_ast($BASEWIDTH);

    $ZEROminW = int($BASEWIDTH*0.5+0.5)-12;
    $ZEROmaxW = int($BASEWIDTH*0.5+0.5)+15;
    $PILOTminW = $ZEROmaxW+1;
    $PILOTmaxW = int($BASEWIDTH*1.75);
    $ONEminW = $PILOTminW;
    $ONEmaxW = $PILOTmaxW;
    $ENDminW = int($BASEWIDTH*1.75)+1;
    $ENDmaxW = $BASEWIDTH*3+12;
    $SYNCminW=$ZEROminW;
    $SYNCmaxW=$ZEROmaxW;
    $PWMS{'iparams'} = ($OPTIONS{'invert'} ? 0b00000001 : 0b00000010);
    $PWMS{'countlevels'} = 0b11;
    $PWMS{'pilotmincycles'} = 1500;
    $OPTIONS{'cksumsane'} = TRUE;
    $OPTIONS{'cksumfrom'} = 1;
    $OPTIONS{'cksumto'} = -2;
    $OPTIONS{'cksumpos'} = -1;
    1;
  };

  sub prepare_att {

  #This is for ATT
    $BASEWIDTH = ($_[0] or 49); # 49
    prepare_ast($BASEWIDTH);

    $PWMS{'pilotmincycles'} = 1500;
    $OPTIONS{'cksumsane'} = FALSE;
    1;
  };

  sub prepare_cs2000 {

  #This is for Czechoslovakian Turbo 2000
    $BASEWIDTH = ($_[0] or 74); # 52-96

    if ($_[0]) {
      $PWMS{'maxpilotjitter'} = 4;
    } else {
      $PWMS{'maxpilotjitter'} = 22;
    }
    $PILOTminW = 52;
    $PILOTmaxW = 96;
    $ZEROminW = 12;	# arbitrary
    $ZEROmaxW = 38;
    $ONEminW = $ZEROmaxW+1;
    $ONEmaxW = 74;
    $ENDminW = 0;
    $ENDmaxW = 0;
    $SYNCminW = $ZEROminW;
    $SYNCmaxW = 37;
    $PWMS{'pilotmincycles'} = 1500;
    $PWMS{'autoedgedetect'} = TRUE;
    $PWMS{'iparams'} = ($OPTIONS{'invert'} ? 0b00000110 : 0b00000101);
    $PWMS{'countlevels'} = 0b11;
    $PWMS{'firstbits'} = 1;
    $PWMS{'endian'}=($PWMS{'iparams'} & 0b100) ? "B8" : "b8";
    $PWMS{'PILOTW'} = $BASEWIDTH;
    $PWMS{'ZEROW'} = int(0.40625*$BASEWIDTH+0.5);
    $PWMS{'ONEW'} = int(0.8125*$BASEWIDTH+0.5);
    $PWMS{'ENDW'} = 0;
    $PWMS{'ENDIMP'} = 4*$BASEWIDTH;
    $PWMS{'ahead'} = 30;
    $OPTIONS{'cksum'} = ($OPTIONS{'cksumtype'} or 'xor');
    $OPTIONS{'cksumsane'} = TRUE;
    $OPTIONS{'cksumfrom'} = 1;
    $OPTIONS{'cksumto'} = -2;
    $OPTIONS{'cksumpos'} = -1;
    $PWMS{'lowpass'} = int(96000/$PWMS{'ZEROW'}+0.5);
    $PWMS{'highpass'} = 1000;
    $PWMS{'highreso'} = 1;
    $PWMS{'lowreso'} = 2;
    $PWMS{'lowgain'} = 2;
    1;
  };

  sub prepare_csst {

  #This is for Czechoslowakian Super Turbo
    $BASEWIDTH = ($_[0] or 31); # 13-48

    if ($_[0]) {
      $PWMS{'maxpilotjitter'} = 4;
    } else {
      $PWMS{'maxpilotjitter'} = 17;
    }

    $ZEROminW = int($BASEWIDTH/2+0.5)-5;
    $ZEROmaxW = int($BASEWIDTH*0.75+0.5);
    $PILOTminW = $ZEROmaxW+1;
    $PILOTmaxW = $BASEWIDTH+11;
    $ONEminW = $PILOTminW;
    $ONEmaxW = $PILOTmaxW;
    $ENDminW = $BASEWIDTH*6-20;
    $ENDmaxW = $BASEWIDTH*6+20;
    $SYNCminW=$ZEROminW;
    $SYNCmaxW=$ZEROmaxW;
    $PWMS{'pilotmincycles'} = 1500;
    $PWMS{'autoedgedetect'} = TRUE;
    $PWMS{'iparams'} = ($OPTIONS{'invert'} ? 0b00000101 : 0b00000110);
    $PWMS{'countlevels'} = 0b11;
    $PWMS{'firstbits'} = 1;
    $PWMS{'endian'}=($PWMS{'iparams'} & 0b100) ? "B8" : "b8";
    $PWMS{'PILOTW'} = $BASEWIDTH;
    $PWMS{'ZEROW'} = int($BASEWIDTH/2+0.5);
    $PWMS{'ONEW'} = $BASEWIDTH;
    $PWMS{'ENDW'} = $BASEWIDTH*3;
    $PWMS{'ENDIMP'} = $BASEWIDTH*6;
    $PWMS{'ahead'} = 30;
    $OPTIONS{'cksum'} = ($OPTIONS{'cksumtype'} or 'xor');
    $OPTIONS{'cksumsane'} = TRUE;
    $OPTIONS{'cksumfrom'} = 1;
    $OPTIONS{'cksumto'} = -2;
    $OPTIONS{'cksumpos'} = -1;
    $PWMS{'lowpass'} = int(96000/$PWMS{'ZEROW'}+0.5);
    $PWMS{'highpass'} = 1000;
    $PWMS{'highreso'} = 1;
    $PWMS{'lowreso'} = 2;
    $PWMS{'lowgain'} = 2;
    1;
  };

  sub prepare_cstt {

  #This is for Czechoslowakian Turbo Tape
    prepare_csst($_[0]);
  }

  sub prepare_csbtape {

  #This is for Czechoslowakian Turbo Tape
    prepare_cs2000($_[0]);

    if ($_[0]) {
      $PWMS{'maxpilotjitter'} = 4;
    } else {
      $PWMS{'maxpilotjitter'} = 40;
    }
    $PWMS{'iparams'} = ($OPTIONS{'invert'} ? 0b00000101 : 0b00000110);
    $PWMS{'endian'}=($PWMS{'iparams'} & 0b100) ? "B8" : "b8";
    #$ZEROminW = int($BASEWIDTH/2+0.5)-7;
    #$ZEROmaxW = int($BASEWIDTH/2+0.5)+6;
    #$ONEminW = $ZEROmaxW+1;
    #$ONEmaxW = $PILOTmaxW;
    1;
  }

  sub prepare_lowsil2000 {

  #This is for Low Silesia Turbo 2000
    prepare_cs2000($_[0]);
    $PWMS{'pilotmincycles'} = 500;
  }

  sub prepare_turborom {
    #7,16
    # there are no adjustments in original code handling turbo rom
    $BASEWIDTH = ($_[0] or 19); # 16
    $PILOTminW = 11;#int(10*$BASEWIDTH/16+0.5);
    $PILOTmaxW = 25;#int(25*$BASEWIDTH/16+0.5);
    $ZEROminW = 4;
    $ZEROmaxW = 10;#int(9*$BASEWIDTH/16+0.5);
    $ONEminW = $ZEROmaxW+1;
    $ONEmaxW  = 25;#int(25*$BASEWIDTH/16+0.5);
    $ENDminW = 0;
    $ENDmaxW = 0;
    $SYNCminW=$ZEROminW;
    $SYNCmaxW=$ZEROmaxW;
    $PWMS{'pilotmincycles'} = 500;
    $PWMS{'maxpilotjitter'} = 1;
    $PWMS{'autoedgedetect'} = FALSE;
    # only one state is counted (low - "1" on data-in)
    $PWMS{'iparams'} = ($OPTIONS{'invert'} ? 0b00000001 : 0b00000010);
    $PWMS{'countlevels'} = 0b01;
    $PWMS{'firstbits'} = 1;
    $PWMS{'endian'}=($PWMS{'iparams'} & 0b100) ? "B8" : "b8";
    # The following consist of 2 halves, ulike the above params
    $PWMS{'PILOTW'} = 36;
    $PWMS{'ZEROW'} = 14;
    $PWMS{'ONEW'} = 36;
    $PWMS{'ENDW'} = 0;
    $PWMS{'ENDIMP'} = $PWMS{'PILOTW'}*2;
    $PWMS{'ahead'} = 15;
    # frequency of end impulses
    #$PWMS{'highpass'} = int(96000/$PWMS{'PILOTW'}+0.5);
    # frequency of narrowest impulses
    #$PWMS{'lowpass'} = int(96000/($PWMS{'ZEROW'})+0.5);
    $PWMS{'highpass'} = 1000;
    $PWMS{'highreso'} = 1;
    $PWMS{'lowpass'} = 6900;
    $PWMS{'lowreso'} = 2;
    $PWMS{'lowgain'} = 2;
    $OPTIONS{'cksum'} = ($OPTIONS{'cksumtype'} or 'xor');
    $OPTIONS{'cksumsane'} = TRUE;
    $OPTIONS{'cksumfrom'} = 2;
    $OPTIONS{'cksumto'} = -1;
    $OPTIONS{'cksumpos'} = 1;
    1;
  };

  sub prepare_6000 {
    #7,16
    $BASEWIDTH = ($_[0] or 32); # 16
    $PILOTminW = 0; #NO traditional pilot
    $PILOTmaxW = 0; #NO traditional pilot;
    $ZEROminW = 16; #int($BASEWIDTH*0.8125/2+0.5)-11;
    $ZEROmaxW = 24; #int($BASEWIDTH*0.8125/2+0.5)+8;
    $ONEminW = 25; #int($BASEWIDTH*0.8125+0.5)-7;
    $ONEmaxW  = 36; #int($BASEWIDTH*0.8125+0.5)+4;
    $ENDminW = 0;
    $ENDmaxW = 0;
    $SYNCminW=$ZEROminW;
    $SYNCmaxW=$ZEROmaxW;
    $PWMS{'pilotmincycles'} = 0; #NO traditional pilot
    $PWMS{'maxpilotjitter'} = 0; #NO traditional pilot
    $PWMS{'autoedgedetect'} = FALSE;
    # only one state is counted (low - "1" on data-in)
    $PWMS{'iparams'} = ($OPTIONS{'invert'} ? 0b00000110 : 0b00000101);
    $PWMS{'countlevels'} = 0b11;
    $PWMS{'firstbits'} = 0;
    $PWMS{'endian'}=($PWMS{'iparams'} & 0b100) ? "B8" : "b8";
    $PWMS{'PILOTW'} = 0;
    $PWMS{'ZEROW'} = 20;
    $PWMS{'ONEW'} = 32;
    $PWMS{'ENDW'} = 0;
    $PWMS{'ENDIMP'} = $PWMS{'ONEW'}*4;
    $PWMS{'ahead'} = 11;
    # frequency of end impulses
    #$PWMS{'highpass'} = int(96000/$PWMS{'PILOTW'}+0.5);
    # frequency of narrowest impulses
    #$PWMS{'lowpass'} = int(96000/($PWMS{'ZEROW'})+0.5);
    $PWMS{'lowpass'} = int(96000/$PWMS{'ZEROW'}+0.5);
    $PWMS{'highpass'} = 1500;
    $PWMS{'highreso'} = 1;
    $PWMS{'lowreso'} = 1;
    $PWMS{'lowgain'} = 1;
    $OPTIONS{'cksum'} = ($OPTIONS{'cksumtype'} or 'xor');
    $OPTIONS{'cksumsane'} = TRUE;
    $OPTIONS{'cksumfrom'} = 1;
    $OPTIONS{'cksumto'} = -2;
    $OPTIONS{'cksumpos'} = -1;
    1;
  };

  sub prepare_mkturbo {

  #This is for MK.Turbo

    # there are no adjustments in original code handling mk.turbo
    $BASEWIDTH = ($_[0] or 12);

    if ($_[0]) {
      $PWMS{'maxpilotjitter'} = 4;
    } else {
      $PWMS{'maxpilotjitter'} = 22;
    }

    $ZEROminW = $BASEWIDTH-int($BASEWIDTH/4+0.5); #arbitrary
    $ZEROmaxW = $BASEWIDTH+int($BASEWIDTH/4+0.5); #arbitrary
    $ONEminW = $ZEROmaxW+1;
    $ONEmaxW = $ONEminW+22;
    $PILOTminW = $ZEROminW;
    $PILOTmaxW = $ZEROmaxW;
    $ENDminW = 0;
    $ENDmaxW = 0;
    $SYNCminW=$ONEminW;
    $SYNCmaxW=$ONEmaxW;
    $PWMS{'pilotmincycles'} = 2;
    $PWMS{'autoedgedetect'} = FALSE;
    $PWMS{'iparams'} = ($OPTIONS{'invert'} ? 0b00000010 : 0b00000001);
    $PWMS{'countlevels'} = 0b11;
    $PWMS{'firstbits'} = 1;
    $PWMS{'endian'}=($PWMS{'iparams'} & 0b100) ? "B8" : "b8";
    $PWMS{'ZEROW'} = int($BASEWIDTH*0.6+0.5);
    $PWMS{'PILOTW'} = $PWMS{'ZEROW'};
    $PWMS{'ONEW'} = $BASEWIDTH;
    $PWMS{'ENDW'} = 90;
    $PWMS{'ENDIMP'} = 400;
    $PWMS{'ahead'} = $BASEWIDTH;
    # frequency of end impulses
    #$PWMS{'highpass'} = int(96000/$PWMS{'ONEW'}+0.5);
    # frequency of narrowest impulses
    $PWMS{'lowpass'} = int(96000/$PWMS{'ZEROW'}+0.5);
    $PWMS{'lowreso'} = 2;
    $PWMS{'lowgain'} = 2;
    $PWMS{'highpass'} = 530;
    $PWMS{'highreso'} = 1;
    #$PWMS{'lowpass'} = 5760;
    $OPTIONS{'cksum'} = ($OPTIONS{'cksumtype'} or 'sum255carry');
    $OPTIONS{'cksumsane'} = TRUE;
    $OPTIONS{'cksumfrom'} = 2;
    $OPTIONS{'cksumto'} = -3;
    $OPTIONS{'cksumpos'} = -2;
    1;
  };

  sub prepare_hardturbo {

  #This is for Czechoslovakian Turbo 2000
    $BASEWIDTH = ($_[0] or 74); # 49-98

    if ($_[0]) {
      $PWMS{'maxpilotjitter'} = 4;
    } else {
      $PWMS{'maxpilotjitter'} = 28;
    }
    $PILOTminW = 49;
    $PILOTmaxW = 98;
    $ZEROminW = 10; #arbitrary
    $ZEROmaxW = 36;
    $ONEminW = $ZEROmaxW+1;
    $ONEmaxW = 70;
    $ENDminW = 0;
    $ENDmaxW = 0;
    $SYNCminW = $ZEROminW;
    $SYNCmaxW = 31;
    $PWMS{'pilotmincycles'} = 256;
    $PWMS{'autoedgedetect'} = TRUE;
    $PWMS{'iparams'} = ($OPTIONS{'invert'} ? 0b00000101 : 0b00000110);
    $PWMS{'countlevels'} = 0b11;
    $PWMS{'firstbits'} = 1;
    $PWMS{'endian'}=($PWMS{'iparams'} & 0b100) ? "B8" : "b8";
    $PWMS{'PILOTW'} = $BASEWIDTH;
    $PWMS{'ZEROW'} = 23;
    $PWMS{'ONEW'} = 53;
    $PWMS{'ENDW'} = 0;
    $PWMS{'ENDIMP'} = 0;
    $PWMS{'ahead'} = 30;
    $OPTIONS{'cksum'} = ($OPTIONS{'cksumtype'} or 'xor');
    $OPTIONS{'cksumsane'} = TRUE;
    $OPTIONS{'cksumfrom'} = 1;
    $OPTIONS{'cksumto'} = -2;
    $OPTIONS{'cksumpos'} = -1;
    $PWMS{'lowpass'} = int(96000/$PWMS{'ZEROW'}+0.5);
    $PWMS{'highpass'} = 1000;
    $PWMS{'highreso'} = 1;
    $PWMS{'lowreso'} = 2;
    $PWMS{'lowgain'} = 2;
    1;
  };

  sub prepare_autoturbo {

  #This is for Czechoslowakian Turbo Tape
    prepare_cs2000($_[0]);

    $PWMS{'firstbits'} = 2 if defined($DATASTORE{'autoturbofirst'}) and $DATASTORE{'autoturbofirst'}==2;
    1;
  }

  sub prepare_tapewiz {
    #7,16
    $BASEWIDTH = ($_[0] or 40); # 20
    $PILOTminW = 34;
    $PILOTmaxW = 50;
    $ZEROminW = 16;
    $ZEROmaxW = 33;
    $ONEminW = 34;
    $ONEmaxW  = 50;
    $ENDminW = 0;
    $ENDmaxW = 0;
    $SYNCminW=$ZEROminW;
    $SYNCmaxW=$ZEROmaxW;
    $PWMS{'pilotmincycles'} = 10;
    $PWMS{'maxpilotjitter'} = 1;
    $PWMS{'autoedgedetect'} = TRUE;
    $PWMS{'iparams'} = ($OPTIONS{'invert'} ? 0b00000110 : 0b00000101);
    $PWMS{'countlevels'} = 0b11;
    $PWMS{'firstbits'} = 0;
    $PWMS{'endian'}=($PWMS{'iparams'} & 0b100) ? "B8" : "b8";
    $PWMS{'PILOTW'} = 40;
    $PWMS{'ZEROW'} = 24;
    $PWMS{'ONEW'} = 40;
    $PWMS{'ENDW'} = 0;
    $PWMS{'ENDIMP'} = $PWMS{'ONEW'}*4;
    $PWMS{'ahead'} = 11;
    $PWMS{'lowpass'} = int(96000/$PWMS{'ZEROW'}+0.5);
    $PWMS{'highpass'} = 1500;
    $PWMS{'highreso'} = 1;
    $PWMS{'lowreso'} = 1;
    $PWMS{'lowgain'} = 1;
    $OPTIONS{'cksum'} = ($OPTIONS{'cksumtype'} or 'xor');
    $OPTIONS{'cksumsane'} = FALSE;
    $OPTIONS{'cksumfrom'} = 1;
    $OPTIONS{'cksumto'} = -2;
    $OPTIONS{'cksumpos'} = -1;
    1;
  };
sub controlsum {

# Compute checksum of data block
# Arguments:
#  - the string containing data
# Returns (context sensitive):
#  array context:
#   - computed checksum of given data
#   - expected checksum of given data
#   - verify status
#  scalar context:
#   - computed checksum

  my %cksumtypes=('sum255carry' => 255,
		  'sum255alt' => 255,
		  'modulo256' => 256,
		  'xor' => undef,
		  'unknown' => undef,
		  );
  my $sum=0;
  my @record=split('',$_[0]);
  my $max=$cksumtypes{$OPTIONS{'cksum'}};

  my $from=$OPTIONS{'cksumfrom'}-1;
  my $to=$#record+$OPTIONS{'cksumto'}+1;

  if ($OPTIONS{'cksum'} eq 'xor')  {
    foreach (@record[$from..$to]) {
      $sum=$sum ^ unpack('C',$_);
    }
  }

  if ($max) {
    foreach (@record[$from..$to]) {
      $sum+=unpack('C',$_);
      $sum-=$max if $sum>$max;
    }
    $sum-=256 if $sum>=256;
  }

  if (wantarray()) {
    if ($OPTIONS{'cksumpos'}==0) {
      return ($sum,							# computed checksum
	      $sum,							# expected checksum
	      TRUE							# checksum verification
	     );
    } elsif ($OPTIONS{'cksum'} eq 'sum255alt') {
      $sum+=ord($record[$#record+$OPTIONS{'cksumpos'}+1]);
      return ($sum,							# computed checksum
	      255,							# expected checksum
	      ($sum==255)						# checksum verification
	     );
    } else {
      return ($sum,							# computed checksum
	      ord($record[($OPTIONS{'cksumpos'}<0 ? $#record : -2)+$OPTIONS{'cksumpos'}+1]),		# expected checksum
	      ($sum==ord($record[($OPTIONS{'cksumpos'}<0 ? $#record : -2)+$OPTIONS{'cksumpos'}+1]))	# checksum verification
	     );
    }
  } else  {
    return $sum;
  }
}

sub infoblock_blizzard {
# Arguments:
# - a reference to cas hash
# - cas block number (block_no, int)
# - program title (string)

  my $cas=$_[0];
  my $block=$_[1];
  my $title=$_[2];

  chomp($title);
  cas_put($cas,$block++,'FUJI',length($title),pack('v',0),$title);
  if ($block==1) {
    cas_put($cas,$block++,'pwms',2,pack('v',$PWMS{'iparams'}),pack('v',$OPTIONS{'samplerate'}));
  }
  $title=sprintf("%-78s",$title);
  my $cksum=chr(controlsum($title));
  $title=substr($title,0,-2);
  cas_put($cas,$block,'pwmc',6,pack('v',2000),
	  pack('C',int($PWMS{'PILOTW'}*$OPTIONS{'samplerate'}/96000+0.5))
	  . pack('v',3072)
	  . pack('C',int($PWMS{'ZEROW'}*$OPTIONS{'samplerate'}/96000+0.5))
	  . pack('v',2));
  $block++;
  cas_put($cas,$block,'pwmc',6,pack('v',200),
	  pack('C',int($PWMS{'PILOTW'}*$OPTIONS{'samplerate'}/96000+0.5))
	  . pack('v',302)
	  . pack('C',int($PWMS{'ZEROW'}*$OPTIONS{'samplerate'}/96000+0.5))
	  . pack('v',2));
  $block++;
  prepare_blizzard();
  cas_put($cas,$block++,'pwmd',78,
	  pack('C',int($PWMS{'ZEROW'}*$OPTIONS{'samplerate'}/96000+0.5))
	  . pack('C',int($PWMS{'ONEW'}*$OPTIONS{'samplerate'}/96000+0.5)),
	  $title . $cksum . chr(0));
  $_[1]=$block;
  1;
}

sub datablock_blizzard {
# Arguments:
# - reference to cas hash
# - cas block number (block_no, int)
# - data (string)
# - number of init calls ($02e2) (int)

  my $cas=$_[0];
  my $block=$_[1];
  my $data=$_[2];
  my $inits=($_[3] or 0);

  my $datalen=length($data);
  $data=pack('va1027',$datalen,$data);	#add length to the beginning and zeros to the end
  my $cksum=chr(controlsum($data));
  $data=substr($data,0,-2);
  $inits+=5 if ${$cas}{$block-1}{'length'}<100;
  cas_put($cas,$block,'pwmc',6,pack('v',200+($inits>0 ? 400 : 0)+400*$inits),
	  pack('C',int($PWMS{'PILOTW'}*$OPTIONS{'samplerate'}/96000+0.5))
	  . pack('v',302)
	  . pack('C',int($PWMS{'ZEROW'}*$OPTIONS{'samplerate'}/96000+0.5))
	  . pack('v',2));
  $block++;
  prepare_blizzard();
  cas_put($cas,$block++,'pwmd',length($data)+2,
	  pack('C',int($PWMS{'ZEROW'}*$OPTIONS{'samplerate'}/96000+0.5))
	  . pack('C',int($PWMS{'ONEW'}*$OPTIONS{'samplerate'}/96000+0.5)),
	  $data . $cksum . chr(0));
  $_[1]=$block;
  1;
}


sub analyse_blizzard {
# Arguments:
#  - type of analysis (string):
#     cksum: checksum adjustments
#     first: firstbits adjustments
#  - data to analyse (string)

  my $analyse=$_[0];
  CASE: {
    if ($analyse eq 'cksum') {
    # argument 1 is record to analyse
      my $record=$_[1];
      my $lenadjust=0;
      my $reclen=length($record);
      $reclen=0 if !defined $record;

      $lenadjust=$reclen-77 if $reclen-77>0 and $reclen-77<4;
      $lenadjust=$reclen-1028 if $reclen-1028>0 and $reclen-1028<4;
      if ($lenadjust>0) {
	$OPTIONS{'cksumto'}-=$lenadjust;
	$OPTIONS{'cksumpos'}-=$lenadjust;
      }
      my ($cksum,$expectsum,$cksumok);
      if (defined $record and length($record)>0) {
        ($cksum,$expectsum,$cksumok)=controlsum($record);
      } else {
        ($cksum,$expectsum,$cksumok)=(0,0,0);
      }
      return ($cksum,$expectsum,$cksumok) if wantarray();
      last CASE;
    }
    if ($analyse eq 'first') {
      # argument 1 is number of impulses
      # argument 2 string of first impulses to analyse
      $_[1]=2;		# there always myst be 2 ...
      $_[2]='00';	# ... zeros
      last CASE;
    }
  }
  1;
}

sub makedata_blizzard {
# Arguments:
# - (ref to) buffer
# - (ref to) block
# Returns:
# ok
  $_[1]=substr($_[0],0,1024,'');
  1;
}

sub paramcorrect_blizzard {

  $OPTIONS{'cksumto'}=-3;
  $OPTIONS{'cksumpos'}=-2;
  $PWMS{'samplerate'}=$OPTIONS{'samplerate'};
  1;
}

sub infoblock_turbo2000 {
# Arguments:
# - a reference to cas hash
# - cas block number (block_no, int)
# - program title (string)

  my $cas=$_[0];
  my $title=$_[2];

  chomp($title);
  cas_put($cas,$_[1]++,'FUJI',length($title),pack('v',0),$title);
  if ($_[1]==1) {
    cas_put($cas,$_[1]++,'pwms',2,pack('v',$PWMS{'iparams'}),pack('v',$OPTIONS{'samplerate'}));
  }
  $title=pack("CCA11",0,255,$title);
  my $cksum=chr(controlsum($title));
  $title=substr($title,0,-1);
  cas_put($cas,$_[1]++,'pwmc',3,pack('v',500),
	  pack('C',int($PWMS{'PILOTW'}*$OPTIONS{'samplerate'}/96000+0.5))
	  . pack('v',4096));
  cas_put($cas,$_[1]++,'pwmd',13,
	  pack('C',int($PWMS{'ZEROW'}*$OPTIONS{'samplerate'}/96000+0.5))
	  . pack('C',int($PWMS{'ONEW'}*$OPTIONS{'samplerate'}/96000+0.5)),
	  $title . $cksum);
  cas_put($cas,$_[1]++,'pwmc',3,pack('v',0),
	  pack('C',int($PWMS{'ENDIMP'}*$OPTIONS{'samplerate'}/96000+0.5))
	  . pack('v',1));
  1;
}

sub datablock_turbo2000 {
# Arguments:
# - reference to cas hash
# - cas block number (block_no, int)
# - data (string)
# - number of init calls ($02e2) (int)  (optional)

  my $cas=$_[0];
  my $data=$_[2];
  my $inits=($_[3] or 0);

  my $datalen=length($data);
  $data=pack('va3073',$datalen,$data);
  my $cksum=chr(controlsum($data));
  $data=substr($data,0,-1);
  cas_put($cas,$_[1]++,'pwmc',3,pack('v',200),
	  pack('C',int($PWMS{'PILOTW'}*$OPTIONS{'samplerate'}/96000+0.5))
	  . pack('v',4096));
  cas_put($cas,$_[1]++,'pwmd',length($data)+1,
	  pack('C',int($PWMS{'ZEROW'}*$OPTIONS{'samplerate'}/96000+0.5))
	  . pack('C',int($PWMS{'ONEW'}*$OPTIONS{'samplerate'}/96000+0.5)),
	  $data . $cksum);
  cas_put($cas,$_[1]++,'pwmc',3,pack('v',0),
	  pack('C',int($PWMS{'ENDIMP'}*$OPTIONS{'samplerate'}/96000+0.5))
	  . pack('v',1));
  1;
}

sub analyse_turbo2000 {
# Arguments:
#  - type of analysis (string):
#     cksum: checksum analysis
#  - data to analyse (string)

  my $analyse=$_[0];
  CASE: {
    if ($analyse eq 'cksum') {
      my ($cksum,$expectsum,$cksumok);
      my $record=$_[1];
      my $header=unpack('v',substr($record,0,2));
      return (0,255,FALSE) if (! $header) and (! $DATASTORE{'t2000-nf'});
      if (($header!=65280 and $header!=3072)
	  or ($header==3072 and length($record)!=3075)
	  or ($header==65280 and length($record)!=13)
	  or $DATASTORE{'t2000-nf'}) {
	print $LOGFILE "Turbo 2000 new format or Ajek detected\n" if defined $LOGFILE;
	my $start=0;
	my $partial;
	my $reclen;
	T2000sum: {
	  while ($start<length($record)) {
	    if (substr($record,$start,2) eq pack('CC',(0xff,0xff))) {
	      if (length($record)==8) {
	        print $LOGFILE "T2000 new format: start of file\n" if defined $LOGFILE;
	        $reclen=7;
		$DATASTORE{'t2000-nf'}=TRUE;
	        $partial=substr($record,$start,$reclen);
	        ($cksum,$expectsum,$cksumok)=controlsum($partial);
	        last;
	      }
	      if (length($record)-$start==2 and (substr($record,$start,2) eq pack('CC',(0xff,0xff)))) {
	        print $LOGFILE "T2000 new format: end of file\n" if defined $LOGFILE;
	        $reclen=2;
	        $partial=substr($record,$start,$reclen);
	        ($cksum,$expectsum,$cksumok)=controlsum($partial);
		last;
	      }
	      if (length($record)==6) {
	        print $LOGFILE "T2000 new format: end of file\n" if defined $LOGFILE;
	        $reclen=5;
	        $partial=substr($record,$start,$reclen);
	        ($cksum,$expectsum,$cksumok)=controlsum($partial);
		$DATASTORE{'t2000-nf'}=FALSE;
	        last;
	      }
	      if (length($record)==2) {
	        print $LOGFILE "Ajek: end of file\n" if defined $LOGFILE;
	        $reclen=2;
	        ($cksum,$expectsum,$cksumok)=controlsum($record);
	        last;
	      }
	    }
	    if ($DATASTORE{'t2000-nf'}) {
		$reclen=length($record)-1;
	    } else {
		$reclen=unpack('v',substr($record,$start+2,2))-unpack('v',substr($record,$start,2))+6;
		last T2000sum if $reclen<=0;	# this is not t2000 new format or decoding error
		#print $LOGFILE "reclen: $reclen\n";
	    }
	    $partial=substr($record,$start,$reclen);
	    #print $LOGFILE "firstbyte: ",ord(substr($partial,0)),"\n";
	    ($cksum,$expectsum,$cksumok)=controlsum($partial);
	    if (!$cksumok) {
	      print $LOGFILE "T2000 nf: Block checksum at $start wrong (expect: $expectsum, got: $cksum),\n" if defined $LOGFILE;
	      # abort t2000 nf if checksum is wrong - there is no guarantee the next data will be OK.
	      print $LOGFILE "T2000 nf: Not continuing - assuming error\n" if defined $LOGFILE;
	      ($cksum,$expectsum,$cksumok)=controlsum($record);
	      last T2000sum;
	    } # else {
	      print $LOGFILE "T2000 nf: Block checksum at $start OK ($cksum)\n" if defined $LOGFILE;
	    #}
	    $start+=$reclen;
	    $start++ if $DATASTORE{'t2000-nf'};
	  }
	}
	($cksum,$expectsum,$cksumok)=controlsum($record) if $reclen<=0;
      } else {
	($cksum,$expectsum,$cksumok)=controlsum($record);
      }
      if ($DATASTORE{'t2000-nf'}) {
	$PWMS{'ENDIMP'} = 0;
      } else {
        $PWMS{'ENDIMP'} = $BASEWIDTH*2;
      }
      last CASE unless wantarray();
      return ($cksum,$expectsum,$cksumok);
    }
  }
  1;
}

sub makedata_turbo2000 {
# Arguments:
# - (ref to) buffer
# - (ref to) block
# Returns:
# ok

  $_[1]=substr($_[0],0,3072,'');
  1;
}

sub paramcorrect_turbo2000 {

  $PWMS{'samplerate'}=$OPTIONS{'samplerate'};
  1;
}

sub infoblock_ast {

  1;
}

sub datablock_ast {

  1;
}

sub analyse_ast {
# Arguments:
#  - type of analysis (string):
#     cksum: checksum analysis
#  - data to analyse (string)

  my $analyse=$_[0];
  CASE: {
    if ($analyse eq 'cksum') {
      my $record=$_[1];
      if (!defined $DATASTORE{'astfirst'} and (length($record)==256)) {
	$DATASTORE{'astblock'}=0;
	$DATASTORE{'astblocks'}=ord(substr($record,0,1));
	$DATASTORE{'astcksums'}=[ unpack('C*',substr($record,200,$DATASTORE{'astblocks'}+1)) ];
	$DATASTORE{'astfirst'}=TRUE;
      } else {
	$DATASTORE{'astfirst'}=undef;
      }
      my ($cksum,$expectsum,$cksumok)=controlsum($record);
      $cksumok=FALSE if (defined($DATASTORE{'astcksums'}) and $cksum!=$DATASTORE{'astcksums'}[$DATASTORE{'astblock'}]);
      #$cksumok=TRUE if (defined($DATASTORE{'astfirst'}) and $cksum==0);
      if (defined($DATASTORE{'astcksums'})) {
	$expectsum=$DATASTORE{'astcksums'}[$DATASTORE{'astblock'}];
	$DATASTORE{'astcksums'}=undef if ++$DATASTORE{'astblock'} > $DATASTORE{'astblocks'}
      }
      return ($cksum,$expectsum,$cksumok) if wantarray();
      last CASE;
    }
  }
  1;
}

sub makedata_ast {

  1;
}

sub paramcorrect_ast {

  $PWMS{'samplerate'}=$OPTIONS{'samplerate'};
  1;
}

sub infoblock_att {

  1;
}

sub datablock_att {

  1;
}

sub analyse_att {
# Arguments:
#  - type of analysis (string):
#     cksum: checksum analysis
#  - data to analyse (string)

  my $analyse=$_[0];
  CASE: {
    if ($analyse eq 'cksum') {
      my $record=$_[1];
      my ($cksum,$expectsum,$cksumok)=controlsum($record);
      return ($cksum,$expectsum,$cksumok) if wantarray();
      last CASE;
    }
  }
  1;
}

sub makedata_att {

  1;
}

sub paramcorrect_att {

  $PWMS{'samplerate'}=$OPTIONS{'samplerate'};
  1;
}

sub infoblock_um {

  1;
}

sub datablock_um {

  1;
}

sub analyse_um {
# Arguments:
#  - type of analysis (string):
#     cksum: checksum analysis
#  - data to analyse (string)

  my $analyse=$_[0];


  CASE: {
    if ($analyse eq 'cksum') {
      my $record=$_[1];
      if (!defined $DATASTORE{'umfirst'} and (length($record)==271 or length($record)==270)) {
	$DATASTORE{'umcksum'}=ord(substr($record,262,1));
	$DATASTORE{'umfirst'}=TRUE;
	$DATASTORE{'umfirstcksumless'}=(length($record)!=271);
      } else {
	$DATASTORE{'umfirst'}=undef;
	$DATASTORE{'umfirstcksumless'}=TRUE;
      }
      if (!defined($DATASTORE{'umfirst'}) and defined($DATASTORE{'umcksum'})) {
	$OPTIONS{'cksumto'}=-1;
	$OPTIONS{'cksumpos'}=0;
      }
      my ($cksum,$expectsum,$cksumok)=controlsum($record);
      $cksumok=FALSE if (!defined($DATASTORE{'umfirst'}) and defined($DATASTORE{'umcksum'}) and $cksum!=$DATASTORE{'umcksum'});
      $cksumok=TRUE if (defined($DATASTORE{'umfirst'}) and $DATASTORE{'umfirstcksumless'});
      if (!defined $DATASTORE{'umfirst'} and defined($DATASTORE{'umcksum'})) {
	$OPTIONS{'cksumto'}=-2;
	$OPTIONS{'cksumpos'}=-1;
	$expectsum=$DATASTORE{'umcksum'} ;
	$DATASTORE{'umcksum'}=undef;
      }
      return ($cksum,$expectsum,$cksumok) if wantarray();
      last CASE;
    }
  }
  1;
}

sub makedata_um {

  1;
}

sub paramcorrect_um {

  $PWMS{'samplerate'}=$OPTIONS{'samplerate'};
  1;
}

sub infoblock_cs2000 {

  1;
}

sub datablock_cs2000 {

  1;
}

sub analyse_cs2000 {
# Arguments:
#  - type of analysis (string):
#     cksum: checksum analysis
#  - data to analyse (string)

  my $analyse=$_[0];
  CASE: {
    if ($analyse eq 'cksum') {
      my $record=$_[1];
      my ($cksum,$expectsum,$cksumok)=controlsum($record);
      return ($cksum,$expectsum,$cksumok) if wantarray();
      last CASE;
    }
  }
  1;
}

sub makedata_cs2000 {

  1;
}

sub paramcorrect_cs2000 {

  $PWMS{'samplerate'}=$OPTIONS{'samplerate'};
  1;
}

sub infoblock_csst {

  1;
}

sub datablock_csst {

  1;
}

sub analyse_csst {
# Arguments:
#  - type of analysis (string):
#     cksum: checksum analysis
#  - data to analyse (string)

  my $analyse=$_[0];
  CASE: {
    if ($analyse eq 'cksum') {
      my $record=$_[1];
      my ($cksum,$expectsum,$cksumok)=controlsum($record);
      return ($cksum,$expectsum,$cksumok) if wantarray();
      last CASE;
    }
  }
  1;
}

sub makedata_csst {

  1;
}

sub paramcorrect_csst {

  $PWMS{'samplerate'}=$OPTIONS{'samplerate'};
  1;
}

sub infoblock_cstt {

  1;
}

sub datablock_cstt {

  1;
}

sub analyse_cstt {
# Arguments:
#  - type of analysis (string):
#     cksum: checksum analysis
#  - data to analyse (string)

  my $analyse=$_[0];
  CASE: {
    if ($analyse eq 'cksum') {
      my $record=$_[1];
      my ($cksum,$expectsum,$cksumok)=controlsum($record);
      return ($cksum,$expectsum,$cksumok) if wantarray();
      last CASE;
    }
  }
  1;
}

sub makedata_cstt {

  1;
}

sub paramcorrect_cstt {

  $PWMS{'samplerate'}=$OPTIONS{'samplerate'};
  1;
}

sub infoblock_csbtape {

  1;
}

sub datablock_csbtape {

  1;
}

sub analyse_csbtape {
# Arguments:
#  - type of analysis (string):
#     cksum: checksum analysis
#  - data to analyse (string)

  my $analyse=$_[0];
  CASE: {
    if ($analyse eq 'cksum') {
      my $record=$_[1];
      my ($cksum,$expectsum,$cksumok)=controlsum($record);
      return ($cksum,$expectsum,$cksumok) if wantarray();
      last CASE;
    }
  }
  1;
}

sub makedata_csbtape {

  1;
}

sub paramcorrect_csbtape {

  $PWMS{'samplerate'}=$OPTIONS{'samplerate'};
  1;
}

sub infoblock_lowsil2000 {

  1;
}

sub datablock_lowsil2000 {

  1;
}

sub analyse_lowsil2000 {
# Arguments:
#  - type of analysis (string):
#     cksum: checksum analysis
#  - data to analyse (string)

  my $analyse=$_[0];
  CASE: {
    if ($analyse eq 'cksum') {
      my $record=$_[1];
      my ($cksum,$expectsum,$cksumok)=controlsum($record);
      return ($cksum,$expectsum,$cksumok) if wantarray();
      last CASE;
    }
  }
  1;
}

sub makedata_lowsil2000 {

  1;
}

sub paramcorrect_lowsil2000 {

  $PWMS{'samplerate'}=$OPTIONS{'samplerate'};
  1;
}

sub infoblock_turborom {

  1;
}

sub datablock_turborom {

  1;
}

sub analyse_turborom {
# Arguments:
#  - type of analysis (string):
#     cksum: checksum analysis
#  - data to analyse (string)

  my $analyse=$_[0];
  CASE: {
    if ($analyse eq 'cksum') {
      my $record=$_[1];
      if (!defined $DATASTORE{'tromfirst'} and (length($record)<80)) {
	$DATASTORE{'tromcksum'}=ord(substr($record,5,1));
	$DATASTORE{'tromfirst'}=TRUE;
      } else {
	$DATASTORE{'tromfirst'}=undef;
      }
      if (!defined($DATASTORE{'tromfirst'}) and defined($DATASTORE{'tromcksum'})) {
	$OPTIONS{'cksumfrom'}=1;
	$OPTIONS{'cksumto'}=-1;
	$OPTIONS{'cksumpos'}=0;
      }
      my ($cksum,$expectsum,$cksumok)=controlsum($record);
      $cksumok=FALSE if (!defined($DATASTORE{'tromfirst'}) and defined($DATASTORE{'tromcksum'}) and $cksum!=$DATASTORE{'tromcksum'});
      #$cksumok=TRUE if defined($DATASTORE{'tromfirst'});
      if (!defined $DATASTORE{'tromfirst'} and defined($DATASTORE{'tromcksum'})) {
	$OPTIONS{'cksumfrom'}=2;
	$OPTIONS{'cksumto'}=-1;
	$OPTIONS{'cksumpos'}=1;
	$expectsum=$DATASTORE{'tromcksum'} ;
	$DATASTORE{'tromcksum'}=undef;
      }
      return ($cksum,$expectsum,$cksumok) if wantarray();
      last CASE;
    }
  }
  1;
}

sub makedata_turborom {

  1;
}

sub paramcorrect_turborom {

  $PWMS{'samplerate'}=$OPTIONS{'samplerate'};
  1;
}

sub infoblock_mkturbo {

  1;
}

sub datablock_mkturbo {

  1;
}

sub analyse_mkturbo {
# Arguments:
#  - type of analysis (string):
#     cksum: checksum adjustments
#  - data to analyse (string)

  my $analyse=$_[0];

  CASE: {
    if ($analyse eq 'cksum') {
      my $record=$_[1];
      my $reclen=length($record);
      my ($cksum,$expectsum,$cksumok);
      if (substr($record,0,2) eq pack('v',0x5555))  {
	$OPTIONS{'cksumfrom'}=3;
	$OPTIONS{'cksumto'}=-2;
	$OPTIONS{'cksumpos'}=-1;
      } else {
	$OPTIONS{'cksumfrom'}=2;
	if ($reclen<=1027) {
	    $OPTIONS{'cksumto'}=-2;
	    $OPTIONS{'cksumpos'}=-1;
	} else {
	    $OPTIONS{'cksumto'}=-3;
	    $OPTIONS{'cksumpos'}=-2;
	}
      }
      ($cksum,$expectsum,$cksumok)=controlsum($record);
      return ($cksum,$expectsum,$cksumok) if wantarray();
      last CASE;
    }
  }
  1;
}

sub makedata_mkturbo {

  1;
}

sub paramcorrect_mkturbo {

  $PWMS{'samplerate'}=$OPTIONS{'samplerate'};
  1;
}

sub infoblock_6000 {

  1;
}

sub datablock_6000 {

  1;
}

sub analyse_6000 {
# Arguments:
#  - type of analysis (string):
#     cksum: checksum adjustments
#  - data to analyse (string)

  my $analyse=$_[0];

  CASE: {
    if ($analyse eq 'cksum') {
      my $record=$_[1];
      my $reclen=length($record);
      my ($cksum,$expectsum,$cksumok);
      my $partial;
      my $count;
      my @arecord=split('',$record);
      my $i=0;
      do $i++ while $arecord[$i] eq chr(2);
      $i+=9;
      if ($arecord[$i] eq chr(01)) {
        $i++;
        $count=65536 - unpack('v',join('',@arecord[$i..$i+1]));
        $i+=24;
        do $i++ while $arecord[$i] ne chr(2);
        do $i++ while $arecord[$i] eq chr(2);
        $i+=10;
        $partial=join('',@arecord[$i..$i+$count]);
      } else {
        $partial=$record;
      }
      ($cksum,$expectsum,$cksumok)=controlsum($partial);
      return ($cksum,$expectsum,$cksumok) if wantarray();
      last CASE;
    }
  }
  1;
}

sub makedata_6000 {

  1;
}

sub paramcorrect_6000 {

  $PWMS{'samplerate'}=$OPTIONS{'samplerate'};
  1;
}

sub infoblock_hardturbo {

  1;
}

sub datablock_hardturbo {

  1;
}

sub analyse_hardturbo {
# Arguments:
#  - type of analysis (string):
#     cksum: checksum analysis
#  - data to analyse (string)

  my $analyse=$_[0];
  CASE: {
    if ($analyse eq 'cksum') {
      my $record=$_[1];
      my ($cksum,$expectsum,$cksumok)=controlsum($record);
      return ($cksum,$expectsum,$cksumok) if wantarray();
      last CASE;
    }
  }
  1;
}

sub makedata_hardturbo {

  1;
}

sub paramcorrect_hardturbo {

  $PWMS{'samplerate'}=$OPTIONS{'samplerate'};
  1;
}


sub infoblock_autoturbo {

  1;
}

sub datablock_autoturbo {

  1;
}

sub analyse_autoturbo {
# Arguments:
#  - type of analysis (string):
#     cksum: checksum analysis
#  - data to analyse (string)

  my $analyse=$_[0];
  CASE: {
    if ($analyse eq 'cksum') {
      my $record=$_[1];
      my ($cksum,$expectsum,$cksumok)=controlsum($record);
      return ($cksum,$expectsum,$cksumok) if wantarray();
      last CASE;
    }
    if ($analyse eq 'first') {
      # argument 1 is number of impulses
      # argument 2 string of first impulses to analyse
      if (!defined($DATASTORE{'autoturbofirst'})) {
	$DATASTORE{'autoturbofirst'}=1;
	$_[1]=1;	# in first block  there must be 1 ...
	$_[2]='0';	# ... zero
      } elsif ($DATASTORE{'autoturbofirst'}==1) {
	$DATASTORE{'autoturbofirst'}=2;
	$_[1]=1;	# in 2nd block there must be 1 ...
	$_[2]='0';	# ... zero
      } else {
        $_[1]=2;	# in next blocks there always must be 2 ...
        $_[2]='00';	# ... zeros
      }
      last CASE;
    }
  }
  1;
}

sub makedata_autoturbo {

  1;
}

sub paramcorrect_autoturbo {

  $PWMS{'samplerate'}=$OPTIONS{'samplerate'};
  1;
}


sub infoblock_tapewiz {
  1;
}

sub datablock_tapewiz {
}


sub analyse_tapewiz {
# Arguments:
#  - type of analysis (string):
#     cksum: checksum adjustments
#  - data to analyse (string)

  my $analyse=$_[0];
  CASE: {
    if ($analyse eq 'cksum') {
      my $record=$_[1];
      my ($cksum,$expectsum,$cksumok)=controlsum($record);
      return ($cksum,$expectsum,$cksumok) if wantarray();
      last CASE;
    }
  }
  1;
}

sub makedata_tapewiz {

  1;
}

sub paramcorrect_tapewiz {

  $PWMS{'samplerate'}=$OPTIONS{'samplerate'};
  1;
}

sub prepare_normal {

  $OPTIONS{'cksum'}=($OPTIONS{'cksumtype'} or 'sum255carry');
  $OPTIONS{'cksumfrom'}=1;
  $OPTIONS{'cksumto'}=-2;
  $OPTIONS{'cksumpos'}=-1;

}

sub infoblock_normal {
# Arguments:
# - a reference to cas hash
# - cas block number (block_no, int)
# - program title (string)

  my $cas=$_[0];
  my $title=$_[2];

  chomp($title);
  if ($_[1]==0) {
    cas_put($cas,$_[1]++,'FUJI',length($title),pack('v',0),$title);
    cas_put($cas,$_[1]++,'baud',0,pack('v',600));
  }

  1;
}

sub datablock_normal {
# Arguments:
# - reference to cas hash
# - cas block number (block_no, int)
# - data (string)
# - number of init calls ($02e2) (int) or sof/eof indicator (optional)
# - length of pilot
  my $cas=$_[0];
  my $data=$_[2];
  my $inits=($_[3] or 0);
  my $pilot=($_[4] or 0);

  my $irg=272;
  $irg=($pilot==0 ? 20000 : $pilot) if $inits==254; #sof(start of file)
  my $datalen=length($data);
  #my $blocktype=pack('C',254); #full length block
  my $blocktype=($datalen<128 ? pack('C',250) : pack('C',252)); #last or full block
  $blocktype=pack('C',254) if $inits==255; #eof
  $data=pack('C',85) . pack('C',85) . $blocktype . pack('a129',$data);
  if ($datalen<128) {
    $data=substr($data,0,-2);
    $data.=pack('C',$datalen) . chr(0);
  }
  my $cksum=chr(controlsum($data));
  $data=substr($data,0,-1);
  cas_put($cas,$_[1]++,'data',length($data)+1,
	  pack('v',$irg),
	  $data . $cksum);
  1;
}

use constant {
  TURBO2600HEAD => pack('H*','0000fffffffffefcfcf8f0c7'),
};

sub analyse_normal {
# Arguments:
#  - type of analysis (string):
#     cksum: checksum adjustments
#  - data to analyse (string)

  my $analyse=$_[0];
  my $record=$_[1];


  sub stdcksum {
    my $lenadjust=0;
    my $record=shift;
    my $reclen=length($record);

    $lenadjust=$reclen-132;
    if ($lenadjust>0 and $lenadjust<2) {
      print $LOGFILE "Ignoring garbage at the end of record...\n" if defined $LOGFILE;
    } else {
      $lenadjust=0;
    }
    $OPTIONS{'cksumto'}=0-2-$lenadjust;
    $OPTIONS{'cksumpos'}=0-1-$lenadjust;
    my ($cksum,$expectsum,$cksumok)=controlsum($record);
    if (!$cksumok) {
      $OPTIONS{'cksum'}='sum255alt';
      my ($tempsum,$tempexpectsum)=($cksum,$expectsum);
      ($cksum,$expectsum,$cksumok)=controlsum($record);
      ($cksum,$expectsum)=($tempsum,$tempexpectsum) if (!$cksumok);
      $OPTIONS{'cksum'}='sum255carry';
    }
    return ($cksum,$expectsum,$cksumok);
  }

  CASE: {
    if ($analyse eq 'cksum') {
      my ($cksum,$expectsum,$cksumok);
      if (substr($record,0,12) eq TURBO2600HEAD) {
	print $LOGFILE "Turbo 2600 format detected\n" if defined $LOGFILE;
	my $start=0;
	my $partial;
	T2600sum: {
	  while ($start<length($record)) {
	    if (substr($record,$start,12) eq TURBO2600HEAD) {
	      $partial=substr($record,$start,21);			# calibration + header
	      $start+=12;
	    } else {
	      $partial=substr($record,$start,9);			#  header only
	    }
	    ($cksum,$expectsum,$cksumok)=controlsum($partial);
	    print $LOGFILE "T2600: Header checksum at $start OK ($cksum)\n" if ($cksumok) and defined $LOGFILE;
	    if (!$cksumok) {
	      # abort t2600 if HEADER is wrong - there is no guarantee the next data will be OK.
	      print $LOGFILE "T2600: Header checksum at $start wrong (expect: $expectsum, got: $cksum).\n" if defined $LOGFILE;
	      print $LOGFILE "T2600: Not continuing - assuming error\n" if defined $LOGFILE;
	      ($cksum,$expectsum,$cksumok)=stdcksum($record);
	      last T2600sum;
	    }
	    my $reclen=unpack('v',substr($record,$start+2,2))-unpack('v',substr($record,$start,2))+1;
	    $start+=9;
	    #print $LOGFILE "reclen: $reclen\n" if defined $LOGFILE;
	    $partial=substr($record,$start,$reclen);
	    ($cksum,$expectsum,$cksumok)=controlsum($partial);
	    if ($cksumok) {
	      print $LOGFILE "T2600: Block checksum at $start OK ($cksum)\n" if defined $LOGFILE;
	    } else {
	      print $LOGFILE "T2600: Block checksum at $start wrong (expect: $expectsum, got: $cksum),\n" if defined $LOGFILE;
	    }
	    $start+=$reclen;
	  }
	}
      } else {
	($cksum,$expectsum,$cksumok)=stdcksum($record);
      }
      last CASE unless wantarray();
      return ($cksum,$expectsum,$cksumok);
    }
  }
  1;
}

sub makedata_normal  {
# Arguments:
# - (ref to) buffer
# - (ref to) block
# Returns:
# ok
  $_[1]=substr($_[0],0,128,'');
  1;

}

sub fms_encode {

# Arguments:
#  - data_start - 2 char string, le encoded first address
#  - data_end - 2 char string, le encoded last address
#  - data - string - data
# Returns"
#  - block (string) in FMS format

    my ($data_start,$data_end,$data)=(shift,shift,shift);

    return chr(255) . chr(255) . $data_start . $data_end . $data;
}

sub fms_decode {

# Arguments:
#  - block (string) in FMS format
# Returns"
#  - data_start - 2 char string, le encoded first address
#  - data_end - 2 char string, le encoded last address
#  - data - string - data

    my $block=shift;

    return (substr($block,2,2),substr($block,4,2),substr($block,6));
}

sub cas_init {
# initialise cas hash
# Arguments:
# 0 - reference to cas hash
  my $cas=$_[0];

  ${$cas}{'records'}=0;
  ${$cas}{'programs'}={};
}

sub cas_put {

# store block in cas hash
# Arguments:
# 0 - reference to cas hash
# 1 - block number (int)
# 2 - block type (string)
# 3 - block length (int)
# 4 - auxilliary bytes (word, le)
# 5 - data (string)
# Returns (context sensitive):
#  scalar context:
#   - computed checksum or OK
#  array context:
#   - computed checksum
#   - expected checksum
#   - verify status

  my $cas=$_[0];
  my $block_no=$_[1];
  my $block_type=$_[2];
  my $block_len=$_[3];
  my $block_aux=$_[4];
  my $block_data=$_[5];

  my ($cksum,$expectsum,$cksumok);

  my $programs;

   if ($block_type=~m/(data)|(pwml)|(pwmc)|(pwms)/) {
     if (unpack("v",$block_aux) > $OPTIONS{'maxgap'}
	 or $block_type eq 'pwms'
	 or ($block_type=~m/(pwmc)|(pwml)/ and defined(${$cas}{$block_no-1})
	    and ${$cas}{$block_no-1}{'type'}=~m/(data)|(fsk )/)
	 or ($block_type=~m/(data)|(fsk)|(baud)/ and defined(${$cas}{$block_no-1})
	    and ${$cas}{$block_no-1}{'type'}=~m/(pwmc)|(pwmd)|(pwml)/)
	 or (defined(${$cas}{$block_no-1}) and ${$cas}{$block_no-1}{'type'} eq 'FUJI')) {
       # file starts at this block
       $programs=(keys %{${$cas}{'programs'}} or 0);
       my $files=(keys %{${$cas}{'programs'}{$programs}{'files'}} or 0);
       if (${$cas}{'records'}>0 and $OPTIONS{'autofuji'}) {
	 if (${$cas}{$block_no-1}{'type'} eq 'baud') {
	    ${$cas}{$block_no}{'type'}='baud';
	    ${$cas}{$block_no}{'length'}=0;
	    ${$cas}{$block_no}{'aux'}=${$cas}{$block_no-1}{'aux'};
	    $block_no--;
	 }
         ${$cas}{$block_no}{'type'}='FUJI';
         ${$cas}{$block_no}{'length'}=0;
         ${$cas}{$block_no}{'aux'}=pack('v',0);
	 ${$cas}{'programs'}{++$programs}=$block_no; # "program" starts at this block
         $block_no++ if defined(${$cas}{$block_no+1});
         $block_no++;
         $_[1]++;
       }
       #print "Adding file $files\n";
       if (defined(${$cas}{$block_no-1}) and ${$cas}{$block_no-1}{'type'} eq 'baud') {
	 ${$cas}{'programs'}{$programs}{'files'}{$files+1}=$block_no-1;
       } else {
	 ${$cas}{'programs'}{$programs}{'files'}{$files+1}=$block_no;
       }
     }
   }

  if (($block_type eq 'pwmc') and
      (defined ${$cas}{$block_no-1}) and
      (${$cas}{$block_no-1}{'type'} eq 'pwmc') and
      (unpack('v',$block_aux)==0)) {
    $block_no--; $_[1]--;
    ${$cas}{$block_no}{'length'}+=$block_len;
    ${$cas}{$block_no}{'data'}.=$block_data;
    return OK;
  }

  if ($block_type eq 'pwml' and
      defined ${$cas}{$block_no-1} and
      ${$cas}{$block_no-1}{'type'} eq 'pwml' and
      unpack('v',$block_aux)==0 and
      ${$cas}{$block_no-1}{'length'}+$block_len < 65533) {
	if (${$cas}{$block_no-1}{'length'} % 2 == 1) {
	    ${$cas}{$block_no-1}{'length'}+=2;
	    ${$cas}{$block_no-1}{'data'}.=pack("v",0);
	    }
	$block_no--; $_[1]--;
	${$cas}{$block_no}{'length'}+=$block_len;
	${$cas}{$block_no}{'data'}.=$block_data;
	return OK;
  }

  # join short 'fsk ' blocks into one bigger
  FSKJOIN: { if ($block_type eq 'fsk ' and defined ${$cas}{$block_no-1} and
	    ${$cas}{$block_no-1}{'type'} eq 'fsk ') {
	  my @fskdata=();
	  @fskdata=unpack('v*',${$cas}{$block_no-1}{'data'}) if defined ${$cas}{$block_no-1}{'data'};
	  if ($#fskdata % 2) {
	    if (defined ${$cas}{$block_no-1}{'data'}) {
	      if ($fskdata[$#fskdata]+unpack('v',$block_aux)*10<=65535)  {
		$fskdata[$#fskdata]+=unpack('v',$block_aux)*10;
	      }  else  {
		last FSKJOIN; # Abort
	      }
	    } else  {
	      if (unpack('v',${$cas}{$block_no-1}{'aux'})+unpack('v',$block_aux)<=65535) {
		${$cas}{$block_no-1}{'aux'}=pack('v',unpack('v',${$cas}{$block_no-1}{'aux'})+unpack('v',$block_aux));
	      } else {
		last FSKJOIN; # Abort
	      }
	    }
	  } else {
	    if (unpack('v',$block_aux)>0) {
	      if (unpack('v',$block_aux)*10<=65535) {
		push(@fskdata,unpack('v',$block_aux)*10);
		$block_len+=2;
	      } else  {
		last FSKJOIN;	# Abort
	      }
	    } else {
	      if ($block_len>0 and $fskdata[$#fskdata]+unpack('v',substr($block_data,0,2))<=65535) {
		$fskdata[$#fskdata]+=unpack('v',substr($block_data,0,2,''));
		$block_len-=2;
	      } else {
		last FSKJOIN;	# Abort
	      }
	    }
	  }
	  push(@fskdata,unpack('v*',$block_data)) if $block_len>0;
	  $block_len=($#fskdata+1)*2;
	  $block_aux=${$cas}{$block_no-1}{'aux'};
	  $block_data=pack('v*',@fskdata);
	  $block_no--; $_[1]--;
	}
  }	# End FSKJOIN

  # split fsk block longer than 32767 signals into smaller
  FSKSPLIT: { my @fskdata;
	  while ($block_type eq 'fsk ' and $block_len > 65535) {
	    @fskdata=unpack('v*',$block_data);
	    ${$cas}{$block_no}{'type'}=$block_type;
	    ${$cas}{$block_no}{'length'}=32767*2;
	    ${$cas}{$block_no}{'aux'}=$block_aux;
	    ${$cas}{$block_no}{'data'}=pack('v*',splice(@fskdata,0,32767));
	    unshift(@fskdata,0);
	    $block_len=($#fskdata+1)*2;
	    $block_data=pack('v*',@fskdata);
	    $block_aux=pack('v',0);
	    $_[1]++;
	    $block_no++;
	  }
  }	# End FSKSPLIT

  ${$cas}{$block_no}{'type'}=$block_type;
  ${$cas}{$block_no}{'length'}=$block_len;
  ${$cas}{$block_no}{'aux'}=$block_aux;
  ${$cas}{$block_no}{'data'}=$block_data if $block_len!=0;
  if ($block_type eq 'data') {
    if ($block_len>0) {
      ($cksum,$expectsum,$cksumok)=analyse_normal('cksum',$block_data);
    } else {
      ($cksum,$expectsum,$cksumok)=(0,0,0);
    }
    ${$cas}{$block_no}{'cksum'}=$cksum;
    ${$cas}{$block_no}{'cksumok'}=$cksumok;
    ${$cas}{$block_no}{'cksumtype'}=$OPTIONS{'cksum'};
  }
  if ($block_type eq 'pwmd') {
    if ($block_len>0) {
      ($cksum,$expectsum,$cksumok)=eval('analyse_' . lc $OPTIONS{'turbo'} . '(\'cksum\',$block_data)');
    } else {
      ($cksum,$expectsum,$cksumok)=(0,0,0);
    }
    ${$cas}{$block_no}{'cksum'}=$cksum;
    ${$cas}{$block_no}{'cksumok'}=$cksumok;
    ${$cas}{$block_no}{'cksumtype'}=$OPTIONS{'cksum'};
    ${$cas}{$block_no}{'turbo'}=$OPTIONS{'turbo'};
  }
  ${$cas}{'records'}=0 if $block_no==0;
  ${$cas}{'records'}++ if $block_type=~m/(data)|(pwmd)/;
  if ($block_type eq 'FUJI') {
    $programs=keys %{${$cas}{'programs'}};
    ${$cas}{'programs'}{++$programs}=$block_no; # "program" starts at this block
  }
  if ($block_type eq 'pwms') {
    $PWMS{'samplerate'}=unpack('v',$block_data);
  }
  if (wantarray()) {
    return ($cksum,$expectsum,$cksumok);
  } else {
    if ($block_type eq 'data' or $block_type eq 'pwmd') {
      return ${$cas}{$block_no}{'cksum'};
    } else {
      return OK;
    }
  }
}

sub cas_get {

# get one block from cas hash
# Arguments:
# 0 - reference to cas hash
# 1 - block number (int)
# 2 - block type (string)
# 3 - block length (int)
# 4 - auxilliary bytes (word, le)
# 5 - data (string)
# Returns:
#  always OK

  my $cas=$_[0];
  my $block_no=$_[1];

	$_[2]=${$cas}{$block_no}{'type'};
	$_[3]=${$cas}{$block_no}{'length'};
	$_[4]=${$cas}{$block_no}{'aux'};
	$_[5]=${$cas}{$block_no}{'data'} if $_[3]!=0;
  1;
}

sub cas_slurp {

#
#Read CAS into memory.
#CAS is small, it does not hurt.
#
#Arguments:
#  filename - name of CAS file to read
#  cas  - a hash
#
#Return value:
#  OK if success
#  error if failure
#

  my $filename=$_[0];
  my $cas=$_[1];
  my $buffer;
  my ($block_type,$block_len,$block_aux);
  my $counter=(keys %{$cas})-2;
	my $oldcounter=$counter;
  # my $maxgap=$OPTIONS{'maxgap'};

  my $result;
  my $programs=0;
  my $files=0;

  open(CASFILE,"<$filename")  or die "Can't open CAS file $filename for reading.\n";

#  cas_init($cas);

  while($result=sysread(CASFILE,$buffer,8)) {
    ($block_type,$block_len,$block_aux)=(
					  substr($buffer,0,4),
					  unpack("v",substr($buffer,4,2)),
					  substr($buffer,6,2)
					 );
    #print $block_type,",",$block_len,"\n";

    if ($block_len>0) {
      sysread(CASFILE,$buffer,$block_len) or die "CAS read error.";
    } else {
      $buffer="";
    }

    #ignore unknown blocks
    next if $block_type !~ m/$CAS_BLOCKS_REGEX/o;

    prepare_normal if $block_type eq 'data';
    #analyse_normal('cksum',$buffer) if $block_type eq 'data';
    eval('prepare_' . lc $OPTIONS{'turbo'} . '()') if $block_type eq 'pwmd';
    #eval('analyse_' . lc $OPTIONS{'turbo'} . '(\'cksum\',$buffer)') if $block_type eq 'pwmd';
    cas_put($cas,$counter,$block_type,$block_len,$block_aux,$buffer);
    $counter++;
	}
  return $! if !defined $result;
#  return NOT_GOOD if $result!=END_OF_FILE;
  close(CASFILE) or die "Can't close input file $filename properly!\n";
  print "Slurped ",$counter-$oldcounter," blocks from CAS file $filename.\n";
  return OK;
}

sub cas_spit {

#Write CAS from memory to disk.
#
#Arguments:
#  filename - name of file to write to
#  cas - a reference to hash for storing CAS data
#
#Return value:
#  OK if success
#  error if failure

  my $filename=$_[0];
  my $cas=$_[1];
  my $blocks=0;
  my $keys=0;

  # for samplerate recalculating
  my $factor=$OPTIONS{'samplerate'}/$PWMS{'samplerate'} if $OPTIONS{'turbo'} ne '';
  my $component=($OPTIONS{'samplerate'} < $PWMS{'samplerate'})*0.5  if $OPTIONS{'turbo'} ne '';

  #don't overwrite existing files
  $filename=findofile($filename);

  open(CASFILE,">$filename") or die "Can't open CAS file $filename for writting\n";
  my $i=0;
  for($keys=0;$keys++<keys %{$cas};$i++) {
    next if not defined ${$cas}{$i};
    if ($OPTIONS{'resamplecas'}) {
      # this may be useful some day...
      CASE: for (${$cas}{$i}{'type'}) {
	/pwms/ && do {
		      ${$cas}{$i}{'data'}=pack('v',$OPTIONS{'samplerate'});
		     };
	/pwml/ && do {
		      my $j=0;
		      my @temparray=unpack('v*',${$cas}{$i}{'data'});
		      while ($j<$#temparray) {
			$temparray[$j]=int($temparray[$j]*$factor+$component);
			$j++;
		      }
		      ${$cas}{$i}{'data'}=pack('v*',@temparray);
		     };
	/pwmc/ && do {
		      my $j=0;
		      my @temparray=unpack('(Cv)*',${$cas}{$i}{'data'});
		      while ($j<$#temparray) {
			$temparray[$j]=int($temparray[$j]*$factor+$component);
			$j+=2;
		      }
		      ${$cas}{$i}{'data'}=pack('(Cv)*',@temparray);
		     };
	/pwmd/ && do {
		      my ($aux1,$aux2)=unpack('CC',${$cas}{$i}{'aux'});
		      ${$cas}{$i}{'aux'}=pack('CC',int($aux1*$factor+$component),int($aux2*$factor+$component));
		     };
      }
    }
    print CASFILE ${$cas}{$i}{'type'};
    print CASFILE pack("v",${$cas}{$i}{'length'});
    print CASFILE ${$cas}{$i}{'aux'};
    print CASFILE ${$cas}{$i}{'data'} if ${$cas}{$i}{'length'}!=0;
    #print "$i,${$cas}{$i}{'type'}\n";
    $blocks++;
  }
  print $blocks," CAS blocks spitted into file $filename.\n";

  close(CASFILE) or die "Can't close output file $filename properly!\n";
  return OK;
}

sub hex_slurp {

#
#Read HEX into memory.
#HEX is small, it does not hurt.
#
#Arguments:
#  filename - name of HEX file to read
#  - a hash referencing cas hash
#
#Return value:
#  OK if success
#  error if failure
#

  my $filename=$_[0];
  my $cas=$_[1];
  my $buffer;
  my ($block_type,$block_len,$block_aux,$block_data);
  my $counter=(keys %{$cas})-2;
	my $oldcounter=$counter;
  my @line;
  my $result;

	eval('prepare_' . lc $OPTIONS{'turbo'} .  '()') if $OPTIONS{'turbo'} ne '';

  open(HEXFILE,"<$filename")  or die "Can't open HEX file $filename for reading.\n";
  $_=<HEXFILE>; chomp;
  die "HEX format error" if ($_ !~ m/A8CAS-HEX/);
  while(<HEXFILE>) {
    chomp;
    $block_type=substr($_,0,5,'');
    chop($block_type) if length($block_type)>4; # only chop if there is anything after header
    s/(^.*?) ?;.*$/$1/;
    @line=split(' ',$_);

    #ignore unknown blocks
    next if $block_type !~ m/$CAS_BLOCKS_REGEX/o;

    CASE: for ($block_type) {
      /FUJI/ && do { $block_aux=pack('v',0);
		     $block_len=length(join(' ',@line));
		     $block_data=join(' ',@line);
		     last CASE;
		   };
      /baud/ && do { $block_aux=pack('v',$line[0]);
		     $block_len=0;
		     $block_data='';
		     last CASE;
		    };
      /data/ && do { prepare_normal();
		     $block_aux=pack('v',$line[0]);
		     $block_len=$#line;
		     $block_data='';
		     foreach (@line[1..$block_len]) {
		      $block_data.=chr(hex);
		     }
		     last CASE;
		    };
      /fsk / && do { $block_aux=pack('v',$line[0]);
		     $block_len=$#line*2;
		     $block_data=pack('v*',@line[1..$#line]);
		     last CASE;
		    };
      /pwms/ && do { my $aux;
		     for (0..1) {
			$aux += ($line[$_] eq 'msb_first' ? 4 : 0);
			$aux += ($line[$_] eq 'rising_edge_first' ? 2 : 0);
			$aux += ($line[$_] eq 'falling_edge_first' ? 1 : 0);
		     }
		     $block_aux=pack('v',$aux);
		     $block_len=2;
		     $block_data=pack('v',$line[2]);
		     $PWMS{'samplerate'}=$line[1];
		     last CASE;
		    };
      /pwml/ && do { $block_aux=pack('v',$line[0]);
		     $block_len=$#line*2;
		     if ($block_len>0) {
		       $block_data=pack('v*',@line[1..$#line]);
		     } else {
		       $block_data='';
		     }
		     last CASE;
		    };
      /pwmc/ && do { $block_aux=pack('v',$line[0]);
		     $block_len=$#line*3/2;
		     if ($block_len>0) {
		       $block_data=pack('(Cv)*',@line[1..$#line]);
		     } else {
		       $block_data='';
		     }
		     last CASE;
		    };
      /pwmd/ && do { eval('prepare_' . lc $OPTIONS{'turbo'} .  '()');
		     $block_aux=pack('CC',@line[0,1]);
		     $block_len=$#line-1;
		     $block_data='';
		     foreach (@line[2 .. $#line]) {
		      $block_data.=chr(hex);
		     }
		     #eval('analyse_' .  lc $OPTIONS{'turbo'} . '(\'cksum\',$block_data)');
		     last CASE;
		    };
    }
      cas_put($cas,$counter,$block_type,$block_len,$block_aux,$block_data);
      $counter++;
  }
  close(HEXFILE);
  print $counter-$oldcounter," HEX blocks slurped from file $filename.\n";
  return OK;
}


sub hex_spit {

#Write HEX from memory to disk.
#
#Arguments:
#  filename - name of file to write to
#  cas - a hash containing HEX data
#
#Return value:
#  OK if success
#  NOT_GOOD if failure

  my $filename=$_[0];
  my $cas=$_[1];
  my $blocks;
  my $keys;

  my $factor=$OPTIONS{'samplerate'}/$PWMS{'samplerate'}  if $OPTIONS{'turbo'} ne '';
  my $component=($OPTIONS{'samplerate'} < $PWMS{'samplerate'})*0.5  if $OPTIONS{'turbo'} ne '';

  $filename=findofile($filename);

  open(HEXFILE,">$filename") or die "Can't open HEX file $filename for writting\n";
  print HEXFILE "A8CAS-HEX\n";
  my $i=0;
  my $datablock=0;
  for ($keys=0;$keys < keys %{$cas};$i++) {
    $keys++;
    next if not defined ${$cas}{$i};
    $blocks++;
    CASE: for (${$cas}{$i}{'type'}) {
      /FUJI/ && do {
		      print HEXFILE "$_ ";
		      print HEXFILE ${$cas}{$i}{'data'} if ${$cas}{$i}{'length'}>0;
		      print HEXFILE "\n";
		      last CASE;
		   };
      /baud/ && do {
		      print HEXFILE "$_ ";
		      printf HEXFILE "%05d\n",unpack("v",${$cas}{$i}{'aux'});
		      last CASE;
		   };
      /data/ && do {
		      print HEXFILE "$_ ";
		      printf HEXFILE "%05d",unpack("v",${$cas}{$i}{'aux'});
		      if (${$cas}{$i}{'length'}>0) {
			foreach (unpack("C*",${$cas}{$i}{'data'})) {
			  printf HEXFILE " %02x",$_;
			}
			print HEXFILE " ; record=",++$datablock;
			print HEXFILE " ; length=",${$cas}{$i}{'length'};
			printf HEXFILE " ; checksum=%02x ",${$cas}{$i}{'cksum'};
			if (${$cas}{$i}{'cksumok'}) {
			  print HEXFILE "OK\n";
			}  else {
			  print HEXFILE "BAD\n";
			}
		      } else {
			printf HEXFILE " ; checksum=0 OK\n";
		      }
		      last CASE;
		   };
      /fsk / && do {
		      print HEXFILE "$_ ";
		      printf HEXFILE "%05d",unpack("v",${$cas}{$i}{'aux'});
		      if (${$cas}{$i}{'length'}>0) {
			foreach (unpack("v*",${$cas}{$i}{'data'})) {
			  printf HEXFILE " %d",$_;
			}
		      }
		      print HEXFILE " ; states=",${$cas}{$i}{'length'}/2;
		      print HEXFILE "\n";
		      last CASE;
		   };
      /pwms/ && do {
		      print HEXFILE "$_ ";
		      print HEXFILE (unpack('v',${$cas}{$i}{'aux'}) & 0x4 ? "msb_first" : "lsb_first")," ";
		      print HEXFILE (unpack('v',${$cas}{$i}{'aux'}) & 0x2 ? "rising_edge_first" : "falling_edge_first")," ";
		      if ($OPTIONS{'resamplecas'}) {
			print HEXFILE $OPTIONS{'samplerate'},"\n";
		      } else {
			my $rate;
			my @rate_tab=split('',${$cas}{$i}{'data'});
			for (my $i=0; $i <= $#rate_tab; $i++) {
				$rate += (ord($rate_tab[$i]) << ($i*8));
			}
			print HEXFILE $rate,"\n";
		      }
		      last CASE;
		   };
      /pwml/ && do {
		      print HEXFILE "$_ ";
		      printf HEXFILE "%05d ",unpack("v",${$cas}{$i}{'aux'});
		      if (${$cas}{$i}{'length'}>0) {
			foreach (unpack("v*",${$cas}{$i}{'data'})) {
			  $_=int($_*$factor+$component) if $OPTIONS{'resamplecas'};
			  printf HEXFILE "%d ",$_;
			}
		      }
		      print HEXFILE "; count=",${$cas}{$i}{'length'}/2;
		      print HEXFILE "\n";
		      last CASE;
		   };
      /pwmc/ && do {
		      print HEXFILE "$_ ";
		      printf HEXFILE "%05d ",unpack("v",${$cas}{$i}{'aux'});
		      if (${$cas}{$i}{'length'}>0) {
			my $j=1;
			foreach (unpack("(Cv)*",${$cas}{$i}{'data'})) {
			  $_=int($_*$factor+$component) if $OPTIONS{'resamplecas'} and ($j++ % 2);
			  printf HEXFILE "%d ",$_;
			}
		      }
		      print HEXFILE "; count=",${$cas}{$i}{'length'}/3;
		      print HEXFILE "\n";
		      last CASE;
		   };
      /pwmd/ && do {
		      print HEXFILE "$_";
		      foreach (unpack("CC",${$cas}{$i}{'aux'})) {
			$_=int($_*$factor+$component) if $OPTIONS{'resamplecas'};
			printf HEXFILE " %d",$_;
		      }
		      if (${$cas}{$i}{'length'}>0)  {
			foreach (unpack("C*",${$cas}{$i}{'data'})) {
			  printf HEXFILE " %02x",$_;
			}
		      }
		      print HEXFILE " ; block no=",++$datablock;
		      print HEXFILE " ; length=",${$cas}{$i}{'length'};
		      $OPTIONS{'cksumtype'}=${$cas}{$i}{'cksumtype'};
		      $OPTIONS{'turbo'}=${$cas}{$i}{'turbo'};
		      printf HEXFILE " ; checksum($OPTIONS{'cksumtype'})=%02x ",${$cas}{$i}{'cksum'};
		      eval('prepare_' .  lc $OPTIONS{'turbo'} . '()');
		      eval('analyse_' .  lc $OPTIONS{'turbo'} . '(\'cksum\',${$cas}{$i}{\'data\'})');
		      if ($OPTIONS{'cksumsane'})  {
			if (${$cas}{$i}{'cksumok'}) {
			  print HEXFILE "OK\n";
			}  else {
			  print HEXFILE "BAD\n";
			}
		      } else {
			print HEXFILE "UNK\n";
		      }
		      last CASE;
		   };
    }
  }
  print "$blocks HEX blocks stored in file $filename.\n";
  close(HEXFILE) or die "Can't close output file $filename properly!\n";
return OK;
}

sub bin_slurp {

#Arguments:
# - input file name
# - a reference to cas hash
# - length of pilot signal
#Returns:
# OK if OK, NOT_GOOD if error

  my $filename=$_[0];
  my $cas=$_[1];
  my $fskpilot=($_[2] or 0);
  my $blocks=(keys %{$cas})-2;
  my $buffer='';
  my $block='';
  my $title;

  if ($OPTIONS{'turbo'} ne '') {
    eval('prepare_' . lc $OPTIONS{'turbo'} .  '()');
  } else {
    prepare_normal();
  }
  open(BINFILE,"<$filename") or die "Can't open BIN file $filename for reading\n";
  my $got=sysread(BINFILE,$buffer,131072) or die "Can't read from BIN file $filename\n";
  close(BINFILE);
  die "What on earth are You converting ? It's too big!" if $got==131072;
  $title=($OPTIONS{'title'} or $filename);
  if ($OPTIONS{'turbo'} ne '') {
    eval('paramcorrect_' . lc $OPTIONS{'turbo'} . '()');
    eval('infoblock_' . lc $OPTIONS{'turbo'} . '($cas,$blocks,$title)');
    eval('makedata_' . lc $OPTIONS{'turbo'} . '($buffer,$block)');
    eval('datablock_' .lc $OPTIONS{'turbo'} . '($cas,$blocks,$block,2)');
    while (length($buffer)>0) {
      eval('paramcorrect_' . lc $OPTIONS{'turbo'} . '()');
      eval('makedata_' . lc $OPTIONS{'turbo'} . '($buffer,$block)');
      eval('datablock_' .lc $OPTIONS{'turbo'} . '($cas,$blocks,$block,0)');
    }
  } else {
    infoblock_normal($cas,$blocks,$title);
    makedata_normal($buffer,$block);
    datablock_normal($cas,$blocks,$block,254,$fskpilot);
    while (length($buffer)>0) {
      makedata_normal($buffer,$block);
      datablock_normal($cas,$blocks,$block,0,0);
    }
    $block=pack('C128',0);
    datablock_normal($cas,$blocks,$block,255,0); #eof block
  }
  1;
}

sub bin_spit {

#Arguments:
# - output program name
# - a reference to cas hash
#Returns:
# OK if sucess
# NOT_GOOD if failure

  my $progname=$_[0];
  my $cas=$_[1];
  my $blocks=0;
  my $keys;
  my $programs=1;
  my $files=1;
  my $filename;

  my $i=0;
  (my $filenamebase=$progname)=~s/((.*\.)*)\.(.*$)/$1/;
  (my $output_ext=$progname)=~s/.*\.(....?)$/$1/;

  for ($keys=0;$keys < keys %{$cas};$i++) {
    $keys++;
    next if not defined ${$cas}{$i};
    # skip, if there is EOF block in standard format ...
    if (${$cas}{$i}{'type'} eq 'data'
	and ord(substr(${$cas}{$i}{'data'},2,1)) == 0xFE)  {
      $files++;
      next;
    }
    # ... or when record in turbo format is short... (this is highly experimental)
#    if (${$cas}{$i}{'type'} eq 'pwmd' DONT DO THIS!
#	and ${$cas}{$i}{'length'} < 1024)  {
#      next;
#    }
    #$a=keys %{${$cas}{'programs'}{$programs}{'files'}};
    #print "files=",$a," i=$i\n";
    if ($files<=keys %{${$cas}{'programs'}{$programs}{'files'}}) {
      #print "start of file=$files\n";
      #print "file starts at: ",${$cas}{'programs'}{$programs}{'files'}{$files},"\n";
      if (${$cas}{'programs'}{$programs}{'files'}{$files}==$i) {
	#print "starting file\n";
	#close(BINFILE) if defined(BINFILE);
	$blocks=0;
	$filename=$filenamebase . "-file" . $files;
	$files++;
      }
    }
    if (${$cas}{$i}{'type'}=~m/(^data$)|(^pwmd$)/) {
      # where the pure data start...
      my $offset;
      if (${$cas}{$i}{'type'} eq 'data') {
	$offset=(${$cas}{$i}{'length'}==132) ? 3 : 2;
      } else {
	 $offset = ($OPTIONS{'turbo'}=~m/blizzard|turbo2000/i) ? 2 : 0;
      }
      # ... and how many bytes we want
      my $len;
      if (${$cas}{$i}{'type'} eq 'data') {
	  $len=(${$cas}{$i}{'length'}==132) ? length(${$cas}{$i}{'data'})-4 : length(${$cas}{$i}{'data'})-3;
      } else {
	$len = ($OPTIONS{'turbo'}=~m/blizzard|turbo2000/i) ? unpack('v',substr(${$cas}{$i}{'data'},0,2))
						 :  length(${$cas}{$i}{'data'});
      }
      if  ($blocks==0) {
	CASE: for (unpack('v',substr(${$cas}{$i}{'data'},$offset,2))) {
	  $_==65535 && do {
			    $filename.='.xex';
			    last CASE;
			  };
	  $filename.=".$output_ext";
	}
	$filename=findofile($filename);
	open(BINFILE,">$filename") or die "Can't open BIN file $filename for writting\n";
      }
      print BINFILE substr(${$cas}{$i}{'data'},$offset,$len);
      $blocks++;
    }
  }
  close(BINFILE);
}

#
#Here starts block responsible for cas->wav conversion
#

sub cas2sound {

  #Locally global variables
  our($baud,$FSKPWM);

  use constant {
      FSKWRITTEN => -2,
      RECORD_TOO_SHORT => -3,
      BAUD_SET => OK,
      PWMS_SET => OK,
      };

  #Subroutines

  sub open_sound_out {

  #  Opens sound file
  #  Starts ecasound processing and directs its output to pipe
  #
    my $command="";
    my $samplerate;

    my $outfile=$_[0];

    my $fifo='/tmp/a8cas-fifo.raw';

    $outfile=findofile($outfile);

    if ($OPTIONS{'left'} ne "") {				# want to add left channel
      # check samplerate ...
      $command="ecalength -s -f -r \Q$OPTIONS{'left'}\E 2>/dev/null";
      open($INFILE,"$command|") or die "Can't determine samplerate of input file\n";
      defined($samplerate=<$INFILE>) or die "Can't determine samplerate of input file\n";
      chomp($samplerate);
      close($INFILE);
      $OPTIONS{'samplerate'}=$samplerate;
      # ...and number of channels in input file
      $command="ecalength -s -f -c \Q$OPTIONS{'left'}\E 2>/dev/null";
      open($INFILE,"$command|") or die "Can't determine number of input channels.\n";
      my $channels;
      defined($channels=<$INFILE>) or die "Can't determine number of input channels.\n";
      chomp($channels);
      close($INFILE);
    }

    unless (-p $fifo)  {
      mkfifo($fifo, 0700) or die "mkfifo $fifo failed: $!";
    }

    $ENV{'MALLOC_CHECK_'}=0;
    $ENV{'ECASOUND'}='ecasound';

    print "Starting ecasound... ";

    defined(my $pid=fork()) or die "Can't fork: $!\n";
    unless ($pid) {
      eci("cs-add a8cas
	   -z:mixmode,sum
	   c-remove default
	   cs-set-audio-format u8,1,$OPTIONS{'samplerate'}
	   c-add fskdata,pwmdata
	   -i:$fifo
	  ");
      eci("c-select fskdata");
      if ($OPTIONS{'squarify'}) {
	eci("cop-add -ea:".$OPTIONS{'gain'}*100);
      } else {
	eci("cop-add -el:range_trans_cr,-1,1,$OPTIONS{'space'},$OPTIONS{'mark'}
	    cop-add -el:sine_faac,$OPTIONS{'gain'}
	   ");
      }
      if ($OPTIONS{'left'}) {							# want add left channel
	eci("c-add ssound
	      -i:\"$OPTIONS{'left'}\"
	    ");
      }
      if ($OPTIONS{'mono'} and ($OPTIONS{'left'} eq '')) {
	eci("cs-set-audio-format s16_le,1,$OPTIONS{'samplerate'}");		# making mono track
      } else {
	eci("c-select fskdata");
	eci("cs-set-audio-format s16_le,2,$OPTIONS{'samplerate'}");
	eci("cop-add -chmove:1,2");						# move to right channel
      }
      if ($OPTIONS{'left'} ne '') {						# want add left channel
	eci("c-select ssound
	cs-set-audio-format s16_le,2,$OPTIONS{'samplerate'}
	cop-add -chmute:2
	c-select ssound,fskdata
	");
      }
      eci("-o:loop,mixstd
	  c-add mixstd
	  -i:loop,mixstd
	  ");
      eci("c-select fskdata
	  c-mute");								#mute std chain by default
      eci("c-select pwmdata");
	  if ($OPTIONS{'mono'} and ($OPTIONS{'left'} eq '')) {
	    eci("cs-set-audio-format s16_le,1,$OPTIONS{'samplerate'}");			# making mono track
	  } else {
	    eci("cs-set-audio-format s16_le,2,$OPTIONS{'samplerate'}
		  cop-add chmove:1,2");							# move to right channel
	  }
      eci("cop-add -ea:" . $OPTIONS{'gain'}*100
	 );
      eci("cop-add -el:inv") if $OPTIONS{'invert'};
      if ($OPTIONS{'shape'} ne 'square') {
	CASE: {
	  if ($OPTIONS{'shape'} eq 'sine') {
	    eci("cop-add -el:bandpass_iir,200,6500,1");
	    last CASE;
	  }
	  if ($OPTIONS{'shape'} eq 'sawtooth') {
	    eci("cop-add -efb:20,2000");
	    last CASE;
	  }
	}
      }
      eci("c-mute");								#mute turbo chain by default
      eci("c-select mixstd,pwmdata");
      if ($OPTIONS{'mono'} and ($OPTIONS{'left'} eq '')) {
	eci("cs-set-audio-format u8,1,$OPTIONS{'samplerate'}");
      } else {
	eci("cs-set-audio-format u8,2,$OPTIONS{'samplerate'}");
      }
      eci("-o:\"$outfile\"
	  ");
      eci("engine-launch");
      until (eci("engine-status") =~ m/(stopped)/)  {
	sleep 1;
      }
      unlink('/tmp/.a8stop'); #just in case it was there...
      eci("start");
      until (eci("engine-status") =~ m/(running)/)  {
	sleep 1;
      }
      on_error("die");
      until (-e '/tmp/.a8stop')  {
	sleep 1;
      }
      until (eci("engine-status") =~ m/(finished)/)  {
	sleep 1;
      }
      eci("quit");
      unlink('/tmp/.a8stop');
      exit 1;
    } else  {
      open($OUTFILE,">$fifo");
      $FSKPWM='NONE';
      sleep 1;
#      print "\nparent: "  . eci("engine-status") .  "\n";
      while (eci("engine-status") ne "running")  {
#	print "\nparent: "  . eci("engine-status") .  "\n";
	sleep 1;
      }
#      print "parent: "  . eci("engine-status") .  "\n";
      print "Started.\n";
    }
    1;
  }

  sub close_sound_out() {
    #Closing my end
    $OUTFILE->flush();
    close($OUTFILE) or die "Can't close output stream.";
    #Touch
    open A8STOP, ">>/tmp/.a8stop" or die "Can't touch /tmp/.a8stop !\n";
    close A8STOP;
    while ( -e '/tmp/.a8stop')  {
      sleep 1;
    }
    unlink('/tmp/a8cas-fifo.raw') or warn "Failed to delete FIFO: $!";
    1;
  }

  sub outtail {		# outputs mark/silence 
			# at the end of tape
      my $i;
      my $length=($_[0] or 10000);
      my $what=($_[1] or $FSKPWM);

      for($i=0;$i<=$length;$i++) {
	  print $OUTFILE ($what eq 'PWM' ? NEUTRAL : MARK) or die "Can't write to output file (tail).";
      }
  }

  sub fskpwm {

  # switch to FSK or PWM channel
  # Arguments:
  #   - 'FSK', 'PWM' or 'NONE';

    return 0 if $FSKPWM eq $_[0];

    # finish FSK/PWM part nicely (flush buffer and wait for ecasound)
    $OUTFILE->flush;
    sleep(1);

    my $chanunmute=($_[0] eq 'FSK' ? 'fskdata' : 'pwmdata');
    my $chanmute=($_[0] eq 'FSK' ? 'pwmdata' : 'fskdata');

    if ($FSKPWM ne 'NONE') {
      eci("
	  c-select $chanmute
	  c-mute
	  ");
    }

    if ($FSKPWM ne $_[0]) {
      eci("
	  c-select $chanunmute
	  c-muting
	  ");
    }
    $FSKPWM=$_[0];
    1;
  }

  sub outrecord {

      my ($block_type,$block_len,$block_aux,$block_data);
      #my $recordlen;
      my $irglen;
      my $fsklen;
      my ($i,$j);
      my @byte;
      my $bitlen;
      my $lastbaud;
      my $cas=$_[0];
      my $block_no=$_[1];

      $lastbaud=$baud;
      cas_get($cas,$block_no,$block_type,$block_len,$block_aux,$block_data);
      #print "\n",$block_no,",",$block_type,",",$block_len,"\n";

      if ($block_type eq 'baud') {
	  die "Bad data in baud block" if $block_len!=0;
	  if ($OPTIONS{'forcebaud'}==-1) {
	      $baud=int(unpack("v",$block_aux)*$OPTIONS{'baudmultiplier'});
	  } else {
	      $baud=$OPTIONS{'forcebaud'};
	  }
	  return BAUD_SET;
      }
      if ($block_type eq 'fsk ') {				# "long zero" (usually)
	  fskpwm('FSK');
	  $irglen=int(unpack('v',$block_aux)/1000*$OPTIONS{'samplerate'});	# length of irg
	  for($i=1;$i<=$irglen;$i++) {
	      print $OUTFILE ONEBIT or die "Can't write irg to output file.";
	  }
	  for ($j=1;$j<=$block_len/2;$j++) {
	    $fsklen=int(unpack('v',(substr($block_data,0,2,'')))/10000*$OPTIONS{'samplerate'});	# length of fsk impulse
	    for($i=1;$i<=$fsklen;$i++) {
		print $OUTFILE (($j % 2) ? ZEROBIT : ONEBIT) or die "Can't write zero to output file.";
	    }
	  }
	  return FSKWRITTEN;
      }
      if ($block_type eq 'data') {
	  fskpwm('FSK');
	  # record of data
	  $irglen=int(unpack('v',$block_aux)/1000*$OPTIONS{'samplerate'});	# length of IRG

	  if (($OPTIONS{'usefirstbaud'}==1 and abs($baud-$lastbaud)<30)) {
	      $baud=$lastbaud;
	  }

	  my $realbitlen=$OPTIONS{'samplerate'}/$baud;
	  $bitlen=int($realbitlen+0.5);
	  my $lendiff=$realbitlen-$bitlen;
	  my $countdiff=0;
	  my $correctlen=0;

	  print $OUTFILE ONEBIT x $irglen or die "Can't write irg to output file.";

	  if ($block_len < $OPTIONS{'skipshorter'}) {			# write additional IRG fragment in place of skipped bytes
	      print $OUTFILE ONEBIT x int($block_len*10*$bitlen*(1+abs($lendiff))+0.5)  or die "Can't write irg to output file.";
	      return RECORD_TOO_SHORT;
	  }

	  my $bytecounter=0;

	  foreach my $byte_ (split('',$block_data)) {
	      @byte=split('',unpack("b8",$byte_));
	      print $OUTFILE START x ($bitlen+$correctlen) or die "Can't write start bit to output file.";   # start bit
	      $correctlen=0;
	      $countdiff+=$lendiff;
	      if ($countdiff > 0.5 or $countdiff < -0.5) {
		#print "adjusting START: countdiff=$countdiff\n\n";
	        $correctlen=($lendiff > 0 ? 1 : -1);
	        $countdiff-=$correctlen;
	      }
	      foreach my $bit (@byte) {
		  print $OUTFILE chr($bit*0xFF) x ($bitlen+$correctlen) or die "Can't write data bit to output file."; # data bits
		  $correctlen=0;
		  $countdiff+=$lendiff;
		  if ($countdiff > 0.5 or $countdiff < -0.5) {
		    #print "adjusting BYTE: countdiff=$countdiff\n\n";
		    $correctlen=($lendiff > 0 ? 1 : -1);
		    $countdiff-=$correctlen;
		  }
	      }
	      if (++$bytecounter == $block_len) {
		# correct length of last stop bit of last byte in record
		$countdiff+=$lendiff;
		if ($countdiff > 0.5 or $countdiff < -0.5) {
		  #print "adjusting STOP: countdiff=$countdiff\n\n";
	          $correctlen+=($lendiff > 0 ? 1 : -1);
	          $countdiff-=$correctlen;
		}
	      }
	      print $OUTFILE STOP x ($bitlen+$correctlen) or die "Can't write stop bit to output file.";   # stop bit
	      $correctlen=0;
	      $countdiff+=$lendiff;
	      if ($countdiff > 0.5 or $countdiff < -0.5) {
		#print "adjusting STOP: countdiff=$countdiff\n\n";
	        $correctlen=($lendiff > 0 ? 1 : -1);
	        $countdiff-=$correctlen;
	      }
	  }
	  return $block_len;
      }
      if ($block_type eq 'pwms') {
	fskpwm('PWM');

	my ($aux1,$aux2)=unpack("CC",$block_aux);
	$PWMS{'iparams'}=$aux1;
	#$PWMS{'bparams'}=$aux2;
	$PWMS{'endian'}=($aux1 & 0b100) ? "B8" : "b8";
	$PWMS{'samplerate'}=unpack("v",substr($block_data,0,2,""));
	#$PWMS{'ZEROW'}=unpack("C",substr($block_data,0,1,""));
	#$PWMS{'ONEW'}=unpack("C",substr($block_data,0,1,""));
	return PWMS_SET;
      }

      if ($block_type eq 'pwmc')  {
	fskpwm('PWM');
	my $factor=$OPTIONS{'samplerate'}/$PWMS{'samplerate'};
	my $component=($OPTIONS{'samplerate'} > $PWMS{'samplerate'})*0.5;

	$irglen=int(unpack("v",$block_aux)/1000*$OPTIONS{'samplerate'}*$factor+0.5);	# length of silence
	# "silence"
	for($i=0;$i<$irglen;$i++) {
	    print $OUTFILE NEUTRAL or die "Can't write irg to output file.";
	}

	my $firsthalf=($PWMS{'iparams'} & 0b10) ? LEVELONE : LEVELZERO;
	my $secondhalf=($firsthalf eq LEVELZERO) ? LEVELONE : LEVELZERO;
	for (my $k=0;$k<$block_len/3;$k++) {
	  my $width=unpack("C",substr($block_data,0,1,''));
	  $width=int($width*$factor+$component);
	  my $cycles=unpack("v",substr($block_data,0,2,''));
	  my $temp=int($width/2);  # round down!

	  for ($i=0;$i<$cycles;$i++) {
	    #first half of impulse
	    for ($j=0;$j<$temp;$j++) {
	      print $OUTFILE $firsthalf
		or die "Can't write data to output.";
	    }
	    #second half of impulse
	    for ($j=0;$j<$temp;$j++) {
	      print $OUTFILE $secondhalf
		or die "Can't write data to output.";
	    }
	    #one samlpe of neutral state if impulse width was odd
	    if (2*$temp < $width) {
	      print $OUTFILE NEUTRAL;
	    }
	  }
	}
      }

      if ($block_type eq 'pwmd') {
	return OK if $block_len==0;
	fskpwm('PWM');
	my $factor=$OPTIONS{'samplerate'}/$PWMS{'samplerate'};
	my $component=($OPTIONS{'samplerate'} > $PWMS{'samplerate'})*0.5;

	my $firsthalf=($PWMS{'iparams'} & 0b10) ? LEVELONE : LEVELZERO;
	my $secondhalf=($firsthalf eq LEVELZERO) ? LEVELONE : LEVELZERO;

	#print "\n",$PWMS{'samplerate'},$OPTIONS{'samplerate'};
	($PWMS{'ZEROW'},$PWMS{'ONEW'})=unpack("CC",$block_aux);
	$PWMS{'ZEROW'}=int($PWMS{'ZEROW'}*$factor+$component);
	$PWMS{'ONEW'}=int($PWMS{'ONEW'}*$factor+$component);

	# data
	foreach my $byte_ (split('',$block_data)) {
	    @byte=split('',unpack($PWMS{'endian'},$byte_));
	      foreach my $bit (@byte) {
		$bit=($bit ? $PWMS{'ONEW'} : $PWMS{'ZEROW'});
		my $temp=int($bit/2);  # round down!
		#first half of impulse
		for ($j=0;$j<$temp;$j++) {
		  print $OUTFILE $firsthalf
		    or die "Can't write data to output.";
		}
		#second half of impulse
		for ($j=0;$j<$temp;$j++) {
		  print $OUTFILE $secondhalf
		    or die "Can't write data to output.";
		}
		#one samlpe of neutral state if impulse width was odd
		if (2*$temp < $bit) {
		  print $OUTFILE NEUTRAL;
		}
	      }
	}
      }
      if ($block_type eq 'pwml') {
	fskpwm('PWM');
	my $factor=$OPTIONS{'samplerate'}/$PWMS{'samplerate'};
	my $component=($OPTIONS{'samplerate'} > $PWMS{'samplerate'})*0.5;

	$irglen=int(unpack("v",$block_aux)/1000*$OPTIONS{'samplerate'}*$factor+0.5);	# length of silence
	# "silence"
	for($i=0;$i<$irglen;$i++) {
	    print $OUTFILE NEUTRAL or die "Can't write irg to output file.";
	}

	my $firsthalf=($PWMS{'iparams'} & 0b10) ? LEVELONE : LEVELZERO;
	my $secondhalf=($firsthalf eq LEVELZERO) ? LEVELONE : LEVELZERO;

	for ($i=0;$i<$block_len/2;$i++)  {
	  my $temp=unpack("v",substr($block_data,0,2,''));
	  $temp=int($temp*$factor+$component);
	  for ($j=0;$j<$temp;$j++) {
	    print $OUTFILE ($i % 2) ? $secondhalf : $firsthalf
	      or die "Can't write data to output.";
	  }
	}
	return OK;
      }
      #die "Bad data in CAS file.";
      return OK;
  }

  ########################
  ### START PROCESSING ###
  ########################

  my $lastbaud=0;		# baud rate for last seen data block
  $baud=600;			# baud rate of current data block

  my $outfile=$_[0];
  my $cas=$_[1];			# hash storing cas data

  # save command line options for future reference
  my $optionsfile=$outfile . ".opt";

  $optionsfile=findofile($optionsfile);

  open(OPTIONSFILE,">$optionsfile") or print "Can't write to options file.";
  foreach (sort keys %OPTIONS) {
    print OPTIONSFILE "$_ => $OPTIONS{$_}\n";
  }
  close(OPTIONSFILE);

  pod2usage(1) if $OPTIONS{'help'};

#  pod2usage(1) if !$OPTIONS{'infile'};

  open_sound_out($outfile);

  my $records=0;
  my $status=1;
  my $rawfsk=0;
  my $nonstd=0;
  my $skippedshort=0;

  #STDOUT->autoflush(1);

  #print "Mastering started.\n";
  if ($OPTIONS{'turbo'} eq '') {
  print "Using ";
    if ($OPTIONS{'forcebaud'}==-1) {
      print "variable baud rate ";
      if ($OPTIONS{'usefirstbaud'}==1) {
	  print "(first significantly different found in cas file).\n";
      } else {
	  print "(as saved in cas file).\n";
      }
    } else {
	print "constant baud rate (specified: $OPTIONS{'forcebaud'})";
    }
  }
  #outheader($OPTIONS{'samplerate'});

  $status=1;

  for (keys %{$cas}) {
      next if !($_ =~ /^\d/);
      $status=outrecord($cas,$records);
      #last if $status==END_OF_FILE;
      if ($status==RECORD_TOO_SHORT) {
	  $skippedshort++;
	  next;
      }
      if ($OPTIONS{'turbo'} eq '' and (abs($baud-$lastbaud)>30)) {
	  print "Sample rate is $OPTIONS{'samplerate'}, baud rate for the block ";
	  print eval($records+1) . " and the following blocks will be around " . $baud . ".\n";
	  $lastbaud=$baud;
      }
      #print "$status\n";
      $records++ if $status>0 or $status==FSKWRITTEN;
      $rawfsk++ if $status==FSKWRITTEN;
      $nonstd++ if  ($status!=132) and ($status>1);
      print "Blocks written: $records",($nonstd>0 ? ", $nonstd non-standard" : ''),($rawfsk>0 ? " $rawfsk raw FSK" : ''),($skippedshort>0 ? "skipped $skippedshort blocks shorter than $OPTIONS{'skipshorter'} bytes" : '')," ... \r";
      STDOUT->flush();
  }

  outtail();

  print "Blocks written: $records",($nonstd>0 ? ", $nonstd non-standard" : ''),($rawfsk>0 ? " $rawfsk raw FSK" : ''),($skippedshort>0 ? "skipped $skippedshort blocks shorter than $OPTIONS{'skipshorter'} bytes" : '')," ... done.\n";
  STDOUT->flush();

  print "Finished.\n";

  close_sound_out();

  1;		# exit nicely

}

#
#Here starts block responsible for sound->cas conversion
#

sub sound2cas {

  #Locally global variables

  our($totalsamples, $cursor, $bitlen, $minbitlen, @abuff, $bufflen,
	    @casrecord, $irglen, $reclen, $longzero, $INFILE, $LOGFILE,
	    @LATCH, @DEJAVU, $NUMBER_OF_STATES);

#	      $STARTIMPULSES

  # constants
  use constant {
    MINBITLEN => 12,	# minimum bitlen for ~2600 baud (in samples, samplerate 44100)
    MAXBITLEN => 126,	# maximum bitlen for ~350 baud (in samples, samplerate 44100)
    };

  use constant {
    FSK_DETECTED => -5,
    LONGZERO_DETECTED => -6,
    REQ_BAUD_CHANGE => -7,
    IRG_DETECTED => -2,
    };

  # for turbo
  use constant {
    PILOT => 'pilot',
    GARBAGE => 'garbage',
    SYNCBIT => 'syncbit',
  };

  use constant {
    SLOPERISING => 1,
    SLOPEFALLING => -1,
    SLOPEPLATEAU => 0,
    LATCHSTATE => 0,	# $LATCH[0] is state
    LATCHSLOPE => 1,	# $LATCH[1] is slope
  };

  # sub subroutines

  sub fillbuffer8 {

#  This subroutine, among other benefits, allows reading input data from pipe

      my $inbuff="";
      my $got=0;
      my $mult=1;		#used to read 2x more during first call

      print $LOGFILE "In fillbuffer\n" if $DEBUG{'fillbuffer'};
      $mult=2 if $bufflen==0;
      for(my $i=1;$i<=4*$mult;$i++) {
	$got=sysread($INFILE,$inbuff,4096,length($inbuff));
	if (!$got and $i==1) {
 	  # for turbo only and turbo uses buffer16 $bufflen=push(@abuff,split('',NEUTRAL x 128)) if $OPTIONS{'turbo'} ne ''; #
	  return END_OF_FILE;
	}
	last if !$got;
	$totalsamples+=$got;
      }
      if ($bufflen > 0) {
	  if ($_[0]!=$cursor) {
	      $cursor-=HALFBUFFLEN;
	  }
	  $_[0]-=HALFBUFFLEN;
      }
      if ($mult==2) {
	#eliminate first 1024 bytes (samples) on first run (buffering effect)
	substr($inbuff,0,1024,'');
	$got=sysread($INFILE,$inbuff,1024,length($inbuff));
	return END_OF_FILE if !$got;
	$totalsamples+=$got-1024;
      }
      splice(@abuff,0,HALFBUFFLEN);
      $bufflen=push(@abuff,split('',$inbuff));
      normalizebuffer() if $OPTIONS{'mynorm'};
      # for debugging
      print $LOGFILE "got: ",length($inbuff)," bufflen: $bufflen, inbuff: ",length($inbuff),", cursor: $cursor,  total: $totalsamples\n" if $DEBUG{'fillbuffer'};
      return 1;
  }

  sub fillbuffer16 {

#  This subroutine, among other benefits, allows reading input data from pipe

      my $inbuff="";
      my $got=0;
      my $mult=1;		#used to read 2x more during first call

      $mult=2 if $bufflen==0;
      for(my $i=1;$i<=8*$mult;$i++) {
	$got=sysread($INFILE,$inbuff,4096,length($inbuff));
	if (!$got and $i==1) {
 	  $bufflen=push(@abuff,unpack('s*',chr(0) x 256));
	  return END_OF_FILE;
	}
	last if !$got;
	$totalsamples+=$got/2;
      }
      if ($bufflen > 0) {
	  if ($_[0]!=$cursor) {
	      $cursor-=HALFBUFFLEN;
	  }
	  $_[0]-=HALFBUFFLEN;
      }
      if ($mult==2) {
	#eliminate first 2048 bytes (1024 samples) on first run (buffering effect)
	substr($inbuff,0,2048,'');
	$got=sysread($INFILE,$inbuff,2048,length($inbuff));
	return END_OF_FILE if !$got;
	$totalsamples+=$got/2;
      }
      splice(@abuff,0,HALFBUFFLEN);
      $bufflen=push(@abuff,unpack('s*',$inbuff));
      if ($OPTIONS{'ln'}) {
        for (HALFBUFFLEN..$#abuff) {
	  $abuff[$_]=($abuff[$_] <=> 0) * log(abs($abuff[$_])) if $abuff[$_]!=0;
        }
      }
      # for debugging
      #print $LOGFILE "got: ",length($inbuff)," bufflen: $bufflen, inbuff: ",length($inbuff),", cursor: $cursor,  total: $totalsamples\n";
      return 1;
  }

  sub finddiff {
    #find minimal and maximal values in next part of buffer
    #$_[0] - numbr of samples to check

    use List::Util qw(min max);

    my $lcnt=$_[0];

    my $min=min @abuff[$cursor..$cursor+$lcnt];
    my $max=max @abuff[$cursor..$cursor+$lcnt];
    #one-third of aplitude is enough to detect edge
    return ($max-$min)/3;
  }

  sub latch {

  # Arguments:
  #  - maximum width of impulse (maxwidth)

    my $maxwidth=($_[0] or 200);
    my $counter=1;

    my $lcnt; #last counter
    if (!defined($DATASTORE{'lcnt'}) or $DATASTORE{'lcnt'}<$PWMS{'ahead'}) {
	$lcnt=$PWMS{'ahead'};
    } else {
	$lcnt=$DATASTORE{'lcnt'}+2; #half of last impulse + 2 samples
    }
    #print $LOGFILE "LCNT: lcnt=$lcnt, ahead=$PWMS{'ahead'}\n";
    #Need at least 30 samples ahead
    if ($cursor+$lcnt >= $bufflen) {
	return END_OF_FILE if fillbuffer16($cursor) < 0;
    }

    my $previous=$abuff[$cursor];

    my $currmindiff=$DATASTORE{'mindiff'};
    #Look ahead
    $currmindiff=finddiff($lcnt) if $DATASTORE{'dynmindiff'};

    $cursor++;
    if ($cursor >= $bufflen) {
	return END_OF_FILE if fillbuffer16($cursor) < 0;
    }
    while ($LATCH[LATCHSLOPE]*($previous-$abuff[$cursor])<$currmindiff and $counter<$maxwidth) {
	if ($LATCH[LATCHSLOPE]*($previous-$abuff[$cursor])<0) {
	    $previous=$abuff[$cursor];
	}
      $cursor++; $counter++;
      if ($cursor >= $bufflen) {
	  return END_OF_FILE if fillbuffer16($cursor) < 0;
      }
    }
    if ($counter>=$maxwidth) {
	print $LOGFILE "Leaving latch (counter! $counter): val=$abuff[$cursor], latch=$LATCH[LATCHSLOPE] at pos ",($totalsamples-$bufflen+$cursor)/2,"\n" if defined($DEBUG{'latch'});
	return -255;
    }
    $LATCH[LATCHSTATE]=1-$LATCH[LATCHSTATE];
    $LATCH[LATCHSLOPE]=-$LATCH[LATCHSLOPE];
    $DATASTORE{'lcnt'}=$counter;
    print $LOGFILE "Leaving latch: counter=$counter, val=$abuff[$cursor], latch=$LATCH[LATCHSLOPE] at pos ",($totalsamples-$bufflen+$cursor)/2,"\n" if defined($DEBUG{'latch'});
    return OK;
  }

  sub skippilot {

  # TURBO
  # Detect pilot
  # Arguments:
  #  none
  # Returns:
  #  - number of pilot cycles or error (pilotcycles)
  #  - number of pilot samples
  #  - number of 'start impulses'

      my $i=0;
      my $bit;
      my $tempbit;
      my $counter;		# sample counter
      my $skippedcounter=0;	# for saving skipped part of impulse (for example turborom)
      my $tempcursor;

      my ($firsthalf,$secondhalf);

      my $pilotcycles=0;
      my $pilotsamples=0;
      my $pilotdetected=0;
      my $start;

      $tempcursor=0;

      $DATASTORE{'dynmindiff'}=FALSE;
      $DATASTORE{'mindiff'}=($OPTIONS{'mindiff'} or ($OPTIONS{'ln'} ? log(6000) : 6000));
      delete($DATASTORE{'lcnt'});

      $start=$totalsamples-$bufflen+$cursor;
      print $LOGFILE "Looking for pilot at sample $start, time ",$start/96000+$OPTIONS{'starttime'}," s, cursor: $cursor\n" if defined($DEBUG{'skippilot'});
      # find first proper slope
      print $LOGFILE "before first slope: ",$LATCH[LATCHSLOPE],", state: ",$LATCH[LATCHSTATE],"\n" if defined($DEBUG{'skippilot'});
      return END_OF_FILE if latch(20000)==END_OF_FILE;
      while ($LATCH[LATCHSLOPE]!=int((0.5-($PWMS{'iparams'} & 0b01))*2)) {
	return END_OF_FILE if latch()==END_OF_FILE;
      }
      #return END_OF_FILE if latch()==END_OF_FILE;
      if ($PWMS{'PILOTW'}==0) {		#don't look for pilot
	$cursor-=2;
	return OK;
      }
      $tempcursor+=$totalsamples-$bufflen+$cursor-$start;
      $start=$totalsamples-$bufflen+$cursor;
      print $LOGFILE "found first slope: ",$LATCH[LATCHSLOPE],", state: ",$LATCH[LATCHSTATE],", start: $start, cursor: $cursor\n" if defined($DEBUG{'skippilot'});

      while (TRUE) {
	  return END_OF_FILE if latch()==END_OF_FILE;
	  $firsthalf=$totalsamples-$bufflen+$cursor-$start;
	  # check, whether count only one level
	  $start=$totalsamples-$bufflen+$cursor;
	  return END_OF_FILE if latch()==END_OF_FILE;
	  $secondhalf=$totalsamples-$bufflen+$cursor-$start;
	  $counter=($PWMS{'countlevels'}&0b01)*$secondhalf+($PWMS{'countlevels'}&0b10)/2*$firsthalf;
	  $skippedcounter=$firsthalf+$secondhalf-$counter;
	  print $LOGFILE "after pilot cycle (start=$start, counter=$counter, firsthalf=$firsthalf, secondhalf=$secondhalf)\n" if defined($DEBUG{'skippilot'});
	  if ($pilotcycles>=$PWMS{'pilotmincycles'} and $PWMS{'autoedgedetect'}) {
	    my $factor = ($PWMS{'countlevels'}==0b11) ? 0.5 : 1;
	    if ($firsthalf>=$SYNCmaxW*$factor
	    and $secondhalf<=$SYNCmaxW*$factor) {
	      print $LOGFILE "Edge switch - start=$start, firsthalf=$firsthalf, counter=$counter, secondhalf=$secondhalf, pilotminw=$PILOTminW, syncmaxw=$SYNCmaxW\n" if defined($DEBUG{'skippilot'});
	      $PWMS{'iparams'}=$PWMS{'iparams'} ^ 0b00000011;
	      $cursor=$cursor-$secondhalf;
	      $LATCH[LATCHSTATE]=1-$LATCH[LATCHSTATE];
	      $LATCH[LATCHSLOPE]=-$LATCH[LATCHSLOPE];
	      $start=$totalsamples-$bufflen+$cursor;
	      $OPTIONS{'invert'} = not $OPTIONS{'invert'};
	      print $LOGFILE "after edge switch (start=$start, firsthalf=$firsthalf,secondhalf=$secondhalf, slope=$LATCH[LATCHSLOPE])\n" if defined($DEBUG{'skippilot'});
	      next;	# start
	    }
	  }
	  # autocompensate if difference is minimal
	  if ($BASEWIDTH-$counter!=0 and abs($BASEWIDTH-$counter)<=$PWMS{'maxpilotjitter'}) {
	    print $LOGFILE "adjusting basewidth: $BASEWIDTH -> $counter\n" if defined($DEBUG{'skippilot'});
	    eval("prepare_" . $OPTIONS{'turbo'} . "($counter)");
	  }
	  print $LOGFILE "found next slope, counter is $counter.\n" if defined($DEBUG{'skippilot'});
	  $start=$totalsamples-$bufflen+$cursor;
	  $pilotcycles++;
	  $pilotsamples+=$counter;
	  $tempcursor+=$counter;
	  #reset cycles counter after impulse shorter than min or longer than max
	  if ((($counter>200 or ($SYNCmaxW<$PILOTminW and ($counter>$PILOTmaxW or $counter<$SYNCminW)) or ($SYNCminW>$PILOTmaxW and ($counter<$PILOTminW or $counter>$PILOTmaxW))) and $pilotcycles < $PWMS{'pilotmincycles'})) {
	    print $LOGFILE "resetting1 - pilotcycles=$pilotcycles, counter=$counter, pilotmaxw=$PILOTmaxW, syncminw=$SYNCminW\n" if defined($DEBUG{'skippilot'});
	    eval("prepare_" . $OPTIONS{'turbo'});
	    $pilotcycles=0;
	    $pilotsamples=0;
	    while ($LATCH[LATCHSLOPE]!=int((0.5-($PWMS{'iparams'} & 0b01))*2)) {
	      return END_OF_FILE if latch()==END_OF_FILE;
	    }
	  }
	  SWITCH: {
	      if (($SYNCmaxW<$PILOTminW and $counter>$SYNCmaxW and $counter<=$PILOTmaxW) or
		  ($SYNCminW>$PILOTmaxW and $counter>$PILOTminW and $counter<=$PILOTmaxW)) {
		    $bit=PILOT; $pilotdetected=1;
		    $DATASTORE{'dynmindiff'}=TRUE if $OPTIONS{'mindiff'}==0;
		    last SWITCH;
	      }
	      if ($counter>=$SYNCminW and $counter<=$SYNCmaxW) { $bit=SYNCBIT; last SWITCH; }
	      $bit=GARBAGE;
	  }
	  print $LOGFILE "$pilotcycles, $start, $counter, ",$bit," in pilot\n" if ($bit eq GARBAGE) and defined($DEBUG{'skippilot'});
	  if (($bit ne PILOT) and ($pilotcycles!=0) and ($bit ne GARBAGE)) {
	      if ($OPTIONS{'recovery'} and $pilotdetected==0 and $pilotcycles <= 1) {
		$pilotsamples=$BASEWIDTH;
		$pilotcycles=1;
		last;
	      }
	      if ($pilotsamples/$pilotcycles<$PILOTminW or
		  $pilotsamples/$pilotcycles>$PILOTmaxW) {
		  print $LOGFILE "resetting2 - pilotcycles=$pilotcycles, pilotsamples=$pilotsamples, pilotminw=$PILOTminW, pilotmaxw=$PILOTmaxW, basewidth=$BASEWIDTH, counter=$counter\n" if defined($DEBUG{'skippilot'});
		  $pilotdetected=0;
		  $pilotsamples=0;
		  $pilotcycles=0;
		  while ($LATCH[LATCHSLOPE]!=int((0.5-($PWMS{'iparams'} & 0b01))*2)) {
		    return END_OF_FILE if latch()==END_OF_FILE;
		  }
		  $start=$totalsamples-$bufflen+$cursor;
	      } else {
		  last if $pilotdetected==1 and $pilotcycles >= $PWMS{'pilotmincycles'};
	      }
	  } elsif ($bit eq GARBAGE) {
	    next if $counter<$SYNCminW;
	    last if $pilotdetected==1 and $pilotcycles >= $PWMS{'pilotmincycles'};
	  }
      }
      if ($bit ne GARBAGE) {
	# rewind first impulse
	$cursor-=($firsthalf+$secondhalf);
	$tempcursor-=($firsthalf+$secondhalf);
      } else {
	print $LOGFILE "Ending at garbage??!! counter=$counter\n";
      }
      print $LOGFILE "pilotcycles: $pilotcycles, pilotwidth: ",int($pilotsamples/$pilotcycles+0.5),"\n";
      print $LOGFILE "firsthalf: $firsthalf, secondhalf:  $secondhalf, counter: $counter, pos: ",$totalsamples-$bufflen+$cursor,"\n";
      $irglen+=$tempcursor-$pilotsamples;
      return ($pilotcycles,$pilotsamples);
  }

  sub skipfirst {

    # Returns:
    #  number of impulses found
    #  impulses in string representing bits

    my $counter=0;
    my $skippedcounter=0;
    my $impulses='';
    my $i;
    my $start=$totalsamples-$bufflen+$cursor;

    print $LOGFILE "first bit at sample: $start, time: ",$start/96000+$OPTIONS{'starttime'}," s \n";
    #print $LOGFILE "skipfirst first  slope: ",$LATCH[LATCHSLOPE],", first state: ",$LATCH[LATCHSTATE],"\n";
    return (0,'') if $PWMS{'firstbits'}==0;
    #skip additional impulses
    for ($i=1;$i<=$PWMS{'firstbits'};$i++) {
      return END_OF_FILE if latch()==END_OF_FILE;
      my $firsthalf=$totalsamples-$bufflen+$cursor-$start;
      $start=$totalsamples-$bufflen+$cursor;
      return END_OF_FILE if latch()==END_OF_FILE;
      my $secondhalf=$totalsamples-$bufflen+$cursor-$start;
      $counter=($PWMS{'countlevels'}&0b01)*$secondhalf+($PWMS{'countlevels'}&0b10)/2*$firsthalf;
      $skippedcounter=$firsthalf+$secondhalf-$counter;
      print $LOGFILE "skipfirst counter: $firsthalf + $secondhalf (counting $counter)\n";
      if ($counter>$SYNCmaxW) {
	print $LOGFILE "skipping counter>$SYNCmaxW\n";
	$cursor-=($counter+$skippedcounter);
	last;
      }
      $start=$totalsamples-$bufflen+$cursor;
      $impulses.='0' if $counter<=$ZEROmaxW;
      $impulses.='1' if $counter>=$ONEminW;
    }
    return ($i-1,$impulses);
  }

  sub skipend_pwm {

    # skip and count possible 'tail' signal
    # after the end obfdata block
    # Returns:
    #  status
    #  count of 0/1 cycles

    my $counter;
    my $skippedcounter=0;
    my $endcycles=0;
    my $endsamples=0;
    my $start=$totalsamples-$bufflen+$cursor;

    return (OK,0) if $PWMS{'ENDW'}==0;

    while (TRUE) {
      return END_OF_FILE if latch()==END_OF_FILE;
      my $firsthalf=$totalsamples-$bufflen+$cursor-$start;
      $start=$totalsamples-$bufflen+$cursor;
      return END_OF_FILE if latch()==END_OF_FILE;
      my $secondhalf=$totalsamples-$bufflen+$cursor-$start;
      $counter=($PWMS{'countlevels'}&0b01)*$secondhalf+($PWMS{'countlevels'}&0b10)/2*$firsthalf;
      $skippedcounter=$firsthalf+$secondhalf-$counter;
      $start=$totalsamples-$bufflen+$cursor;
      if ($counter>=$ENDminW and $counter<=$ENDmaxW) {
	$endcycles++;
	$endsamples+=$skippedcounter+$counter;
      } else {
	$cursor-=($counter+$skippedcounter);
	last;
      }
    }
    #print $LOGFILE "END - counter: $counter, cycles: $endcycles\n";
    return ($endsamples,$endcycles);
  }

  sub getbyte_pwm {

  # Arguments:
  #  byte - for storing decoded  byte
  # Returns
  #  status

    my $byte='';
    my $counter;
    my $skippedcounter=0;
    my $i;
    my $tempbit;
    my $casbyte;
    my $start=$totalsamples-$bufflen+$cursor;
    my $bit;
    my $status=OK;

    $i=0;

    my $LongerImpulse=($ZEROmaxW>$ONEmaxW ? $ZEROmaxW : $ONEmaxW);
    my $ShorterImpulse=($ZEROminW<$ONEminW ? $ZEROminW : $ONEminW);

    while($i++<8) {			#read 8 bits
      #print $LOGFILE "start: $start, i: $i, cursor: $cursor, bufflen: $bufflen\n";
      $counter=0;
      $start=$totalsamples-$bufflen+$cursor;
    LOOPSTART:
      $status=latch();
#      last if $status==END_OF_FILE;
      my $firsthalf=$totalsamples-$bufflen+$cursor-$start;
      $start=$totalsamples-$bufflen+$cursor;
      $status=latch();
#      last if $status==END_OF_FILE;
      my $secondhalf=$totalsamples-$bufflen+$cursor-$start;
      $counter+=$firsthalf if ($PWMS{'countlevels'}&0b10);
      $counter+=$secondhalf if ($PWMS{'countlevels'}&0b01);
      #print $LOGFILE "latchstate: $LATCH[LATCHSTATE], countlevels: $PWMS{'countlevels'}, first: $firsthalf, second: $secondhalf, counter: $counter\n";
      $skippedcounter=$firsthalf+$secondhalf-$counter;
      $start=$totalsamples-$bufflen+$cursor;
      #Skip obvious errors
      goto LOOPSTART if $counter<$ShorterImpulse and $skippedcounter<=$LongerImpulse;
      if ($OPTIONS{'autoadjust'}) {
	    SWITCH: {
		if ($counter>=$ONEminW and $counter<=$ONEmaxW) { $bit=ONEBIT; last SWITCH; }
		if ($counter>=$ZEROminW and $counter<=$ZEROmaxW) { $bit=ZEROBIT; last SWITCH; }
		$bit=GARBAGE;
	    }
	my $diff;
	if ($bit eq ZEROBIT) {
	  $diff=int(($counter-($ZEROminW+$ZEROmaxW)/2)/2+0.5);
	  $diff=($diff==0 ? 0 : $diff<0 ? -1 : 1);
	  #$ZEROminW+=$diff;
	  $ZEROmaxW+=$diff;
	  $ONEminW+=$diff;
	  $ONEmaxW+=$diff;
	}
	if ($bit eq ONEBIT) {
	  $diff=int(($counter-($ONEminW+$ONEmaxW)/2)/2+0.5);
	  $diff=($diff==0 ? 0 : $diff<0 ? -1 : 1);
	  #$ZEROminW+=$diff;
	  $ZEROmaxW+=$diff;
	  $ONEminW+=$diff;
	  $ONEmaxW+=$diff;
	}
	#print $LOGFILE "adjust ",$bit eq ZEROBIT ? "0" : "1"," by $diff at sample $start, zeromax: $ZEROmaxW, onemin: $ONEminW, counter was: $counter\n"
	#  if $bit ne GARBAGE;
      }
      SWITCH: {
	if ($counter>=$ONEminW and $counter<=$ONEmaxW) { $bit=ONEBIT; $byte.='1'; last SWITCH; }
	if ($counter>=$ZEROminW and $counter<=$ZEROmaxW) { $bit=ZEROBIT; $byte.='0'; last SWITCH; }
	$bit=GARBAGE;
      }
      print $LOGFILE "$bit in byte at sample $start, ($ZEROminW,$ZEROmaxW,$ONEminW,$ONEmaxW) firsthalf: $firsthalf, secondhalf: $secondhalf, counting: $counter\n" if $bit eq GARBAGE;
      print $LOGFILE "lastbits: ",$i-1,", it is ",unpack("C", pack($PWMS{'endian'},$byte)),"\n" if $counter > $LongerImpulse or $status==END_OF_FILE;
      if ($counter > $LongerImpulse or $skippedcounter > $LongerImpulse or ($bit eq GARBAGE)) {
	$cursor-=($counter+$skippedcounter);
	$_[0]=$byte;
	return IRG_DETECTED;
      }
      last if $status==END_OF_FILE and $cursor >= $bufflen;
    }
    #found a byte
    #print $LOGFILE "byte: $byte\n";
    $_[0]=pack($PWMS{'endian'},$byte); #convert bin to char
    return END_OF_FILE if $status==END_OF_FILE;
    return OK;
  }

  sub findbyte {
  # find first occurence of given byte
  # Arguments:
  #  - byte we search for
  # Returns:
  #  - status

    my $result;
    my $searching=shift;
    my $byte=chr(0);	# a trick - never will look for 0

    while ($byte ne $searching) {
	my $start=$totalsamples-$bufflen+$cursor;
	$result=getbyte_pwm($byte);
	return $result if ($result!=OK) and ($result!=IRG_DETECTED);
	if ($byte ne $searching) {
	    $cursor=$cursor-($totalsamples-$bufflen+$cursor-$start);
	    #skip one impulse
	    return END_OF_FILE if latch(2000)==END_OF_FILE;
	    return END_OF_FILE if latch(2000)==END_OF_FILE;
	}
    }
    return OK;
  }

  sub getblock_pwm {

  # get one record
  # Arguments:
  #  - reference to cas hash (cas)
  #  - number of block in cas until now (blocks)
  # Returns:
  # - status
  # - number of endcycles

  #  my $reclen=0;
    my $result=OK;
    my $impulses='';
    my $casdatabyte;
    my $basewidth;
    my ($cycles,$samples);
    my ($i,$j);
    my $cas=$_[0];
    #my $blocks=$_[1];

    @casrecord=();
    eval("prepare_" . lc $OPTIONS{'turbo'} . "()");
    ($result,$samples)=skippilot();
    return ($result,0) if (($result==END_OF_FILE) or ($result==NOT_GOOD));
    if ($PWMS{'PILOTW'}!=0) {	# is there any pilot at all ?
	$cycles=$result;
	$basewidth=int($samples/$cycles+0.5); # (what if zero cycles?)
	eval("prepare_" . lc $OPTIONS{'turbo'} . "($basewidth)");
    } else {
	$cycles=0;
    }
    $irglen=int($irglen/96+0.5);
    if ($irglen>0) {
      print $LOGFILE "IRG: $irglen ms.\n";
      cas_put($cas,$_[1],'pwmc',0,pack("v",$irglen));
      $_[1]++;
    }							# silence
    $irglen=0;
    #print $LOGFILE "pilotcycles: $cycles\n";
    my ($block_type,$block_len,$block_aux,$block_data);
    $block_type='pwmc';
    $block_len=0;
    $block_aux=pack("v",0);
    $block_data='';
    if ($cycles>0) {
      $block_len+=3;
      $block_data.=pack("C",int($PWMS{'PILOTW'}*$OPTIONS{'samplerate'}/96000+0.5)) .
		   pack("v",$cycles);
    }

    ($result,$impulses)=skipfirst();
    if ($OPTIONS{'turbo'}=~m/6000/) {
	$result=findbyte(chr(2));
    }
    return ($result,0) if (($result==END_OF_FILE) or ($result==NOT_GOOD));
    eval('analyse_' . lc $OPTIONS{'turbo'} . '(\'first\',$result,$impulses)');
    for ($i=0;$i<$result;$i++) {
      $j=1;
      my $impulse=substr($impulses,0,1,'');
      while (substr($impulses,0,1) eq $impulse and $i<=$result) {
	$i++; $j++;
	$impulse=substr($impulses,0,1,'');
      }
      $block_len+=3;
      $block_data.=pack("C",int(($impulse ? $PWMS{'ONEW'} : $PWMS{'ZEROW'})*$OPTIONS{'samplerate'}/96000+0.5)) .
		  pack("v",$j);
    }
    if ($block_len>0) {
      cas_put($cas,$_[1],$block_type,$block_len,		# pilot and first impulses
	      $block_aux,
	      $block_data);
      $_[1]++;
    }
    while (TRUE) {
#	print $LOGFILE "getbyte\n";
	$casdatabyte='';
	$result=getbyte_pwm($casdatabyte);
	$casrecord[$#casrecord+1]=$casdatabyte;	# contains lastbits if before irg
	$reclen++;
	last if ($result==IRG_DETECTED);
	return ($result,0) if $result==END_OF_FILE;
    }
    ($result,$cycles)=skipend_pwm();
    return ($result,$cycles); # return
  }

  sub getlevels_pwm {

    my $result;
    my $start;

    my $maxlatch=int(65535*96000/$OPTIONS{'samplerate'}+0.5);
    my $found=FALSE;

    @casrecord=();
    while (TRUE)  {
	$start=$totalsamples-$bufflen+$cursor;
	$result=latch($maxlatch);
#	if ($#casrecord<0 and ($result==-255 or $result==END_OF_FILE)) {
#	    $irglen+=int(($totalsamples-$bufflen+$cursor-$start)/96000*1000+0.5);
#	    last if $result==END_OF_FILE;
#	    next;
#	}
	if ($#casrecord<0 and not $found) {
	    $found=TRUE if $result!=-255;
	    if (not $found or $irglen>0) {
		$irglen+=int(($totalsamples-$bufflen+$cursor-$start)/96000*1000+0.5);
		last if $result==END_OF_FILE;
		next;
	    }
	}
	$casrecord[$#casrecord+1]=int(($totalsamples-$bufflen+$cursor-$start)*$OPTIONS{'samplerate'}/96000+0.5);
	last if $result==-255;
	last if $result==END_OF_FILE;
	if ($#casrecord==32765) {	#finish when number of levels is even, next pwml will start normally
	    $result=OK;
	    last;
	}
    }
    return IRG_DETECTED if $result==-255;
    return $result;
  }

#####################
## FSK subroutines ##
#####################

use constant {
      GET_MILISECONDS  => FALSE,
      GET_SAMPLES => TRUE,
      LIMIT_STATES => FALSE,
      LIMIT_SAMPLES =>  TRUE,
    };

  sub getfsk {
  # get "raw" fsk data until next IRG
  # Arguments:
  #  - limit of data to get (optional), -1 for unlimited
  #  - flag telling, what to return: lenghth in samples or tenths of ms (optional)
  #  - flag telling, whether limit is in fsk states or samples
  # Returns:
  #  status and an array containing lengths of space/mark signals

    my $limit=($_[0] or ($OPTIONS{'minfsk'} ? 2 : 200000));		# get states until irg or eof or only as much we want
    my $return_samples_or_ms=($_[1] or GET_MILISECONDS);	# return tenths of miliseconds or samples
    my $limit_states_or_samples=($_[2] or LIMIT_STATES); # limit is in states or in samples
    my $fskcounter=0;
    my $fskdata='';
    my $bit;
    my $longzero=FALSE;
    my $result=0;

    $limit-- if $limit;

    LOOP: while ($limit>=0) {
      return (END_OF_FILE,$fskdata) if $result==END_OF_FILE;
      $bit=$abuff[$cursor];
      print $LOGFILE "Counting state: " . unpack('H2',$bit) .", counter=$fskcounter, limit=$limit\n" if defined($DEBUG{'getfsk'});
      while ($bit eq $abuff[$cursor]) {
#	print $LOGFILE "cursor: $cursor\n";
	if ($fskcounter>11*$bitlen+$minbitlen) {
	  print $LOGFILE "irg: counter=$fskcounter, limit=$limit\n" if defined($DEBUG{'getfsk'});
	  if ($return_samples_or_ms==GET_MILISECONDS) {
	    # we are getting raw fsk, don't store beginning of irg or long zero
	    $cursor=$cursor-$fskcounter;
	  }  else {
	    # irg detected - get stop bit, return the rest to buffer
	    $fskcounter+=$limit;
	    $cursor+=$limit;
	    if ($cursor >= $bufflen) {
	      if (fillbuffer8($cursor) < 0) {
		$fskcounter-=$limit;
		$cursor-=$limit;
		return (END_OF_FILE,$fskdata);
	      }
	    }
	    $fskdata.=' ' if $fskdata;
	    $fskdata.=($return_samples_or_ms ? $fskcounter : int($fskcounter/$OPTIONS{'samplerate'}*10000));
	  }
	  return (($longzero ? LONGZERO_DETECTED : IRG_DETECTED),$fskdata);
	}
	if ($longzero==FALSE and ($bit eq ZEROBIT) and ($fskcounter>10*$bitlen)) {
	  $longzero=TRUE;
	}
	if ($cursor+1 >= $bufflen) {
	    $result=fillbuffer8($cursor);
	};
	last if $result==END_OF_FILE;
	$cursor++;$fskcounter++;
	$limit-- if $limit_states_or_samples==LIMIT_SAMPLES;			# count samples
      }
      if ($limit<0)  {
	$fskcounter+=$limit;
	$cursor+=$limit;
      }
      $fskdata.=' ' if $fskdata and $fskcounter;
      $fskdata.=($return_samples_or_ms ? $fskcounter : int($fskcounter/$OPTIONS{'samplerate'}*10000)) if $fskcounter;
      $limit-- if $limit_states_or_samples==LIMIT_STATES;			# count states
      $fskcounter=0;
    }
    return (($longzero ? LONGZERO_DETECTED : OK),$fskdata);
  }

  sub detectbit {

  # Detect marker at start of data block
  # and compute bit length
  # Arguments:
  #  - DEJAVUc - DEJAVU control switch (boolean), update DEJAVU if FALSE;

      my ($i,$j)=(0,0);
      my $bit;
      my $count=0;	# sample counter
      my $tempcursor;
      my $savedbitlen;
      my $wideststate=0;
      my $start;
      my $result;

      my $DEJAVUc=($_[0] or FALSE);

      my $MAXBITLEN=int(MAXBITLEN*$OPTIONS{'samplerate'}/44100+0.5);
      my $MINBITLEN=int(MINBITLEN*$OPTIONS{'samplerate'}/44100+0.5);

      if ($OPTIONS{'baudmin'}) {
	$MAXBITLEN=int($OPTIONS{'samplerate'}/$OPTIONS{'baudmin'}+0.5);
      }

      if ($OPTIONS{'baudmax'}) {
	$MINBITLEN=int($OPTIONS{'samplerate'}/$OPTIONS{'baudmax'}+0.5);
      }

      $start=$totalsamples-$bufflen+$cursor;

      print $LOGFILE "real start: $start, dejavu start: $DEJAVU[0], cursor: $cursor\n" if defined($DEBUG{'detectbit'});
      if ($DEJAVU[0]==$start and $DEJAVU[1]==$NUMBER_OF_STATES) {
	$NUMBER_OF_STATES+=10 if $NUMBER_OF_STATES>=10;
	$DEJAVU[1]=$NUMBER_OF_STATES;
 	return FSK_DETECTED if $NUMBER_OF_STATES>$OPTIONS{'minsignals'}+80;	# >100 is ridiculous
      } elsif (!$DEJAVUc) {
	print $LOGFILE "resetting DEJAVU - numofstates was $NUMBER_OF_STATES, dejavu1 was $DEJAVU[1]\n" if defined($DEBUG{'detectbit'});
	$NUMBER_OF_STATES=$OPTIONS{'minsignals'};
	$DEJAVU[0]=$start;
	$DEJAVU[1]=$NUMBER_OF_STATES;
      } else {
	print $LOGFILE "real start: $start, dejavu2 start: $DEJAVU[2]\n" if defined($DEBUG{'detectbit'});
	return FSK_DETECTED if $start==$DEJAVU[2];
	if ($start>$DEJAVU[2]) {
	  $DEJAVU[2]=$start;
	} else {
	$NUMBER_OF_STATES+=10 if $NUMBER_OF_STATES>=10;
	$DEJAVU[1]=$NUMBER_OF_STATES;
 	return FSK_DETECTED if $NUMBER_OF_STATES>$OPTIONS{'minsignals'}+80;	# >100 is ridiculous
	}
      }

      $bitlen=$MAXBITLEN;		# (set bitlen to maximum possible value)
      $minbitlen=$MINBITLEN; 		# (bit should never be shorter than this)

      # try to detect bit length
      my ($fskstates,@fskstates);
      # get 20 fsk states
      ($result,$fskstates) = getfsk($NUMBER_OF_STATES,GET_SAMPLES,LIMIT_STATES);
      @fskstates=split(' ',$fskstates);
      return END_OF_FILE if ($result==END_OF_FILE and $#fskstates<3);			# FIXME? must not loose last bytes
      # exit if only one state was returned
      if (0+@fskstates==1 and ($result==IRG_DETECTED or $result==LONGZERO_DETECTED)) {
	$cursor=$cursor-($totalsamples-$bufflen+$cursor-$start),
	return $result;
      }
      if (($result==IRG_DETECTED or $result==LONGZERO_DETECTED) and $#fskstates<3) {
	$cursor=$cursor-($totalsamples-$bufflen+$cursor-$start),
	return FSK_DETECTED;
      }
      $count=0;
      print $LOGFILE "states: ",join(' ',@fskstates),"\n" if defined($DEBUG{'detectbit'});
      $count=splice(@fskstates,$#fskstates) if $result==IRG_DETECTED or $result==END_OF_FILE;	# don't count irg or last state at the end
      print $LOGFILE "count: $count, cursor: $cursor, start: $start, total: $totalsamples, bufflen: $bufflen\n" if defined($DEBUG{'detectbit'});
      $cursor-=$count;
      $count=$totalsamples-$bufflen+$cursor-$start;
      print $LOGFILE "count: $count, states: ",join(' ',@fskstates),"\n" if defined($DEBUG{'detectbit'});

      my $spikes=0;
      SPIKES: { for (0..$#fskstates) {
	  last SPIKES if @fskstates==1 or $spikes>10;
	  last SPIKES if $fskstates[$_]>$MAXBITLEN*10;
	  if ($fskstates[$_]<$minbitlen) {				# was it a random spike ?
	    $spikes++;
	    print $LOGFILE "Spike was: $fskstates[$_],  states: $#fskstates, pos: $_, minbitlen: $MINBITLEN, merging states\n" if defined($DEBUG{'detectbit'});
	    my $tempfskstate;
	    if ($_==0) {
	      $fskstates[1]+=$fskstates[0];
	      splice(@fskstates,0,1);
	      ($result,$tempfskstate) = getfsk(1,GET_SAMPLES);			# get 1 additional fsk state
	      return END_OF_FILE if $result==END_OF_FILE;
	      if ($result==IRG_DETECTED or $result==LONGZERO_DETECTED) {
		$cursor-=$tempfskstate;
		redo SPIKES;
	      }
	    } elsif ($_==$#fskstates) {
	      $fskstates[$_-1]+=$fskstates[$_];
	      splice(@fskstates,$_);
	      ($result,$tempfskstate) = getfsk(1,GET_SAMPLES);			# get 1 additional fsk state
	      return END_OF_FILE if $result==END_OF_FILE;
	      if ($result==IRG_DETECTED or $result==LONGZERO_DETECTED) {
		$cursor-=$tempfskstate;
		redo SPIKES;
	      }
	    } else {
	      $fskstates[$_-1]+=$fskstates[$_]+$fskstates[$_+1];
	      splice(@fskstates,$_,2);
	      ($result,$tempfskstate) = getfsk(2,GET_SAMPLES);			# get 2 additional fsk states
	      return END_OF_FILE if $result==END_OF_FILE;
	      if ($result==IRG_DETECTED or $result==LONGZERO_DETECTED) {
		(my $irg=$tempfskstate)=~s/(\w*)(\ )(\w*)*/$3/;
		if ($irg) {
		    $cursor-=$irg;
		} else {
		    $tempfskstate=~s/(\w*)(\ \w*)*/$1/;
		    $cursor-=$tempfskstate;
		}
		redo SPIKES;
	      }
	    }
	    redo SPIKES if !$tempfskstate;
	    #print $LOGFILE "fskstate: $tempfskstate\n";
	    push(@fskstates,split(' ',$tempfskstate));			# and store it
	    foreach (split(' ',$tempfskstate)) {
		$count+=$_;
	    }
	    #print $LOGFILE "count: $count, states: ",join(' ',@fskstates),"\n";
	    redo SPIKES;
	  }
	}
      }

      if ((!$#fskstates and $result==IRG_DETECTED) or $spikes>10 or $count<10*$MINBITLEN) {
	$cursor=$cursor-($totalsamples-$bufflen+$cursor-$start),
	return FSK_DETECTED;
      }

      my $grouplen=1;
      for ($grouplen=1;$grouplen<=5;$grouplen++) {			# this is for baud rates < 600
	foreach (@fskstates) {
	#find shortest state which will give baud rate between 300 and 2600)
	# at the same time find widest state
	$bitlen=$_ if ($_<$bitlen and $_>$MINBITLEN and $_<$MAXBITLEN*$grouplen);
	$wideststate=$_ if ($_ > $wideststate);
	}
	last if $bitlen<$MAXBITLEN*$grouplen;
	$bitlen+=$MAXBITLEN;			# search for wider fragments (2,3,4,5-bit width)
      }
      $minbitlen=int($bitlen/2.5);

      #now get average of shortest states
      my $sum=0; $i=0;
      foreach (@fskstates) {
	if (abs($_-$bitlen)<$minbitlen)  {
	  $sum+=$_; $i++;
	}
      }

      print $LOGFILE "sum: $sum, grouplen: $grouplen, i: $i\n" if defined($DEBUG{'detectbit'});

      $bitlen=int($sum/$i/$grouplen+0.5) if $i!=0;

      print $LOGFILE "Average shortest bitlen: $bitlen\n" if defined($DEBUG{'detectbit'});
      my $avgbitlen=$bitlen;
      #now get real bitlen
      $j=2;
      do {
	  $i=0;
	  foreach (@fskstates) {
	    $i+=int($_/$bitlen+0.5);
	  }
	  $bitlen=int($avgbitlen/$j+0.5);	# search for the width which gives at least 20 bits
	  $j++;
      } while (($i<0+@fskstates and $bitlen>$MINBITLEN));# or $wideststate/$bitlen/2<1.6); 	# this HACK is for baud rates > 600

      $i=20 if $NUMBER_OF_STATES==20 and $i==40 and abs($avgbitlen-$bitlen)<$MINBITLEN;	# HACK - stupid but effective, for high baud rates like 1200
      print $LOGFILE "Bit after dividing: $bitlen, i: $i, count: $count\n" if defined($DEBUG{'detectbit'});
      #There can not be more than 5*x bits in x states
      $i=5*$NUMBER_OF_STATES if ($i>5*$NUMBER_OF_STATES);

      $bitlen=int($count/$i+0.5);

      if ($wideststate/$bitlen>9.4) {
	# that's impossible.
	print $LOGFILE "Bit too narrow ($bitlen) - adjusting...\n" if defined($DEBUG{'detectbit'});
	until ($wideststate/$bitlen<9.4) {
	    $bitlen++;
        }
	if ($bitlen>$MAXBITLEN) {
	  print $LOGFILE "And now too wide ($bitlen) - getting raw FSK\n" if defined($DEBUG{'detectbit'});
	  $cursor-=$count;
	  return FSK_DETECTED;
	}
      }

      $bitlen=int($bitlen/2+0.5) if $bitlen>$MAXBITLEN;	# HACK 2 - also stupid but also works

      print $LOGFILE "Real shortest bitlen: $bitlen ($count/$i), MAXBITLEN: $MAXBITLEN, cursor: $cursor\n" if defined($DEBUG{'detectbit'});

      if ($count+($count/$NUMBER_OF_STATES)*10 > HALFBUFFLEN) {  #avoid data loss
	  print $LOGFILE "Buffer too small to continue, assuming raw FSK detected\n" if defined($DEBUG{'detectbit'});
	  $cursor=$cursor-$count;
	  return FSK_DETECTED;
      }

      $minbitlen=int($bitlen/2.5); 		# (bit should never be shorter than this)

      $cursor=$cursor-$count;			# rewind

      # high baudrate and small number of states = fsk
      # return FSK_DETECTED if $bitlen<49 and 0+@fskstates<4;
      # commented out, because it generally does not hurt

    return OK;
  }

  sub skipstate {

  #
  # Skip one FSK state
  #
  # Arguments:
  #  - state to skip
  # Returns:
  #  - status
  #  - lenghth of skipped state

      my $state=($_[0] or MARK);
      my $tempcursor;
      my $counter;

      my $statelen=0;

      if ($cursor >= $bufflen) {
        return (END_OF_FILE,$statelen) if fillbuffer8($cursor) < 0;
      }

      print $LOGFILE "Start state in skipstate: " . ($abuff[$cursor] eq STOP ? "STOP" : "START") . ", cursor is $cursor.\n" if defined($DEBUG{'skipstate'});
      while(TRUE) {
	  while (($abuff[$cursor] eq $state)) {
	      $statelen++; $cursor++;
	      if ($cursor >= $bufflen) {
		  return (END_OF_FILE,$statelen) if fillbuffer8($cursor) < 0;
	      }
	  }
	  $counter=0;

	  #look for spike (maybe unneded)
	  $tempcursor=$totalsamples-$bufflen+$cursor;
	  while (($abuff[$cursor] ne $state)) {
	      $cursor++; $counter++;
	      if ($cursor >= $bufflen) {
		  return (END_OF_FILE,$statelen) if fillbuffer8($cursor) < 0;
	      }
	    if ($counter > $minbitlen) {	# found a start bit after IRG or long zero
		last; }
	  }
	  if ($counter > $minbitlen) {	# found a start bit after IRG
	      $cursor=$cursor-($totalsamples-$bufflen+$cursor-$tempcursor);
	      last; }
	  else {
	      $statelen+=$counter;			# found a random spike, go on
	  }
      }
      return (OK,$statelen);
  }

  sub getbyte {

  #  get one byte

  # Arguments:
  # - a reference to variable for storing a byte

    my $i;
    my $bit;
    my $savedbit;
    my $fskbyte='';
    my (@states,@fskstates)=((),());
    my $byte;
    my $sum=0;
    my $result;
    my $bitcounter=0;
    my $tempcursor=0;
    my $spikedetect;
    my $start;
    our $initialbitlen=$bitlen;

WHOLE: {

    #skip until start bit
    $bitcounter=0;
    while($abuff[$cursor] ne START) {
	$cursor++; $bitcounter++;
	if ($cursor >= $bufflen) {
	    return END_OF_FILE if fillbuffer8($cursor) < 0;
	}
    }
    print $LOGFILE "Skipped $bitcounter samples at the beginning\n" if defined($DEBUG{'getbyte'}) and $bitcounter;

    $start=$totalsamples-$bufflen+$cursor;

    ($result,$fskbyte)=getfsk(10*$bitlen+1,GET_SAMPLES,LIMIT_SAMPLES);   # FIXME +1 looks stupid
    if ($result==END_OF_FILE)  {
      $_[0]='';
      return END_OF_FILE;
    }

    print $LOGFILE "Got 10xbitlen states: $fskbyte, bitlen: $bitlen, minbitlen: $minbitlen\n" if defined($DEBUG{'getbyte'});

    @fskstates=split(' ',$fskbyte);

    # remove random spikes
    SPIKES: { for (1..$#fskstates-1) {
	last SPIKES if $_>$#fskstates-1;
	if ($fskstates[$_]<$minbitlen) {				# was it a random spike ?
	  print $LOGFILE "Spike was: $fskstates[$_], merging 3 states\n" if defined($DEBUG{'getbyte'});
	  $fskstates[$_-1]+=$fskstates[$_]+$fskstates[$_+1];
	  splice(@fskstates,$_,2);
	  print $LOGFILE "states: ",join(' ',@fskstates)," ,minbitlen: $minbitlen\n" if defined($DEBUG{'getbyte'});
	  redo SPIKES;
	}
      }
    }

    if (@fskstates==1) {
      print $LOGFILE "Got only ONE state - bit length must be higher.\n" if defined($DEBUG{'getbyte'});
      $cursor=$cursor-($totalsamples-$bufflen+$cursor-$start+$bitcounter);
      $_[0]='';
      return REQ_BAUD_CHANGE;
    }

    if (!($#fskstates % 2)) {
	  if ($fskstates[$#fskstates] >= $bitlen-$minbitlen or
	      $fskstates[$#fskstates] < $minbitlen) {
	    print $LOGFILE "Removing last state\n" if defined($DEBUG{'getbyte'});
	    $cursor-=$fskstates[$#fskstates];
	    splice(@fskstates,$#fskstates);
	  }
    } else {
      if ($fskstates[$#fskstates] < $minbitlen and $abuff[$cursor] eq ONEBIT) {
	print $LOGFILE "Filling up state, curent is ",join(',',@fskstates),"\n" if defined($DEBUG{'getbyte'});
	my ($result,$fskstate);
	($result,$fskstate)=getfsk(1,GET_SAMPLES,LIMIT_STATES);
	return END_OF_FILE if $result==END_OF_FILE;
	$fskstates[$#fskstates]+=$fskstate;
	print $LOGFILE "After filing is ",join(',',@fskstates),"\n" if defined($DEBUG{'getbyte'});
	my $newbitlen=int((10*$bitlen+$fskstate)/10);
	abs($newbitlen-$bitlen)>4 && do {
	    print $LOGFILE "Byte after filling up is too long. Baud change\n" if defined($DEBUG{'getbyte'});
	    $cursor=$cursor-($totalsamples-$bufflen+$cursor-$start+$bitcounter);
	    $_[0]='';
	    return REQ_BAUD_CHANGE;
	  };
	$bitlen=$newbitlen;
	$minbitlen=int($bitlen/2.5);
      }
    }

    sub checknextbaud {
      print $LOGFILE "Next baud check...\n" if defined($DEBUG{'getbyte'});
      my ($savedbitlen,$savedminbitlen)=($bitlen,$minbitlen);
      getfsk(1,GET_SAMPLES,LIMIT_STATES) if $abuff[$cursor] eq MARK;
      my $result=detectbit(TRUE);
      return (END_OF_FILE,0) if $result==END_OF_FILE;
      if (abs($bitlen-$initialbitlen)>$OPTIONS{'maxbauddiff'} or ($result==FSK_DETECTED and $bitlen!=$savedbitlen)) {	# bitlen changed
	print $LOGFILE "Difference too big, bitlen: $bitlen, initial: $initialbitlen, result: $result.\n" if defined($DEBUG{'getbyte'});
	my $newbitlen=$bitlen;
	($bitlen,$minbitlen)=($savedbitlen,$savedminbitlen);
	return (FSK_DETECTED,0) if $result==FSK_DETECTED;
	return (IRG_DETECTED,0) if $result==IRG_DETECTED;
	return (LONGZERO_DETECTED,0) if $result==LONGZERO_DETECTED;
	return (REQ_BAUD_CHANGE,($newbitlen<$bitlen ? 1 : -1));
      }
      return (FSK_DETECTED,0) if $result==FSK_DETECTED;
      print $LOGFILE "Difference acceptable (old: $initialbitlen, new: $bitlen), adjusting bit length\n" if defined($DEBUG{'getbyte'});
      #($bitlen,$minbitlen)=($savedbitlen,$savedminbitlen); # Dont do it - will autoadjust
      return (OK,0);
    }

    if (!($#fskstates % 2) and $result!=IRG_DETECTED) {
      print $LOGFILE "Number of fsk states in byte must be even\n" if defined($DEBUG{'getbyte'});
      #print $LOGFILE "Rewinding and requesting baud change\n";
      $cursor=$cursor-($totalsamples-$bufflen+$cursor-$start);
      ($result,my $change)=checknextbaud();
      $cursor=$cursor-($totalsamples-$bufflen+$cursor-$start);
      $DEJAVU[0]=$totalsamples-$bufflen+$cursor;
      redo WHOLE if $result==OK;
      $cursor-=$bitcounter;
      $_[0]='';
      return FSK_DETECTED if $result==FSK_DETECTED;
      return REQ_BAUD_CHANGE;
    }

    @states=@fskstates;


    $states[0][0]=0;

    $i=0;
    my $bits=0;
    my $count=0;
    foreach (@fskstates) {
      $states[$i][0]+=$_;
      $states[$i+1][0]=0;
      $states[$i+1][1]=0;
      $states[$i][1]=int($states[$i][0]/$bitlen);
      if (($states[$i][0] % $bitlen)>$minbitlen+1) {
	$states[$i][1]++;
      } else {
	$states[$i+1][0]+=($states[$i][0] % $bitlen);
      }
      $count+=$_;
      $i++;
    }
       for (0..$#states) {
	  $bits+=$states[$_][1];
	}

    if ($result==LONGZERO_DETECTED and 10*$bitlen-$count>$minbitlen) {	# this is for long zero
	print $LOGFILE "Long zero detected.\n" if defined($DEBUG{'getbyte'});
	$cursor=$cursor-($totalsamples-$bufflen+$cursor-$start+$bitcounter);
	$_[0]='';
	return LONGZERO_DETECTED;
    }

    if (defined($DEBUG{'getbyte'})) {
      print $LOGFILE "Table: ";
      for $i (0..$#states) {
        print $LOGFILE "$states[$i][0]($states[$i][1]),";
      }
      print $LOGFILE "; count=$count, bits=$bits.\n";
    }

    if (!($states[0][1])) {
      print $LOGFILE "Start bit too short  ($states[0][0])\n" if defined($DEBUG{'getbyte'});
      #print $LOGFILE "Rewinding and requesting baud change\n";
      $cursor=$cursor-($totalsamples-$bufflen+$cursor-$start+$bitcounter);
      $_[0]='';
      return REQ_BAUD_CHANGE;
    }

    if (abs(int($count/10+0.5)-$bitlen)>4)  {
	print $LOGFILE "Trying baud change (1)\n" if defined($DEBUG{'getbyte'});
	my $savedbitlen=$bitlen;
	(my $result, my $change)=checknextbaud();
	if ($result==OK) {
	     if ($savedbitlen!=$bitlen) {
	      $cursor=$cursor-($totalsamples-$bufflen+$cursor-$start);
	      $DEJAVU[0]=$totalsamples-$bufflen+$cursor;
	      $cursor-=$bitcounter;
	      redo WHOLE;
	    }
	} else {
	  $cursor=$cursor-($totalsamples-$bufflen+$cursor-$start+$bitcounter);
	  $_[0]='';			# .. finish the data block
          return END_OF_FILE if $result==END_OF_FILE;
	  return FSK_DETECTED if $result==FSK_DETECTED;
	  return REQ_BAUD_CHANGE if $result==REQ_BAUD_CHANGE;
	}
    }

    if ($bits<10 or $bits>10) {
      print $LOGFILE "byte must have exactly 10 bits (has $bits)\n" if defined($DEBUG{'getbyte'});
      if ($bits==11 and @fskstates==10)  {
	# impossible - all previous tests passed, 10 states and 11 bits.
	# redistribute extra bit to neighbours
	print $LOGFILE "10 states and 11 bits - impossible. Redistributing.\n" if defined($DEBUG{'getbyte'});
	my $x=0;
	for (0..$#states-1) {	# last does not count
	  $states[$_][0]+=$x;
	  $states[$_][1]-- if $states[$_][1]==2;
	  $x=$states[$_][0] % $bitlen;
	  #print $LOGFILE "state=$_, x=$x\n";
	  $x=$x-$bitlen if $states[$_][0]==$x;
	  $states[$_][0]=$bitlen;
	  #print $LOGFILE "state=$_, x=$x\n";
	  }
	$states[$#states][0]+=$x;
	#print $LOGFILE "New table: ";
	#for $i (0..$#states) {
	#  print $LOGFILE "$states[$i][0]($states[$i][1]),";
	#}
      } else {
	print $LOGFILE "Trying baud change (2)\n" if defined($DEBUG{'getbyte'});
	my ($savedbitlen,$savedminbitlen)=($bitlen,$minbitlen);
	(my $result, my $change)=checknextbaud();
	if ($result==OK) {
	    $cursor=$cursor-($totalsamples-$bufflen+$cursor-$start);
	    $DEJAVU[0]=$totalsamples-$bufflen+$cursor;
	    $cursor-=$bitcounter;
	    redo WHOLE;
	} else {
	  $cursor=$cursor-($totalsamples-$bufflen+$cursor-$start+$bitcounter);
	  $_[0]='';			# .. finish the data block
          return END_OF_FILE if $result==END_OF_FILE;
	  return FSK_DETECTED if $result==FSK_DETECTED;
	  return REQ_BAUD_CHANGE if $result==REQ_BAUD_CHANGE;
	  return REQ_BAUD_CHANGE;
	}
      }
    }

    if ($states[$i-1][1]==0) {
      print $LOGFILE "Stop bit too short\n" if defined($DEBUG{'getbyte'});
      #print $LOGFILE "Rewinding and requesting baud change\n";
      $cursor=$cursor-($totalsamples-$bufflen+$cursor-$start+$bitcounter);
      $_[0]='';
      return REQ_BAUD_CHANGE;
    }

    CASE: for ($OPTIONS{'fskdecoder'}) {
      /new/ && do {
		# method 1 - simple  - best
		for $i (0..$#states-1) {		# last is extra
		  $byte.=(($i % 2) ? '1' : '0') x ($states[$i][1]-($i==0 or $i==$#states-1));  # start from 0 (start)
											  # don't count start and stop
		}
		last CASE;
	       };
      /old/ && do {
		# method 2 - wchih state is wider in $bitlen - 2nd best
		$i=0;
		my $rest=0;
		$byte='';

		foreach (@fskstates) {
		  my $b=(($i % 2) ? '1' : '0');
		  my $c=(int(($_+$rest)/$bitlen)-($i==0 or $i==$#fskstates));	# dont count start and stop
		  my $x=($_+$rest) % $bitlen;
		  if ($x>=$minbitlen) {
		    $c++;
		    $rest=$x-$bitlen;
		  } else {
		    $rest=$x;
		  }
		  $byte.=$b x $c;
		  $i++;
		}
		last CASE;
	       };
      /pokey/ && do {
		# method 3 - like a POKEY - as good as previous two
		$sum=int($bitlen/2+0.5);	# probe in the middle of bit
		my $st=0;
		my $sum2=$fskstates[$st];
		my $b='0';
		for (0..7) {
		  $sum+=$bitlen;
		  if ($sum>$sum2) {
		    $st++;
		    $sum2+=$fskstates[$st];
		    $b=($b eq '0' ? '1' : '0');
		  }
		  $byte.=$b;
		}
	       };
    }
    $_[0]=pack("b8",$byte); #convert bin to char

    print $LOGFILE "Byte is: ",ord($_[0]),"\n" if defined($DEBUG{'getbyte'});

    if ($result==IRG_DETECTED) {
      #OK, let's check, if it really is IRG or only baudrate change
      print $LOGFILE "Checking irg...\n" if defined($DEBUG{'getbyte'});
      my ($savedbitlen,$savedminbitlen)=($bitlen,$minbitlen);
      $cursor-=$fskstates[$#fskstates];
      $result=detectbit(TRUE);
      $cursor=$cursor-($totalsamples-$bufflen+$cursor-$start);
      SWITCH: {
	if ($result==END_OF_FILE or $result==IRG_DETECTED or $result==FSK_DETECTED or $result==LONGZERO_DETECTED) {
	  print $LOGFILE "irg is ok (result=$result).\n" if defined($DEBUG{'getbyte'});
	  $cursor+=$count;
	  $result=IRG_DETECTED;
	  last SWITCH;
	}
	if (abs($bitlen-$savedbitlen)>4)  {
	  print $LOGFILE "irg is not ok - baud change.\n" if defined($DEBUG{'getbyte'});
	  $cursor-=$bitcounter;
	  $_[0]='';
	  $result=REQ_BAUD_CHANGE;
	  last SWITCH;
	}
	$result=IRG_DETECTED;
      }
      ($bitlen,$minbitlen)=($savedbitlen,$savedminbitlen);
      return $result;
    }
    STOPCHECK: { if ($abuff[$cursor] eq STOP) {
      my $fskstate;
      ($result,$fskstate)=getfsk(1,GET_SAMPLES,LIMIT_STATES);
      if ($result==END_OF_FILE) {
	$cursor-=$fskstate;
	return OK;
      }
      print $LOGFILE "Stop check: was $fskstate\n" if defined($DEBUG{'getbyte'});
      if ($fskstate>$bitlen) {		# it will end here, don't mind destroying variables
	if ($fskstate > 2*MAXBITLEN) {	# it is not a new bit for sure
	    $cursor-=$fskstate;
	    return FSK_DETECTED;
	}
	# check what lies ahead
	my $tempcursor=$totalsamples-$bufflen+$cursor;
	#($result,$fskbyte)=getfsk(9*$bitlen,GET_SAMPLES,LIMIT_SAMPLES);
	($result,my $change)=checknextbaud();
	#print $LOGFILE "result: $result\n";
	$cursor=$cursor-($totalsamples-$bufflen+$cursor-$tempcursor+$fskstate);
	return $result if $result==IRG_DETECTED or $result==FSK_DETECTED or $result==OK;
	return OK if $result==LONGZERO_DETECTED;
	return OK if $change==1;  # higher baud, but stop longer - it does not suit, byte is ok.
	$cursor=$cursor-($totalsamples-$bufflen+$cursor-$start+$bitcounter);
	$_[0]='';
	return REQ_BAUD_CHANGE;
      } else {
	$cursor-=$fskstate;
      }
      $fskbyte=(($fskstates[$#fskstates] > $bitlen+$minbitlen) ? $fskstate+$bitlen : $fskstate+$fskstates[$#fskstates]);
      print $LOGFILE "Before stop check fskbyte: ",$fskbyte,", minbitlen: $minbitlen.\n" if defined($DEBUG{'getbyte'});
      last STOPCHECK if
	    ($fskbyte <= $bitlen + $minbitlen and
	    $fskbyte >= $bitlen - $minbitlen ) or
	    $result==IRG_DETECTED or $result==LONGZERO_DETECTED or
	    $result==FSK_DETECTED;		# everything OK, deviation small
      $cursor+=$fskstate;
      if ($cursor >= $bufflen) {
	return END_OF_FILE if fillbuffer8($cursor)<0;
      }
      (my $result, my $change)=checknextbaud();
      if ($result==OK or $result==LONGZERO_DETECTED or $result==IRG_DETECTED or $result==FSK_DETECTED) {
	#$cursor-=$fskstate; # !!!
	last STOPCHECK;
      } else {
	$cursor=$cursor-($totalsamples-$bufflen+$cursor-$start+$bitcounter);
        return END_OF_FILE if $result==END_OF_FILE;
	$_[0]='';
	return FSK_DETECTED if $result==FSK_DETECTED;
	return REQ_BAUD_CHANGE if $result==REQ_BAUD_CHANGE;
      }
      if ($fskbyte>$bitlen+$minbitlen) {
	# check what lies ahead
	my $tempcursor=$totalsamples-$bufflen+$cursor;
	($result,$fskbyte)=getfsk(9*$bitlen,GET_SAMPLES,LIMIT_SAMPLES);
	$cursor=$cursor-($totalsamples-$bufflen+$cursor-$tempcursor);
	return OK if $result==IRG_DETECTED;
        $cursor=$cursor-($totalsamples-$bufflen+$cursor-$start+$bitcounter);
	$_[0]='';
	return REQ_BAUD_CHANGE;
      }
      print $LOGFILE "Stop bit may be too long: ",$fskbyte,", minbitlen: $minbitlen. Checking...\n" if defined($DEBUG{'getbyte'}); # and here stop is too long
      my ($savedbitlen,$savedminbitlen)=($bitlen,$minbitlen);
      $cursor+=$fskstate;
      if ($cursor >= $bufflen) {
	return END_OF_FILE if fillbuffer8($cursor)<0;
      }
      $result=detectbit(TRUE);
      return END_OF_FILE if $result==END_OF_FILE;
      return OK if $result==FSK_DETECTED or $result==IRG_DETECTED;
      print $LOGFILE "New bitlen might be: $bitlen\n" if defined($DEBUG{'getbyte'});
      if (abs($bitlen-$savedbitlen)>1) {			# stop deviation is accidental
	$cursor=$cursor-($totalsamples-$bufflen+$cursor-$start+$bitcounter);
	($bitlen,$minbitlen)=($savedbitlen,$savedminbitlen);	# restore bitlen
	$_[0]='';
	return REQ_BAUD_CHANGE;
      }
      $cursor=$cursor-($totalsamples-$bufflen+$cursor-$start+$bitcounter);
      redo WHOLE;
     }
    } #STOPCHECK

    }
    return OK;
  }

  sub getrecord {

#
#  get one record of bytes
#

      my $reclen=0;
      my $result=OK;
      my $casdatabyte;
      my $start;

      @casrecord=();
      if ($abuff[$cursor] eq NEUTRAL) {
	my $silence=0;
	($result,$silence)=skipstate(NEUTRAL);
	print $LOGFILE "SILENCE: $silence.\n";
      }
	my $saveirg=$irglen;
      ($result,$irglen)=skipstate(MARK);
	$irglen+=$saveirg;
      return ($result,0) if $result==END_OF_FILE;
      print $LOGFILE "IRG: $irglen.\n" if $DEBUG{'getrecord'};
      print $LOGFILE "Record starts at sample ",$totalsamples-$bufflen+$cursor,", time ",($totalsamples-$bufflen+$cursor)/$OPTIONS{'samplerate'}+$OPTIONS{'starttime'},"\n";
      $DATASTORE{'recordstart'}=$totalsamples-$bufflen+$cursor;
      if ($OPTIONS{'fskonly'}) {
	$bitlen=MAXBITLEN;
	($result,$casrecord[$reclen++])=getfsk();
	if ($result==LONGZERO_DETECTED) {
	    print $LOGFILE "zeroIRG: $irglen\n";
	    ($result,$casrecord[$reclen])=skipstate(SPACE);
	    $casrecord[$reclen]=int($casrecord[$reclen]*10000/$OPTIONS{'samplerate'});	# store in tenths of ms
	    print $LOGFILE "ZERO: ",$casrecord[$reclen],", cursor: $cursor.\n";
	    $casrecord[$reclen-1] .= " " . $casrecord[$reclen];
	    print $LOGFILE $casrecord[$reclen-1] . "\n";
	    print $LOGFILE "zeroIRG: $irglen\n";
	    splice(@casrecord,$#casrecord);
	    return (FSK_DETECTED,$reclen);
	}
	return ($result,0) if $result==END_OF_FILE and ! $casrecord[$reclen-1];
	return (FSK_DETECTED,$reclen);
      }
      print $LOGFILE "I'm at: ",$totalsamples-$bufflen+$cursor,"\n" if $DEBUG{'getrecord'};
      $result=detectbit();
      print $LOGFILE "I'm after detectbit at: ",$totalsamples-$bufflen+$cursor,"\n" if $DEBUG{'getrecord'};
      return ($result,0) if ($result==END_OF_FILE or $result==IRG_DETECTED);
      if ($result==LONGZERO_DETECTED) {
	($result,$casrecord[$reclen])=skipstate(SPACE);
	$casrecord[$reclen]=int($casrecord[$reclen]*10000/$OPTIONS{'samplerate'});	# store in tenths of ms
	return (LONGZERO_DETECTED,++$reclen);
      }
      if ($result==FSK_DETECTED) {
	print $LOGFILE "Raw FSK detected - collecting. Irg: $irglen\n" if $DEBUG{'getrecord'};
	#store fsk states in tenths of miliseconds
	($result,$casrecord[$reclen++])=getfsk();
	return (FSK_DETECTED,$reclen);
      }
      print $LOGFILE "Bit length: $bitlen samples\n";
      print $LOGFILE "I'm before getbytes at: ",$totalsamples-$bufflen+$cursor,"\n" if $DEBUG{'getrecord'};
      while (TRUE) {
	  $result=getbyte($casdatabyte);
	  $casrecord[$reclen]=$casdatabyte;
	  if ($result==FSK_DETECTED or $result==LONGZERO_DETECTED) {
	    $reclen++ if $casdatabyte ne '';
	    $casrecord[$reclen]='';
	  }
	  $reclen++ if $result!=END_OF_FILE;
	  last if $result!=OK;
      }
      # store point of end of data
      $DATASTORE{'recordend'}=$totalsamples-$bufflen+$cursor;
      return ($result,$reclen);
  }

  sub close_sound_in() {
    close($INFILE) or die "Can't close input stream.";
    1;
  }

  sub open_sound_in {

  #  Opens sound file
  #  Starts ecasound processing and directs its output to pipe
  #

    my $command="";
    my $filename=$_[0];
    my $samplerate;

    pod2usage(-exitval => 1,
	      -verbose => 99,
	      -sections => "conv options",
	      ) if !$filename;

    # check samplerate ...
    $command="ecalength -s -f -r \Q$filename\E 2>/dev/null";
    open($INFILE,"$command|") or die "Can't determine samplerate of input file. Is ecalength installed ?\n";
    defined($samplerate=<$INFILE>) or die "Can't determine samplerate of input file. Is ecalength installed ?\n";
    close($INFILE);
    chomp($samplerate);
    $OPTIONS{'samplerate'}=$samplerate;
    # ...and number of channels in input file
    $command="ecalength -s -f -c \Q$filename\E 2>/dev/null";
    open($INFILE,"$command|") or die "Can't determine number of input channels. Is ecalength installed ?\n";
    my $channels;
    defined($channels=<$INFILE>) or die "Can't determine number of input channels. Is ecalength installed ?\n";
    chomp($channels);
    close($INFILE);

    my $OPT="";
    $OPT="-f:s16_le,1,$samplerate -chmove:2,1" if $channels==2;

    my $AMP=100;
    if ($OPTIONS{'normalize'}) {
      print "Checking, how much the signal can be amplified... ";
      $command="ecasound -i:\Q$filename\E $OPT -ev -o:null 2>/dev/null";
      open($INFILE,"$command|") or die "Can't normalize input file. Is ecasound installed ?\n";
      while (<$INFILE>) {
	($AMP=$_)=~s/^.*Max\ gain\ without\ clipping:\ (.*)\.$/$1/ or $AMP='';
	chomp($AMP);
	last if ($AMP);
      }
      $AMP*=100;
      print "$AMP%\n";
    }
    close($INFILE);

    if ($OPTIONS{'endtime'}) {
      $OPTIONS{'endtime'}+=1024/$samplerate; #remember extra samples at the beginning ?
    }


    # open input file
    FORMAT: {
      if ($OPTIONS{'turbo'} ne '') {
	  $AMP=$AMP*$OPTIONS{'amp'};
	  $command="ecasound -q -x ";
	  $command.="-t:".($OPTIONS{'endtime'}-$OPTIONS{'starttime'})." " if $OPTIONS{'endtime'};
	  $command.="-f:s16_le,$channels,96000 ";
	  if ($OPTIONS{'hqresample'}) {
	    $command.="-i:resample-hq,$samplerate,\Q$filename\E ";
	  } elsif ($OPTIONS{'lqresample'}) {
	    $command.="-i:resample-lq,$samplerate,\Q$filename\E ";
	  } else {
	    $command.="-i:resample,$samplerate,\Q$filename\E ";
	  }
	  $command.="-y:" . $OPTIONS{'starttime'} . " " if $OPTIONS{'starttime'};
	  $command.="-el:butthigh_iir,$PWMS{'highpass'},$PWMS{'highreso'} " if $OPTIONS{'highpass'};
	  $command.="-el:hard_gate,$OPTIONS{'noise'} " if $OPTIONS{'noise'};
	  $command.="-ef3:$PWMS{'lowpass'},$PWMS{'lowreso'},$PWMS{'lowgain'} " if $OPTIONS{'lowpass'};
	  #$command.="-el:BoosterM,1,10,1 " if $OPTIONS{'lowpass'};
	  #$command.="-el:lowpass_iir,$PWMS{'lowpass'},1 " if $OPTIONS{'lowpass'};
	  $command.="$OPT -ea:$AMP ";
	  #$command.="-el:inv ";
	  $command.="-f:s16_le,1,96000 ";
	  $command.="-o:stdout 2>/dev/null";
	  last FORMAT;
	}
	$command="ecasound -q -x ";
	$command.="-t:".($OPTIONS{'endtime'}-$OPTIONS{'starttime'})." " if $OPTIONS{'endtime'};
	$command.="-a:space,mark -i:\Q$filename\E ";	# open input
	$command.="-y:" . $OPTIONS{'starttime'} . " " if $OPTIONS{'starttime'};
	#next line - unofficial feature. Highpass for standard.
	$command.="-a:space -efh:3500 -a:mark -efh:3500 " if $OPTIONS{'highpass'};
	$command.="-a:space $OPT -efr:".$OPTIONS{'mark'}.",1000 ";	# reject mark
	$command.="-el:bandpass_iir,".$OPTIONS{'space'}.",".$OPTIONS{'bandwidth'}.",3 ";	# pass space
	$command.="-ea:$AMP ";					# amplify space
	$command.="-el:diode,2 ";				# diode
	$command.="-f:s16_le,2,$samplerate -chmove:1,2 ";		# switch to 2ch
	$command.="-a:mark $OPT -efr:".$OPTIONS{'space'}.",1000 ";	# reject space
	$command.="-el:bandpass_iir,".$OPTIONS{'mark'}.",".$OPTIONS{'bandwidth'}.",3 ";	# pass mark
	$command.="-ea:$AMP ";					# amplify mark
	$command.="-el:diode,2 ";				# diode
	$command.="-f:s16_le,2,$samplerate -chmute:2 ";		# switch to 2ch
	$command.="-z:mixmode,sum ";				# sum not avg
	$command.="-a:space,mark -o:loop,mix ";			# mix
	$command.="-a:diff -i:loop,mix ";			# continue mixed
	$command.="-el:difference_iama_oa ";			# comparator ;)

    # Signal after leaving "comparator" is mixed 3 times with its delayed version.
    # Delays are mnimal, 0.04, 0.05, and 0.06 ms.
    # It affects only several samples near zero crossing points,
    # eliminating ALL unwanted random spikes near signal edges.

	$command.="-eli:1058,0.00004,0.5,0.4 "; #fbdelay_0.01s, avoid locale problems
	$command.="-eli:1058,0.00005,0.5,0.4 "; #fbdelay_0.01s
	$command.="-eli:1058,0.00006,0.5,0.4 "; #fbdelay_0.01s

	$command.="-f:u8,1,$samplerate ";		# to 8bit unsig
	$command.="-ezx:1,".$OPTIONS{'resistor'}." ";	# add constant
	$command.="-eli:2602,-10,0,-11,0 -el:hard_gate,0.001"." " if $OPTIONS{'detectsilence'};
	$command.="-ea:6553600 -ea:6553600 ";		# amplify!
	if ($OPTIONS{'rawsquare'}) {
	  $command.="-o:\Q$OPTIONS{'outfile'}\E 2>/dev/null";
	} else {
	  $command.="-o:stdout 2>/dev/null";		# output to pipe, ignore stderr
	}
    }
    $ENV{'MALLOC_CHECK_'}=0;				# avoid LADPSPA stupid crash mesg
    if ($OPTIONS{'rawsquare'}) {
      system("$command");
    } else {
      print "Starting ecasound... ";
      open($INFILE,"$command|") or die "Can't run ecasound.";
      binmode $INFILE;
      print "started.\n";
    }
    1;
  }

### sound2cas processing starts here

  $totalsamples=0;	# total number of input bytes
  $cursor=0;		# number of bytes read so far (skip wav header)
  $bufflen=0;		# buffer is initially empty
  $bitlen=-1;		# length of bit (in samples)
  $minbitlen=int($bitlen/2.5);	# bit should never be shorter than this

  my $lastbitlen=-1;	# remember previous bit length
  my $lastbaud=-1;	# remember previous baud rate
  
  my $i;		# just loop index
  my $records=0;	# data blocks counter
  my $cksum;		# checksum
  my $status;
  my $std=0;		# number of standard data records
  my $errors=0;		# checksum error counter for standard blocks
  my $rawfsk=0;		# number of raw fsk blocks
  my $nonstd=0;		# number of non-standard data records
  my $nonstderr=0;	# checksum errors for non-standard blocks
  $reclen=0;		# length of data record

  my $multibaudrecord='';	# for multi-baud records
  my $multibaud=0;		# number of multibaud records
  my $multibaudparts=0;		# number of parts in multibaud record
  my $multibauderr=0;		# checksum errors for multibaud blocks
  my @multibaudsaved=();	# for temp saving/restoring counters

  my $std_toggle=FALSE;
  my $nonstd_toggle=FALSE;
  my $multibaud_toggle=FALSE;

  my $infile=$_[0];
  my $cas=$_[1];	# the hash table for storing CAS structure
  my $blocks=(keys %{$cas})-2;		# data blocks counter

  $DEJAVU[0]=0;		# for getbyte - endless loop  control
  $DEJAVU[1]=0;
  $DEJAVU[2]=0;

  $NUMBER_OF_STATES = $OPTIONS{'minsignals'};

  $irglen=0;		# initialize IRG

  pod2usage(-exitval => 0,
	    -verbose => 99,
	    -sections => "sound2cas options",
	  ) if $OPTIONS{'help'};

  eval("prepare_" . lc $OPTIONS{'turbo'} . "()")  if $OPTIONS{'turbo'} ne '';

  if ($OPTIONS{'rawsquare'} and $OPTIONS{'turbo'} eq '') {
    print "\nThe audio file will be converted to squared wave.\n";
    print "The output filename will be: $OPTIONS{'outfile'}\n";
    print "Please wait, processing...";
  }

  open_sound_in($infile,$OPTIONS{'turbo'});

  if ($OPTIONS{'rawsquare'}) {
    print " finished.\n";
    exit 0;
  }

  $infile=~s/^(.*)\.(....?)$/$1/;

  # save command line options for future reference
  my $optionsfile=$infile . "-sound2cas.opt";
  $optionsfile=findofile($optionsfile);

  open(OPTIONSFILE,">$optionsfile") or print "Can't write to options file.";
  foreach (sort keys %OPTIONS) {
    print OPTIONSFILE "$_ => $OPTIONS{$_}\n";
  }
  close(OPTIONSFILE);

  my $logfile=$infile . ".log";
  $logfile=findofile($logfile);

  open($LOGFILE,">$logfile") or die "Can't open log file.";
  
  # uncomment for a bit of debugging
  # $LOGFILE->autoflush(1);

  @abuff=();
  @casrecord=();

  if ($OPTIONS{'turbo'} eq '') {
    fillbuffer8();
  } else {
    fillbuffer16();
  }

  # start writting to ,cas file
  cas_put($cas,$blocks++,'FUJI',length($OPTIONS{'title'}),pack("v",0),$OPTIONS{'title'});

  FORMAT: {
    if ($OPTIONS{'turbo'} eq '') {
      # main loop for standard tapes
      prepare_normal();
      while (TRUE) {
	  #@record=();
	  print "Data blocks: $records:";
	  print ", std: $std ($errors E)" if $std_toggle;
	  print ", nonstd: ",$nonstd-$multibaudparts," (",$nonstderr-$multibaudparts," E)" if $nonstd_toggle;
	  print ", multibaud: $multibaud ($multibauderr E)" if $multibaud_toggle;
	  print ", raw FSK: $rawfsk" if $rawfsk;
	  print ".\r";
	  STDOUT->flush;
	  #$bitlen=-1;
	  ($status,$reclen)=getrecord();
	  my $irgsamples=$irglen;
	  $irglen=int($irglen/$OPTIONS{'samplerate'}*1000+0.5);	# recalc IRG to ms
	  my $realirg=$irglen;
	  if ($irgsamples!=0 and $irglen==0) {
	    # lost short signal - fix it by adding fsk signal
	    my $shortfsk=int($irgsamples/$OPTIONS{'samplerate'}*10000+0.5);
	    if ($shortfsk>0) {
		cas_put($cas,$blocks,'fsk ',4,pack('v',0),pack('v*',0,$shortfsk));
		print $LOGFILE "turned into fsk - samples: $irgsamples\n" if $DEBUG{'main'};
		$blocks++;
		$rawfsk++;
	    }
	    $irgsamples=0;
	  }
	  if ($status==END_OF_FILE and $reclen==0) {
	    if ($irglen>0 and $OPTIONS{'allfsk'}) {
	      # save MARK signal at the end  of tape
	      while ($irglen>65535) {
		cas_put($cas,$blocks,"fsk ",0,pack("v",65535),'');
		$blocks++;
		$irglen-=65535;
	      }
	      cas_put($cas,$blocks,"fsk ",0,pack("v",$irglen),'') if $irglen>0;
	      $blocks++;
	    }
	    last;
	  }
	  while ($irglen>65535) {
	    print $LOGFILE "IRGLEN: $irglen\n";
	    cas_put($cas,$blocks,"fsk ",0,pack("v",65535),'');
	    $blocks++;
	    $irglen-=65535;
	  }
	  my @fskdata=();
	  if ($status==FSK_DETECTED or $status==REQ_BAUD_CHANGE or $status==LONGZERO_DETECTED) {
	    @fskdata=split(' ',splice(@casrecord,$#casrecord,1));
	    $reclen--;
	  }
	  if ($reclen>0) {
	    if ($status==REQ_BAUD_CHANGE or $irgsamples==0) {
	      @multibaudsaved=($std,$errors,$nonstd,$nonstderr) if !$multibaudparts;
	      $multibaudrecord.=join('',@casrecord);
	      $multibaudparts++;
	      print $LOGFILE "Multibaud part (parts: $multibaudparts, bytes: ",length($multibaudrecord),", status $status, irg: $irgsamples):",join(',',unpack('C*',$multibaudrecord)),"\n" if $DEBUG{'main'};
	    }
	    print $LOGFILE "Record ends at sample ",$DATASTORE{'recordend'},", time ",$DATASTORE{'recordend'}/$OPTIONS{'samplerate'}+$OPTIONS{'starttime'},"\n";
	    print $LOGFILE "Record samples: ",$DATASTORE{'recordend'}-$DATASTORE{'recordstart'},", baud is ",$reclen*10/($DATASTORE{'recordend'}-$DATASTORE{'recordstart'})*$OPTIONS{'samplerate'},"\n";
	    $records++;
	    if ($reclen!=132) { $nonstd++; } else { $std++; $std_toggle=TRUE; }
	    $nonstd_toggle=TRUE if $nonstd-$multibaudparts>0;
	    # store baud rate whenever it changes, unless skipping bytes
	    if ($bitlen!=$lastbitlen and $reclen>=$OPTIONS{'skipshorter'}) {
	      my $baud=int($reclen*10/($DATASTORE{'recordend'}-$DATASTORE{'recordstart'})*$OPTIONS{'samplerate'}+0.5);
#		  pack("v",int((74/(44100/$OPTIONS{'samplerate'}*$bitlen))*596+0.5)));
		print $LOGFILE "Bitlen=" . ($DATASTORE{'recordend'}-$DATASTORE{'recordstart'})/($reclen*10) . "\n";
	      if ($baud!=$lastbaud) {
		cas_put($cas,$blocks,'baud',0,pack("v",$baud));
		$blocks++;
		$lastbaud=$baud;
	      }
	      $lastbitlen=$bitlen;
	    }
	    if ($reclen>=$OPTIONS{'skipshorter'}) {
#		while ($irglen>65535) {
#		  cas_put($cas,$blocks,"fsk ",0,pack("v",65535),'');
#		  $irglen-=65535;
#		}
		my ($cksum,$expectsum,$cksumok)=cas_put($cas,$blocks,"data",$reclen,	#record length
			    pack("v",$irglen),	# irg
			    join('',@casrecord));					# data
		print $LOGFILE "IRG: $realirg ms.\n";
		print $LOGFILE "record: $records, bytes: $reclen\n";
		print $LOGFILE join(',',unpack('C*',join('',@casrecord)));
		print $LOGFILE "\ncksum: " . $cksum . ", from record: "
			      . $expectsum . "\n";
		if (!$cksumok) {
		    print $LOGFILE "CHECKSUM ERROR in ",
			$reclen==132 ? "standard" : "non-standard",
			" record $records after sample ",$totalsamples-$bufflen+$cursor," !\n" ;
		    if ($reclen!=132) { $nonstderr++; } else { $errors++; }
		}
	    } else {
		print $LOGFILE "skipped short ($reclen bytes) record\n" if $reclen!=0;
		$irglen+=int($reclen*10*$bitlen*1000/$OPTIONS{'samplerate'});	# transform bytes into IRG in ms
		while ($irglen>65535) {
		  cas_put($cas,$blocks,"fsk ",0,pack("v",65535),'');
		  $blocks++;
		  $irglen-=65535;
		}
		cas_put($cas,$blocks,"fsk ",0,
			    pack("v",$irglen),'');	# put irg only
	    }
	    $blocks++;
	    $irglen=0; # IMPORTANT ?
	    $irgsamples=0;
	  } elsif ($#fskdata<0) {
	      # restore IRG
	      $irglen=$irgsamples;
	  }
	if ($multibaudrecord ne '' and ($status==IRG_DETECTED or $status==END_OF_FILE or $status==OK or $status==FSK_DETECTED))  {
	  if ($multibaudparts>1) {
	    print $LOGFILE "Multibaud complete (parts: $multibaudparts, bytes: ",length($multibaudrecord),", status $status):",join(',',unpack('C*',$multibaudrecord)),"\n" if $DEBUG{'main'};
	    $multibaud_toggle=TRUE;
	    $multibaud++;
	    my ($cksum,$expectsum,$cksumok)=controlsum($multibaudrecord);
	    print $LOGFILE "Multibaud checksum: expected: $expectsum, real: $cksum (",$cksumok ? 'OK' : 'BAD',")\n" if $DEBUG{'main'};
	    $multibauderr++ if !$cksumok;
	    ($std,$errors,$nonstd,$nonstderr)=@multibaudsaved;
	  }
	  $multibaudrecord='';
	  $multibaudparts=0;
	  print $LOGFILE "Multibaud reset (status $status, irg: $irgsamples)\n" if $DEBUG{'main'};
	}
	if (($status==FSK_DETECTED and $#fskdata>=0 and $OPTIONS{'allfsk'}) or $status==LONGZERO_DETECTED or $#fskdata>9) {
	      # split very long signals to parts not longer than 6553.5 ms
	      my @tempfsk=();
	      foreach (@fskdata) {
		while($_>65535) {
		  push(@tempfsk,(65535,0));
		  $_-=65535;
		}
		push(@tempfsk,$_);
	      }
	      # print $LOGFILE "IRG: $irglen ms.\n";
	      # print $LOGFILE "Raw FSK data, ",$#fskdata+1," state(s).\n";
	      cas_put($cas,$blocks,"fsk ",($#fskdata+1)*2,
			pack("v",$irglen),
			pack("v*",@tempfsk)
		      );
	  $blocks++;
	  $rawfsk++;
	  $irglen=0;
	} elsif ($status==FSK_DETECTED and !$OPTIONS{'allfsk'} and $#fskdata>=0) {
	    # count fsk as irg, don't nullify irg befor fsk
	    my $i=0;
	    foreach (@fskdata) { 
		$i+=$_;
	    }
	    $irglen=$irgsamples+int($i/10000*$OPTIONS{'samplerate'}+0.5);
	}
	last if ($status==END_OF_FILE or $status==NOT_GOOD);
      }

      print "SUMMARY: Data blocks: $records";
      print ", std: $std ($errors E)" if $std_toggle;
      print ", nonstd: $nonstd ($nonstderr E)" if $nonstd_toggle;
      print ", multibaud: $multibaud ($multibauderr E)" if $multibaud_toggle;
      print ", raw FSK: $rawfsk" if $rawfsk;
      print ", analysed $totalsamples samples.\n";
      STDOUT->flush;

      print $LOGFILE "SUMMARY: Data blocks: $records";
      print $LOGFILE ", std: $std ($errors E)" if $std_toggle;
      print $LOGFILE ", nonstd: $nonstd ($nonstderr E)" if $nonstd_toggle;
      print $LOGFILE ", multibaud: $multibaud ($multibauderr E)" if $multibaud_toggle;
      print $LOGFILE ", raw FSK: $rawfsk" if $rawfsk;
      print $LOGFILE ", analysed $totalsamples samples.\n";
      last FORMAT;
    }
    if ($OPTIONS{'turbo'}=~m/$TURBO_FORMATS_REGEX/io) {
      my @err_records=();
      my $pilotcycles=0;
      my $endcycles=0;
      my $endsamples=0;
      my $lastbits='';
      my $lastbitslen=0;

      $LATCH[LATCHSTATE]=0;		# assume that initial state is '0' ...
      $LATCH[LATCHSLOPE]=SLOPEFALLING;	# ... and the last slope was falling

      $irglen=0;
      eval("prepare_" . lc $OPTIONS{'turbo'} . "()");

      cas_put($cas,$blocks++,"pwms",2,
	      pack("CC",$PWMS{'iparams'},0),
	      pack("v",$OPTIONS{'samplerate'}));
      while (TRUE) {
	  print "Data blocks: $records";
	  if ($OPTIONS{'cksumsane'}) {
	    print " ($errors Errors",$#err_records==-1 ? ").\r" : (" - in records: ",join(",",@err_records),").\r");
	  } else {
	    print " (N/A E - analyze the log file please).\r";
	  }
	  STDOUT->flush;
	  ($status,$endcycles)=getblock_pwm($cas,$blocks);
	  $reclen=$#casrecord;
	  $endsamples=$status if $status>0;
	  #print $LOGFILE "reclen:$reclen\n";
	  $lastbits=splice(@casrecord,$reclen,1) if $reclen>=0;
	  if ($reclen>0) {
	      $records++;
	      print $LOGFILE "record: $records, bytes: $reclen\n";
	      # print $LOGFILE "IRG::,int($irglen*$OPTIONS{'samplerate'}/96000+0.5),"ms\n";
	      print $LOGFILE join(',',unpack('C*',join('',@casrecord))),"\n";
	      #eval('analyse_' . lc $OPTIONS{'turbo'} . '(\'cksum\',join(\'\',@casrecord))');
	      my ($cksum,$expectsum,$cksumok)=cas_put($cas,$blocks,'pwmd',$reclen,
						   pack("CC",int($PWMS{'ZEROW'}*$OPTIONS{'samplerate'}/96000+0.5),
						   int($PWMS{'ONEW'}*$OPTIONS{'samplerate'}/96000+0.5)),
						   join('',@casrecord));
	      if ($OPTIONS{'cksumsane'}) {
		print $LOGFILE "cksum(",$OPTIONS{'cksum'},"): " . $cksum . ", from record: "
			      . $expectsum . "\n";
		if (!$cksumok) {
		    print $LOGFILE "CHECKSUM ERROR in record $records!\n" ;
		    push(@err_records,$records);
		    $errors++;
		}
	      } else {
		print $LOGFILE "\ncksum($OPTIONS{'cksum'},INSANE): ",$cksum,"\n";
	      }
	      $blocks++;
	      print $LOGFILE "\n";
	  }
	  my ($block_type,$block_len,$block_aux,$block_data);
	  $lastbitslen=length($lastbits);
	  $block_type='pwmc';
	  $block_len=0;
	  $block_aux=pack("v",0);
	  $block_data='';
	  # last free bits after data, if any
	  for ($i=1;$i<=$lastbitslen;$i++) {
	    my $j=1;
	    my $impulse=substr($lastbits,0,1,'');
	    while (substr($lastbits,0,1) eq $impulse and $i<=$lastbitslen) {
	      $i++; $j++;
	      $impulse=substr($lastbits,0,1,'');
	    }
	    $block_len+=3;
	    $block_data.=pack("C",int(($impulse ? $PWMS{'ONEW'} : $PWMS{'ZEROW'})*$OPTIONS{'samplerate'}/96000+0.5)) .
		  pack("v",$j);
	  }
	  $lastbits='';
	  # end impulses, if any
	  if ($endcycles>0) {
	    $block_len+=3;
	    $block_data.=pack("C",int($endsamples/$endcycles*$OPTIONS{'samplerate'}/96000+0.5)) .
		pack("v",$endcycles);
	    $endcycles=0;
	  }
	  # additional end impulses, if any
	  if ($PWMS{'ENDIMP'} and $reclen>0) {
	    $block_len+=3;
	    $block_data.=pack("C",int(($PWMS{'ENDIMP'})*$OPTIONS{'samplerate'}/96000+0.5)) .
		pack("v",1);
	  }
	  if ($block_len>0)  {
	    cas_put($cas,$blocks,$block_type,$block_len,
		  $block_aux,
		  $block_data);
	    $blocks++;
	    $block_len=0;			# chunk stored - forget it
	  }
	  #last if (($status==END_OF_FILE) and ($reclen<=0));
	  last if (($status==END_OF_FILE) or ($status==NOT_GOOD));
      }
      print "SUMMARY: Data blocks: $records";
      if ($OPTIONS{'cksumsane'}) {
	print " ($errors Errors",$#err_records==-1 ? ").\n" : (" - in records: " . join(",",@err_records) . ").\n");
      } else {
	print " (N/A E - analyze the log file please).\n";
      }
      STDOUT->flush;
      print $LOGFILE "SUMMARY: Records: $records";
      if ($OPTIONS{'cksumsane'}) {
	print $LOGFILE " ($errors Errors",$#err_records==-1 ? ").\n" : (" - in records: " . join(",",@err_records) . ").\n");
      } else {
	print $LOGFILE " (N/A E - analyze the log file please).\n";
      }
      last FORMAT;
    }
    if ($OPTIONS{'turbo'} eq 'generic') {

      $LATCH[LATCHSTATE]=0;		# assume that initial state is '0' ...
      $LATCH[LATCHSLOPE]=SLOPEFALLING;	# ... and the last slope was falling

      my $saveirg=0;
      $irglen=0;
      eval("prepare_" . lc $OPTIONS{'turbo'} . "()");

      cas_put($cas,$blocks++,"pwms",2,
	      pack("CC",2,0),
	      pack("v",$OPTIONS{'samplerate'}));
      while (TRUE) {
	  print "Data blocks: $records\r";
	  STDOUT->flush;
	  $status=getlevels_pwm($cas,$blocks);
	  $reclen=$#casrecord;
	  if ($status==IRG_DETECTED) {
	    $saveirg=int(splice(@casrecord,$reclen,1)/$OPTIONS{'samplerate'}*1000+0.5);
	    $reclen--;
	  } elsif ($saveirg>0) {
	    $irglen+=$saveirg;
	    $saveirg=0;
	  }
	  if ($reclen>=0) {
	      while ($irglen>65535) {
		cas_put($cas,$blocks,'pwml',4,
			    pack("v",65535),
			    pack('vv',0,0));
		$blocks++;
		$irglen-=65535;
	      }
	      cas_put($cas,$blocks,'pwml',($reclen+1)*2,
			    pack("v",$irglen),
			    pack('v*',@casrecord));
	     $blocks++;
	     $irglen=0;
	  }
	  if ($reclen<0 and $status==END_OF_FILE and $irglen>0) {
	      while ($irglen>65535) {
		cas_put($cas,$blocks,'pwml',4,
			    pack("v",65535),
			    pack('vv',0,0));
		$blocks++;
		$irglen-=65535;
	      }
	      cas_put($cas,$blocks,'pwml',4,
			    pack("v",$irglen),
			    pack('vv',0,0));
	     $blocks++;
	     $irglen=0;
	     last;
	  }
	  last if (($status==END_OF_FILE) or ($status==NOT_GOOD));
	  $records++;
      }
      last FORMAT;
    }
    die "Unknown input format\n" if $OPTIONS{'turbo'} ne '';
  }
  close_sound_in();
  close $LOGFILE;
  $LOGFILE=undef;
}

sub read_input {

# Read input file and store it in cas hash
# Arguments:
#  - a reference to cas hash (cas)
# Returns:
#  OK

  # save all converted stuff when ^C pressed
  sub controlc {
    $SIG{'INT'}='DEFAULT';
    print "\nControl-C pressed\n";
    die
  }
  

  my $input_ext;
  my $cas=$_[0];

	cas_init($cas);
	foreach my $infile (@INFILES)  {
		my $infile=$infile;
		my %SAVEOPTIONS=%OPTIONS;
		if (grep ':',$infile) {
		    my @a=split(':',$infile);
		    $infile=shift @a;
		    foreach (0..$#a) {
			$a[$_]="--".$a[$_];
		    }
		    GetOptionsFromArray (\@a, 'help' => \$OPTIONS{'help'},
				'man' => \$OPTIONS{'man'},
				'rmfuji' => \$OPTIONS{'rmfuji'},
				'splitto=s' => \$OPTIONS{'splitto'},

				'title=s' => \$OPTIONS{'title'},
				'skipshorter=i' => \$OPTIONS{'skipshorter'},
				'turbo|t=s' => \$OPTIONS{'turbo'},
				'autoadjust' => \$OPTIONS{'autoadjust'},
				'maxgap=i' => \$OPTIONS{'maxgap'},
				'autofuji' => \$OPTIONS{'autofuji'},

				'fskdecoder=s' => \$OPTIONS{'fskdecoder'},
				'allfsk' => \$OPTIONS{'allfsk'},
				'minfsk' => \$OPTIONS{'minfsk'},
				'fskonly' => \$OPTIONS{'fskonly'},
				'mark=i' => \$OPTIONS{'mark'},
				'space=i' => \$OPTIONS{'space'},
				'resistor=f' => \$OPTIONS{'resistor'},
				'bandwidth=i' => \$OPTIONS{'bandwidth'},
				'normalize' => \$OPTIONS{'normalize'},
				'minsignals=i' => \$OPTIONS{'minsignals'},

				'noise=f' => \$OPTIONS{'noise'},
				'invert' => \$OPTIONS{'invert'},
				'lowpass' => \$OPTIONS{'lowpass'},
				'highpass!' => \$OPTIONS{'highpass'},
				'shape=s' => \$OPTIONS{'shape'},
				'recovery' => \$OPTIONS{'recovery'},

				'baud=i' => \$OPTIONS{'forcebaud'},
				'baudmultiplier=f' => \$OPTIONS{'baudmultiplier'},
				'samplerate=i' => \$OPTIONS{'samplerate'},
				'usefirstbaud' => \$OPTIONS{'usefirstbaud'},
				'maxbauddiff=i' => \$OPTIONS{'maxbauddiff'},
				'gain|g=f' => \$OPTIONS{'gain'},
				'left|l=s' => \$OPTIONS{'left'},
				'mono' => \$OPTIONS{'mono'},
				'squarify' => \$OPTIONS{'squarify'},
				'hqresample' => \$OPTIONS{'hqresample'},
				'lqresample' => \$OPTIONS{'lqresample'},
				'resamplecas' => \$OPTIONS{'resamplecas'},

				'detail|d' => \$OPTIONS{'detail'},
				'mindiff=i' => \$OPTIONS{'mindiff'},
				'amplify=f' => \$OPTIONS{'amp'},
				'cksum=s' => \$OPTIONS{'cksumtype'},
				'cksumsane!' => \$OPTIONS{'cksumsane'},

				'starttime|st=f' => \$OPTIONS{'starttime'},
				'endtime|et=f' => \$OPTIONS{'endtime'},

				'debug=s' => \$OPTIONS{'debug'},
				'ln' => \$OPTIONS{'ln'},
				'detectsilence' => \$OPTIONS{'detectsilence'},
				'pilot=i' => \$OPTIONS{'pilot'},
		    ) or pod2usage(1);

		}

		pod2usage(-exitval => 1,
			-verbose => 0,
		) if !$infile;

		($input_ext=$infile)=~s/^(.*)\.(....?)$/$2/;
		INCASE: {
			if ($input_ext=~m/cas/i) { cas_slurp($infile,$cas); last INCASE; }
			if ($input_ext=~m/hex/i) { hex_slurp($infile,$cas); last INCASE; }
			if ($input_ext=~m/(wav)|(flac)|(mp3)|(ogg)/i) { eval { $SIG{'INT'}=\&controlc; sound2cas($infile,$cas); }; last INCASE; }
			bin_slurp($infile,$cas,$OPTIONS{'pilot'})
		}
		%OPTIONS=%SAVEOPTIONS;
		undef %SAVEOPTIONS;
	}
1;
}

sub save_output {

# Build output file from cas hash
# Arguments:
#  - a reference to cas hash (cas)
# Returns:
#  OK

  my $output_ext='cas';
  my $cas=$_[0];

  my $infile=$INFILES[0];

	pod2usage(-exitval => 1,
	  -verbose => 0,
	 ) if !$infile;

  $infile=~s/((.*\.)*)\.(.*$)/$1/;

  my $outfile=($OPTIONS{'outfile'}) ? $OPTIONS{'outfile'} : $infile . '.cas';

  ($output_ext=$outfile)=~s/.*\.(....?)$/$1/;

  OUTCASE: {
    if ($output_ext=~m/cas/i) { cas_spit($outfile,$cas); last OUTCASE; }
    if ($output_ext=~m/hex/i) { hex_spit($outfile,$cas); last OUTCASE; }
    if ($output_ext=~m/(bin)|(xex)|(bas)|(com)/i) { bin_spit($outfile,$cas); last OUTCASE; }
    cas2sound($outfile,$cas);
  }
1;
}

sub conv {

  my %cas;

  read_input(\%cas);

  save_output(\%cas);

1;
}

sub cas_info {

  my %cas;

  pod2usage(-exitval => 0,
	  -verbose => 99,
	  -sections => "Info options",
	 ) if $OPTIONS{'help'};

  #$OPTIONS{'autofuji'}=TRUE;
  read_input(\%cas);
  my $programs=keys %{$cas{'programs'}};
  print "Information about $OPTIONS{'infile'}\n";
  print "------------------\n";
  print "Programs: ",$programs,"\n";
  for(my $i=1;$i<=$programs;$i++) {
    print " Program $i:\n";
    print "  Title: ",$cas{${$cas{'programs'}}{$i}}{'length'}==0 ? "<none>" : $cas{${$cas{'programs'}}{$i}}{'data'},"\n";
    print "  Starts at block ",${$cas{'programs'}}{$i},"\n";
    my $files=keys %{${${$cas{'programs'}}{$i}}{'files'}};
    print "  Files: ",$files,"\n";
    for(my $j=1;$j<=$files;$j++) {
      my $startblock=${${${$cas{'programs'}}{$i}}{'files'}}{$j};
      my $nextstartblock=${${${$cas{'programs'}}{$i}}{'files'}}{$j+1} ?
			 ${${${$cas{'programs'}}{$i}}{'files'}}{$j+1} :
			 (keys %cas)-2;
      print "   file $j starts at block ";
      print $startblock,"\n";
      if ($OPTIONS{'detail'}) {
	for (my $k=$startblock;$k<$nextstartblock;$k++) {
	  last if ${$cas{$k}}{'type'} eq 'FUJI';
	  printf "    block %3d: ${$cas{$k}}{'type'}",$k;
	  CASE: { for (${$cas{$k}}{'type'}) {
		    /data/ && do {
				    printf "; irg: %5d",unpack('v',${$cas{$k}}{'aux'});
				    printf " ms; data bytes: %4d\n",${$cas{$k}}{'length'};
				 };
		    /baud/ && do {
				    printf "; sets baud rate: %5d\n",unpack('v',${$cas{$k}}{'aux'});
				 };
		    /fsk / && do {
				    printf "; irg: %5d",unpack('v',${$cas{$k}}{'aux'});
				    printf " ms; number of FSK signals: %3d\n",${$cas{$k}}{'length'}/2;
				 };
		    /pwms/ && do {
				    print "; sets PWM parameters\n";
#				    print "; sample rate:";
#				    print "; endian: ";
#				    print "; first edge is: \n";
				 };
		    /pwml/ && do {
				    printf "; silence: %5d",unpack('v',${$cas{$k}}{'aux'});
				    printf " ms; states: %2d\n",${$cas{$k}}{'length'}/2;
				 };
		    /pwmc/ && do {
				    printf "; silence: %5d",unpack('v',${$cas{$k}}{'aux'});
				    printf " ms; impulse groups: %2d\n",${$cas{$k}}{'length'}/3;
				 };
		    /pwmd/ && do {
				    printf "; data bytes: %4d\n",${$cas{$k}}{'length'};
				 };
		  }
	        }
	}
      }
    }
  }
  print "Data records: ",$cas{'records'},"\n";
  print "All blocks: ",(keys %cas)-2,"\n";
}

sub syscheck {
  my $cmd;
  my $notok = 0;

  open $cmd, "ecasound -h|" or $notok = 1;
  close $cmd;
  print "SYSCHECK: ecasound not found, please install it first.\n" if $notok;
  $notok = 0;
  open $cmd, "ecalength -h|" or $notok = 1;
  close $cmd;
  print "SYSCHECK: ecalenght not found, please install it first.\n" if $notok;
  $notok = 0;
  open $cmd, "listplugins -h|" or $notok = 1;
  if ($notok) {
    print "SYSCHECK: \"listplugins\" command not found, can't check if required LADSPA plugins are installed.\n";
  } else {
    my @pluginlist = (<$cmd>);
    if (!grep(/1058/, @pluginlist)) {
	print "SYSCHECK: LADSPA plugin 1058/fbdelay_0,01s not found.\n";
	$notok=1
    }
    if (!grep(/\/diode/, @pluginlist)) {
	print "SYSCHECK: LADSPA plugin 1185/diode not found.\n";
	$notok=1
    }
    if (!grep(/2602/, @pluginlist)) {
	print "SYSCHECK: LADSPA plugin 2602/Noisegate not found.\n";
	$notok=1
    }
    if (!grep(/hard_gate/, @pluginlist)) {
	print "SYSCHECK: LADSPA plugin 1845/hard_gate not found.\n";
	$notok=1
    }
    if (!grep(/range_trans_cr/, @pluginlist)) {
	print "SYSCHECK: LADSPA plugin 4210/range_trans_cr not found.\n";
	$notok=1
    }
    if (!grep(/sine_faac/, @pluginlist)) {
	print "SYSCHECK: LADSPA plugin 1064/sine_faac or 1045/sine_faac not found.\n";
	$notok=1
    }
    if (!grep(/1429\//, @pluginlist)) {
	print "SYSCHECK: LADSPA plugin 1429/inv not found.\n";
	$notok=1
    }
    if (!grep(/\/bandpass_iir/, @pluginlist)) {
	print "SYSCHECK: LADSPA plugin 1892/bandpass_iir not found.\n";
	$notok=1
    }
    if (!grep(/\/butthigh_iir/, @pluginlist)) {
	print "SYSCHECK: LADSPA plugin 1904/butthigh_iir not found.\n";
	$notok=1
    }
    if (!grep(/\/difference_iama_oa/, @pluginlist)) {
	print "SYSCHECK: LADSPA plugin 2030/difference_iama_oa not found.\n";
	$notok=1
    }
    close $cmd;
  }
  if ($notok) {
    print "\nSYSCHECK: Some system checks have failed\n\n";
    return 0;
  } else {
    print "\nSYSCHECK: All system checks succeeded.\n\n";
    return 1;
  }
  1;
}

########################
######## START #########
########################

my $command="none";
my %commandlist=( 'conv' => 'conv',
		  'squarify' => 'squarify',
		  'info' => 'cas_info',
		);

%OPTIONS=( 'help' => FALSE,
	   'man' => FALSE,
	   'syscheck' => FALSE,
	   'rmfuji' => FALSE,
	   'splitto' => 'programs',	# for split - split to programs by default

	   'title' => '',		# tape title
	   'skipshorter' => 0,		# skip data blocks shorter than this
	   'turbo' => '',		# turbo format (blizzard,turbo2000,turborom)
	   'autoadjust' => FALSE,	# auto adjust baud rate ?
	   'maxgap' => 5000,		# minimum gap between files
	   'autofuji' => FALSE,		# automatic FUJI blocks ?

	   'fskdecoder' => 'new',	# which fsk decoder to  use
	   'allfsk' => FALSE,		# whether to store all raw fsk blocks
	   'minfsk' => FALSE,		# get minimum number of FSK signals or all until IRG
	   'fskonly' => FALSE,		# whether to skip decoding and store raw fsk only
	   'mark' => 5327,		# freq of mark signal
	   'space' => 3995,		# freq of space signal
	   'resistor' => 0.05,		# resistor
	   'normalize' => FALSE,	# whether sound should be normalized
	   'bandwidth' => 1000,		# bandpass filter bandwidth
	   'minsignals' => 20,		# nuber of fsk states needed to detect bit length
	   'noise' => 0,		# level of noise tu be cut off
	   'invert' => FALSE,		# invert ...

	   'forcebaud' => -1,		# whether we want specific baud rate
	   'baudmultiplier' => 1,	# whether we want multiply original baud by this
	   'usefirstbaud' => FALSE,	# use first baud value found in cas
	   'maxbauddiff' => 4,		# max baud difference before switching to new "first baud"
	   'baudmax' => 0,		# maximum baud rate
	   'baudmin' => 0,		# minimum baud rate
	   'samplerate' => 48000,	# sample rate of output file
	   'left' => '',		# name of file containing left channel
	   'mono' => FALSE,		# make mono audio
	   'squarify' => FALSE,		# FSK - produce square wave for debugging
	   'gain' => 0.7,		# amplitude of output sound
	   'lowpass' => FALSE,		# use lowpass filter for turbo ?
	   'highpass' => FALSE,		# use highpass filter for turbo ?
	   'shape' => 'square',		# the wave shape of audio output (turbo)
	   'hqresample' => FALSE,	# whether to use high quality resampling (slow)
	   'lqresample' => FALSE,	# whether to use low quality resampling (fast), for good quality square wave
	   'resamplecas' =>  FALSE,	# do we want to "resample" cas/hex ?
	   'recovery' => FALSE,		# recovery mode (don't wait for pilot)

	   'detail' => FALSE,		# whether info should be detailed
	   'infile' => '',
	   'outfile' => '',

	   'mindiff' => 0,		# turbo only, 0=automatic
	   'amp' => 1,			# signal amplification
	   'cksumtype' => '',		# type of checksum - xor, modulo256, sum255carry
	   'cksumsane' => TRUE,		# whether checksum is well defined

	   'starttime' => 0,		# position in audio file from which start processing
	   'endtime' => 0,		# position in audio file where to stop processing

	   'debug' => '',		# debugging
	   'ln' => FALSE,		# use logarithmic scaling
	   'detectsilence' => FALSE,	# silence detection for FSK encoding
	   'pilot' => 20000,		# length of piloting signal in fsk format (bin->cas/hex)
	 );

$OPTIONS{'cmdline'} = join(" ",@ARGV);

GetOptions ( 'help' => \$OPTIONS{'help'},
	    'man' => \$OPTIONS{'man'},
	    'syscheck' => \$OPTIONS{'syscheck'},
	    'rmfuji' => \$OPTIONS{'rmfuji'},
	    'splitto=s' => \$OPTIONS{'splitto'},

	    'title=s' => \$OPTIONS{'title'},
	    'skipshorter=i' => \$OPTIONS{'skipshorter'},
	    'turbo|t=s' => \$OPTIONS{'turbo'},
	    'autoadjust' => \$OPTIONS{'autoadjust'},
	    'maxgap=i' => \$OPTIONS{'maxgap'},
	    'autofuji' => \$OPTIONS{'autofuji'},

	    'fskdecoder=s' => \$OPTIONS{'fskdecoder'},
	    'allfsk' => \$OPTIONS{'allfsk'},
	    'minfsk' => \$OPTIONS{'minfsk'},
	    'fskonly' => \$OPTIONS{'fskonly'},
	    'mark=i' => \$OPTIONS{'mark'},
	    'space=i' => \$OPTIONS{'space'},
	    'resistor=f' => \$OPTIONS{'resistor'},
	    'bandwidth=i' => \$OPTIONS{'bandwidth'},
	    'normalize' => \$OPTIONS{'normalize'},
	    'minsignals=i' => \$OPTIONS{'minsignals'},

	    'noise=f' => \$OPTIONS{'noise'},
	    'invert' => \$OPTIONS{'invert'},
	    'lowpass' => \$OPTIONS{'lowpass'},
	    'highpass!' => \$OPTIONS{'highpass'},
	    'shape=s' => \$OPTIONS{'shape'},
	    'recovery' => \$OPTIONS{'recovery'},

	    'baud=i' => \$OPTIONS{'forcebaud'},
	    'baudmultiplier=f' => \$OPTIONS{'baudmultiplier'},
	    'samplerate=i' => \$OPTIONS{'samplerate'},
	    'usefirstbaud' => \$OPTIONS{'usefirstbaud'},
	    'maxbauddiff=i' => \$OPTIONS{'maxbauddiff'},
	    'baudmax=i' => \$OPTIONS{'baudmax'},
	    'baudmin=i' => \$OPTIONS{'baudmin'},
	    'gain|g=f' => \$OPTIONS{'gain'},
	    'left|l=s' => \$OPTIONS{'left'},
	    'mono' => \$OPTIONS{'mono'},
	    'squarify' => \$OPTIONS{'squarify'},
	    'hqresample' => \$OPTIONS{'hqresample'},
	    'lqresample' => \$OPTIONS{'lqresample'},
	    'resamplecas' => \$OPTIONS{'resamplecas'},

	    'detail|d' => \$OPTIONS{'detail'},
	    'mindiff=i' => \$OPTIONS{'mindiff'},
	    'amplify=f' => \$OPTIONS{'amp'},
	    'cksum=s' => \$OPTIONS{'cksumtype'},
	    'cksumsane!' => \$OPTIONS{'cksumsane'},

	    'starttime|st=f' => \$OPTIONS{'starttime'},
	    'endtime|et=f' => \$OPTIONS{'endtime'},

	    'debug=s' => \$OPTIONS{'debug'},
	    'ln' => \$OPTIONS{'ln'},
	    'detectsilence' => \$OPTIONS{'detectsilence'},
	    'pilot=i' => \$OPTIONS{'pilot'},
	  ) or pod2usage(1);

pod2usage(-exitval => 0, -verbose => 2) if $OPTIONS{'man'};
pod2usage(-exitval => 0, -verbose => 1) if $OPTIONS{'help'};

if ($OPTIONS{'syscheck'}) {
  syscheck;
}

pod2usage(-exitval => 1, -verbose => 0) if !@ARGV;

if ($OPTIONS{'endtime'} and $OPTIONS{'starttime'}>$OPTIONS{'endtime'}) {
  print "\nEnd time must be greater than start time!\n";
  exit 1;

}

if ($OPTIONS{'turbo'} and $OPTIONS{'turbo'}!~m/$TURBO_FORMATS_REGEX/io and ($OPTIONS{'turbo'} ne 'generic')) {
  print "\nUnknown turbo format specified.\n";
  exit 1;

}

$DATASTORE{'mindiff'}=($OPTIONS{'mindiff'} or 6000);

$command=$commandlist{splice(@ARGV,0,1)};

pod2usage(-exitval => 1, -verbose => 0) if !@ARGV;

#@INFILES=@ARGV[0..$#ARGV-1];
#$OPTIONS{'infile'}=$ARGV[0];

@INFILES=();
foreach my $inputstr (@ARGV[0..$#ARGV-1]) {
	my $inputfile;
	if (grep ':',$inputstr) {
	  my @a=split(':',$inputstr);
	  $inputfile=shift @a;
	} else {
	  $inputfile=$inputstr;
	}
	if (!-e $inputfile) {
		print "\nInput file not found! Please check the given arguments.\n";
		exit NOT_GOOD;
	} else {
		push (@INFILES,$inputstr);
	}
}

$OPTIONS{'outfile'}=$ARGV[$#ARGV];
push(@INFILES,$ARGV[$#ARGV]) if $command eq 'cas_info';

if ($command eq 'squarify') {
  $command='conv';
  $OPTIONS{'rawsquare'}=TRUE;
  if (!$OPTIONS{'outfile'}) {
    $OPTIONS{'outfile'}=$OPTIONS{'infile'} . "-squared.wav";
    $OPTIONS{'outfile'}=findofile($OPTIONS{'outfile'});
  }
}

if ($OPTIONS{'debug'}) {
    foreach (split(',',$OPTIONS{'debug'})) {
    $DEBUG{$_}=TRUE;
    }
}

if ($command) {
  my $result=eval($command);
  print $@ if (!$result);
} else {

  pod2usage(-exitval => 0, -verbose => 1) if $OPTIONS{'help'};
  pod2usage(-exitval => 0, -verbose => 2) if $OPTIONS{'man'};
  pod2usage(-exitval => 1, -verbose => 0) if @ARGV;
}

1;	#exit nicely

__END__

=head1 NAME

=over 1

a8cas-util.pl - the utility for manipulating 8-bit Atari tape images.

=back

=head1 DESCRIPTION

=over 1

B<a8cas-util.pl> can do conversion from/to audio/cas/hex/bin. More conversion capabilities and various operations on
Atari 8-bit tape images in extended CAS format (A8CAS) will be addedd in the (near) future.

=back

=head1 LICENSE

=over 1

Copyright (C) 2009-2013 by FUJI

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program. If not, see <http://www.gnu.org/licenses/>.

Nick 'FUJI' was registered at atari.area (http://atariarea.krap.pl/) in 2009.
You can contact FUJI through the forums or by writting to fuji at arus dot net dot pl.

=back

=head1 Requirements

=over 1

If You want to use a8cas-util to convert from/to audio files, the following software must be installed in Your system:
    Ecasound (http://www.eca.cx/ecasound/)
    (LADSPA) SWH plugins (http://plugin.org.uk/)
    (LADSPA) Blop plugins (http://blop.sourceforge.net/index.html)
    (LADSPA) CMT plugins (http://www.ladspa.org/cmt/)
    (LADSPA) Omins plugins (http://www.nongnu.org/om-synth/omins.html#plugins)

Additionally Ecasound uses mpg321 to decode mp3 files

Otherwise plain perl 5.8 or newer should be sufficient.

=back

=head1 SYNOPSIS

=over 1

a8cas-util.pl [global options] E<lt>commandE<gt> [command options] E<lt>input_fileE<gt>[:option[=value][:option[=value] ...]]
[input_file[:option[=value][:option[=value] ...]] ...] E<lt>output_fileE<gt>

=over 1

Global options:

--help | -h - brief help

--man - full documentation

--syscheck - check for required system components

=back

=back

=head1 OPTIONS

=over 1

=item B<--help | -h>

Prints a brief help message and exits.

=item B<--man>

Displays the manual page and exits.

=item B<--syscheck>

Checks whether required helper programs and LADSPA plugins are installed before any operation is started.

=item B<<commandE<gt>>

Required. Specifies the desired action.
Valid commands (for now) are: conv, squarify, info.

=over 2

B<Commands:>

=over 4

=item B<conv>

Converts data provided in <input_file> to another format.

=item B<squarify>

This command is for standard tapes only. It allows converting FSK signal to square wave,
which shows zeros and ones as low and high states instead of mark/space frequencies.
It is usefull for debugging and recovery purposes.
Also, You can compare audio file produced by "squarify" command with the one created
from cas/hex with the use of "--squarify" option. Such comparision allows precise adjustments
(in HEX file) of IRG and baud rates, which in turn helps to match all timings between CAS file
and left audio track from original tape.

=item B<info>

Prints informations about cas/hex file.

=back

=back

=back

=over 1

=item B<input_file>

Name of input file to process. This is required. Input format is determined
by file extension: anything other than cas,hex,com,xex,bas and bin is treated as an audio file.
When multiple input files are specified, they all are "merged" together and stored in
one output file.
Each input file can have it's own set of processing options (without leading dashes). All options specified below
can be used. Options have to be delimited by ":" (colon). If an option takes a value, the "="
must be placed between the option name and it's value. Example:

a8cas-util.pl conv a.wav:resistor=0.01 b.wav:turbo=turbo2000 output.cas

=item B<output_file>

Name of output file. Required. The default output format is cas.
Otherwise output format is determined by given file extension (cas,hex,bin,wav).
The script tries to NEVER overwrite existing files by adding numeric suffixes
to file's base name if <output_file> already exists.

=back

=head1 conv options

=over 1

=item B<--title | -t <TitleE<gt>>

Provides the main name (description) of tape image.

=item B<--skipshorter <bytesE<gt>>

Use it if You want to skip obviously wrong, short data records,
don't store them in .cas file. Standard tapes only.

=item B<--maxgap <milisecondsE<gt>>

Specifies the minimum length of IRG (silence) which separates tape "files"
from each other. This option is relevant when converting to binary format.
The default is 5000 (5 seconds).

=item B<--autoadjust>

Automatically adjust speed changes (when tape is old and damaged).
For tapes in turbo format, but only for recovery purposes
(can cause unpredictable, unwanted results, will be very rarely needed).

=item B<--mark <frequencyE<gt>>

The frequency of "mark" signal in Hz. 5327 by default. For FSK encoded tapes.

=item B<--space <frequencyE<gt>>

The frequency of "space" signal in Hz. 3995 by default.  For FSK encoded tapes.

=item B<--bandwidth <frequencyE<gt>>

Bandwidth of the bandpass filter. Specifying value lower than default (1000)
can help sometimes. For FSK encoded tapes.

=item B<--resistor <valueE<gt>>

This is the most usefull command line option for standard format (FSK encoded) tapes.
It corresponds to resistor connected to output of comparator in real
hardware, causing the output value to shift towards logical "1".
The default value (0.05) will work most of the time for good quality tape
dumps. You may need to adjust this, if a8cas-util.pl reports checksum errors.
Values may range from 0.001 to 0.2, sometimes small negative values may be needed for recovery purposes.

=item B<--allfsk>

For standard (FSK) format only. All FSK signals, which don't belong to data blocks,
will be stored in CAS/HEX file. Usually You don't want it, use only for recovery purposes.
By default only significant FSK signals (like long space signals, which are elements of
tape protection) are stored in output file.

=item B<--minfsk>

For standard (FSK) format only. For recovery purposes mostly, sometimes helps with certain types of protected
tapes. Tries to convert to bytes as many signals as possible instead of saving them in 'fsk ' chunks.

=item B<--minsignals>

For standard (FSK) format only. Minimal number of signals used to detect a width of a bit. Should always be >=4.
The default value is 20, Should be changed only for very unusual recordings (protections) or for recovery purposes.

=item B<--maxbauddiff <differenceE<gt>>

For standard (FSK) format only. Maximum difference of baud rate between start of data record and any part of it.
If bigger change of baud rate is detected, a new data block is started. 

=item B<--squarify >

Standard (FSK encoded) tapes only. It can be used to produce audio files,
in which "mark" and "space" tones are represented by low and high levels of signal instead of
different frequencies; this is mostly for debugging purposes, see also description of command "squarify".

=item B<--turbo|-t <formatE<gt>>

Required for turbo formats. Specifies a kind of turbo format of sound file
(input or output).
Valid formats are: blizzard, turbo2000, ast, att, um, turborom, mkturbo, cs2000, csst, cstt, csbtape, autoturbo, tapewiz, lowsil2000, generic.

=item B<--noise <levelE<gt>>

Optional, turbo only. Specifies noise cut-off level.
Default is 0. Removes unwanted noise. Can remove weak signal too,
so use with care.

=item B<--amplify <levelE<gt>>

Optional, turbo only. How mutch to amplify the sound before processing.
This is quite important option, can greatly improve the results, but
also can break things. The default is 1 (no amplification).

=item B<--mindiff <differenceE<gt>>

Optional. For turbo formats only. Specifies minimum difference in signal level
which triggers the edge detection. Values may vary from 1 to 65535 (16-bit samples are used for turbo formats).
When this option is used, dynamic adjustments to signal amplitude are turned off.
Use it when automatic procedures fail.

=item B<--invert>

Optional, turbo only. Allows to choose which signal edge (raising or falling)
starts the impulse. The default is to use the hard coded setting, depending on
the tape format.

=item B<--lowpass>

Optional, turbo only. Resonant lowpass filter can be used to selectively
amplify amplitude of weak impulses without clipping of the rest of the
signal. Turned off by default. Experimental.

=item B<--highpass>

Optional, turbo only. Highpass filter is used to filter out
low frequency components of signal, which means centering it towards
level zero. Turned off by default. The filter sometimes may reverse the phase
of the signal, use --invert to compensate.

=item B<--lqresample>

Optional, turbo only. Makes ecasound to use low quality resampling algorithm. Sometimes it gives better results than default one.

=item B<--hqresample>

Optional, turbo only. Makes ecasound to use highest quality resampling algorithm. Sometimes it gives better results than default one.

=item B<--shape <square|sine|sawtoothE<gt>>

Optional, turbo only. Allows to choose the shape of the wave during
conversion cas/hex -> audio. The default is square.

=item B<--left|-l <filenameE<gt>>

Only for cas/hex/audio -> audio conversion. Allows to mix data track and audio track into one
stereo file. <filename> is a name of file containing left audio track.

=item B<--mono>

Only for cas/hex/audio -> audio conversion. Make monophonic audio file.

=item B<--cksum <checksum typeE<gt>>

Allows to change default checksum type for non-standard recordings (for example
protections in Turbo 2000). Valid types are: sum255carry, modulo256 and xor.

=item B<--autofuji>

Automatically adds FUJI blocks between parts of tape. Part detection is currently based
only on length of gap between subsequent data blocks (determined by --maxgap).

=item B<--starttime|--st <secondsE<gt>>

Sets the position (in seconds) in audio file, where the processing should start.
Usefull for processing only parts of audio files (recovery, tape images containing different formats).

=item B<--endtime|--et <secondsE<gt>>

Sets the position (in seconds) in audio file, where the processing should stop.
Usefull for processing only parts of audio files (recovery, tape images containing different formats).

=item B<--pilot <milisecondsE<gt>>

Sets the length (in miliseconds) of pilot signal. Applicable only to conversion from binary files (bin/xex/com/bas)
to cas/hex/wav and only to standard (fsk) format. If not defined, the pilot length is set to 20 seconds.

=back

=head1 info options

=over 1

=item B<--detail |-d>

Optional.
Prints detailed report about all blocks found in cas.

=back

=head1 squarify options

=over 1

All options valid for "conv" command, applicable to audio->cas/hex conversion can be used, though
some of them will be ignored. Use of the following options makes sense:
--resistor, --mark, --space, --bandwidth.

=back

=cut
